@@ -38,8 +38,8 @@ class WXDLLIMPEXP_CORE wxPoint2DInt
 public :
     inline wxPoint2DInt();
     inline wxPoint2DInt( wxInt32 x , wxInt32 y );
-    inline wxPoint2DInt( const wxPoint2DInt &pt );
     inline wxPoint2DInt( const wxPoint &pt );
+    // default copy ctor and copy-assign operator are OK
 
     // noops for this class, just return the coords
     inline void GetFloor( wxInt32 *x , wxInt32 *y ) const;
@@ -58,9 +58,8 @@ public :
     inline wxInt32 GetCrossProduct( const wxPoint2DInt &vec ) const;
 
     // the reflection of this point
-    inline wxPoint2DInt operator-();
+    wxPoint2DInt operator-() const;
 
-    inline wxPoint2DInt& operator=(const wxPoint2DInt& pt);
     inline wxPoint2DInt& operator+=(const wxPoint2DInt& pt);
     inline wxPoint2DInt& operator-=(const wxPoint2DInt& pt);
     inline wxPoint2DInt& operator*=(const wxPoint2DInt& pt);
@@ -105,12 +104,6 @@ inline wxPoint2DInt::wxPoint2DInt( wxInt32 x , wxInt32 y )
     m_y = y;
 }
 
-inline wxPoint2DInt::wxPoint2DInt( const wxPoint2DInt &pt )
-{
-    m_x = pt.m_x;
-    m_y = pt.m_y;
-}
-
 inline wxPoint2DInt::wxPoint2DInt( const wxPoint &pt )
 {
     m_x = pt.x;
@@ -133,7 +126,7 @@ inline void wxPoint2DInt::GetRounded( wxInt32 *x , wxInt32 *y ) const
 inline wxDouble wxPoint2DInt::GetVectorLength() const
 {
     // cast needed MIPSpro compiler under SGI
-    return sqrt( (double)(m_x)*(m_x) + (m_y)*(m_y) );
+    return sqrt( (wxDouble)(m_x)*(m_x) + (wxDouble)(m_y)*(m_y) );
 }
 
 inline void wxPoint2DInt::SetVectorLength( wxDouble length )
@@ -155,7 +148,8 @@ inline wxDouble wxPoint2DInt::GetDistance( const wxPoint2DInt &pt ) const
 
 inline wxDouble wxPoint2DInt::GetDistanceSquare( const wxPoint2DInt &pt ) const
 {
-    return ( (pt.m_x-m_x)*(pt.m_x-m_x) + (pt.m_y-m_y)*(pt.m_y-m_y) );
+    return ( ((wxDouble)pt.m_x-m_x)*((wxDouble)pt.m_x-m_x) +
+             ((wxDouble)pt.m_y-m_y)*((wxDouble)pt.m_y-m_y) );
 }
 
 inline wxInt32 wxPoint2DInt::GetDotProduct( const wxPoint2DInt &vec ) const
@@ -173,21 +167,11 @@ inline wxPoint2DInt::operator wxPoint() const
     return wxPoint( m_x, m_y);
 }
 
-inline wxPoint2DInt wxPoint2DInt::operator-()
+inline wxPoint2DInt wxPoint2DInt::operator-() const
 {
     return wxPoint2DInt( -m_x, -m_y);
 }
 
-inline wxPoint2DInt& wxPoint2DInt::operator=(const wxPoint2DInt& pt)
-{
-    if (this != &pt)
-    {
-        m_x = pt.m_x;
-        m_y = pt.m_y;
-    }
-    return *this;
-}
-
 inline wxPoint2DInt& wxPoint2DInt::operator+=(const wxPoint2DInt& pt)
 {
     m_x = m_x + pt.m_x;
@@ -287,11 +271,11 @@ class WXDLLIMPEXP_CORE wxPoint2DDouble
 public :
     inline wxPoint2DDouble();
     inline wxPoint2DDouble( wxDouble x , wxDouble y );
-    inline wxPoint2DDouble( const wxPoint2DDouble &pt );
     wxPoint2DDouble( const wxPoint2DInt &pt )
         { m_x = (wxDouble) pt.m_x ; m_y = (wxDouble) pt.m_y ; }
     wxPoint2DDouble( const wxPoint &pt )
         { m_x = (wxDouble) pt.x ; m_y = (wxDouble) pt.y ; }
+    // default copy ctor and copy-assign operator are OK
 
     // two different conversions to integers, floor and rounding
     inline void GetFloor( wxInt32 *x , wxInt32 *y ) const;
@@ -310,9 +294,8 @@ public :
     inline wxDouble GetCrossProduct( const wxPoint2DDouble &vec ) const;
 
     // the reflection of this point
-    inline wxPoint2DDouble operator-();
+    wxPoint2DDouble operator-() const;
 
-    inline wxPoint2DDouble& operator=(const wxPoint2DDouble& pt);
     inline wxPoint2DDouble& operator+=(const wxPoint2DDouble& pt);
     inline wxPoint2DDouble& operator-=(const wxPoint2DDouble& pt);
     inline wxPoint2DDouble& operator*=(const wxPoint2DDouble& pt);
@@ -352,12 +335,6 @@ inline wxPoint2DDouble::wxPoint2DDouble( wxDouble x , wxDouble y )
     m_y = y;
 }
 
-inline wxPoint2DDouble::wxPoint2DDouble( const wxPoint2DDouble &pt )
-{
-    m_x = pt.m_x;
-    m_y = pt.m_y;
-}
-
 inline void wxPoint2DDouble::GetFloor( wxInt32 *x , wxInt32 *y ) const
 {
     *x = (wxInt32) floor( m_x );
@@ -407,21 +384,11 @@ inline wxDouble wxPoint2DDouble::GetCrossProduct( const wxPoint2DDouble &vec ) c
     return ( m_x * vec.m_y - vec.m_x * m_y );
 }
 
-inline wxPoint2DDouble wxPoint2DDouble::operator-()
+inline wxPoint2DDouble wxPoint2DDouble::operator-() const
 {
     return wxPoint2DDouble( -m_x, -m_y);
 }
 
-inline wxPoint2DDouble& wxPoint2DDouble::operator=(const wxPoint2DDouble& pt)
-{
-    if (this != &pt)
-    {
-        m_x = pt.m_x;
-        m_y = pt.m_y;
-    }
-    return *this;
-}
-
 inline wxPoint2DDouble& wxPoint2DDouble::operator+=(const wxPoint2DDouble& pt)
 {
     m_x = m_x + pt.m_x;
@@ -606,7 +573,7 @@ public:
 
     void ConstrainTo( const wxRect2DDouble &rect );
 
-    inline wxPoint2DDouble Interpolate( wxInt32 widthfactor , wxInt32 heightfactor )
+    wxPoint2DDouble Interpolate( wxInt32 widthfactor, wxInt32 heightfactor ) const
         { return wxPoint2DDouble( m_x + m_width * widthfactor , m_y + m_height * heightfactor ); }
 
     static void Intersect( const wxRect2DDouble &src1 , const wxRect2DDouble &src2 , wxRect2DDouble *dest );
@@ -629,7 +596,6 @@ public:
         { m_x *= ((wxDouble)num)/((wxDouble)denum); m_y *= ((wxDouble)num)/((wxDouble)denum);
                 m_width *= ((wxDouble)num)/((wxDouble)denum); m_height *= ((wxDouble)num)/((wxDouble)denum);}
 
-    wxRect2DDouble& operator = (const wxRect2DDouble& rect);
     inline bool operator == (const wxRect2DDouble& rect) const
         { return wxIsSameDouble(m_x, rect.m_x) && wxIsSameDouble(m_y, rect.m_y) && HaveEqualSize(rect); }
     inline bool operator != (const wxRect2DDouble& rect) const
@@ -654,7 +620,7 @@ public:
        wxRect2DInt(wxInt32 x, wxInt32 y, wxInt32 w, wxInt32 h) { m_x = x; m_y = y; m_width = w;  m_height = h; }
        wxRect2DInt(const wxPoint2DInt& topLeft, const wxPoint2DInt& bottomRight);
        inline wxRect2DInt(const wxPoint2DInt& pos, const wxSize& size);
-       inline wxRect2DInt(const wxRect2DInt& rect);
+    // default copy ctor and copy-assign operator are OK
 
         // single attribute accessors
 
@@ -714,7 +680,8 @@ public:
             { m_x += left; m_y += top; m_width -= left + right; m_height -= top + bottom;}
         inline void Offset( const wxPoint2DInt &pt ) { m_x += pt.m_x; m_y += pt.m_y; }
         void ConstrainTo( const wxRect2DInt &rect );
-        inline wxPoint2DInt Interpolate( wxInt32 widthfactor , wxInt32 heightfactor ) { return wxPoint2DInt( m_x + m_width * widthfactor , m_y + m_height * heightfactor ); }
+        wxPoint2DInt Interpolate( wxInt32 widthfactor, wxInt32 heightfactor ) const
+            { return wxPoint2DInt( m_x + m_width * widthfactor, m_y + m_height * heightfactor ); }
 
         static void Intersect( const wxRect2DInt &src1 , const wxRect2DInt &src2 , wxRect2DInt *dest );
         inline void Intersect( const wxRect2DInt &otherRect ) { Intersect( *this , otherRect , this ); }
@@ -731,7 +698,6 @@ public:
             { m_x *= ((wxInt32)num)/((wxInt32)denum); m_y *= ((wxInt32)num)/((wxInt32)denum);
                 m_width *= ((wxInt32)num)/((wxInt32)denum); m_height *= ((wxInt32)num)/((wxInt32)denum);}
 
-       wxRect2DInt& operator = (const wxRect2DInt& rect);
        bool operator == (const wxRect2DInt& rect) const;
        bool operator != (const wxRect2DInt& rect) const;
 
@@ -746,14 +712,6 @@ public:
        wxInt32 m_height;
 };
 
-inline wxRect2DInt::wxRect2DInt( const wxRect2DInt &r )
-{
-    m_x = r.m_x;
-    m_y = r.m_y;
-    m_width = r.m_width;
-    m_height = r.m_height;
-}
-
 inline wxRect2DInt::wxRect2DInt( const wxPoint2DInt &a , const wxPoint2DInt &b)
 {
     m_x = wxMin( a.m_x , b.m_x );
