@@ -23,7 +23,7 @@ public:
         const wxPoint& pos = wxDefaultPosition,
         const wxSize& size = wxDefaultSize,
         long style = 0,
-        const wxString& name = wxStaticTextNameStr)
+        const wxString& name = wxASCII_STR(wxStaticTextNameStr))
     {
         Create(parent, id, label, pos, size, style, name);
     }
@@ -33,7 +33,7 @@ public:
         const wxPoint& pos = wxDefaultPosition,
         const wxSize& size = wxDefaultSize,
         long style = 0,
-        const wxString& name = wxStaticTextNameStr);
+        const wxString& name = wxASCII_STR(wxStaticTextNameStr));
 
     // implementation
     // --------------
@@ -51,8 +51,8 @@ public:
     virtual WXWidget GetLabelWidget() const
         { return m_labelWidget; }
 
-    virtual void DoSetLabel(const wxString& str);
-    virtual wxString DoGetLabel() const;
+    virtual void WXSetVisibleLabel(const wxString& str);
+    virtual wxString WXGetVisibleLabel() const;
 
     virtual wxSize DoGetBestSize() const;
 protected:
