@@ -29,7 +29,7 @@ public:
                    const wxPoint& pos = wxDefaultPosition,
                    const wxSize& size = wxDefaultSize,
                    long style = 0,
-                   const wxString& name = wxStaticBitmapNameStr)
+                   const wxString& name = wxASCII_STR(wxStaticBitmapNameStr))
     {
         Init();
 
@@ -42,22 +42,22 @@ public:
                 const wxPoint& pos = wxDefaultPosition,
                 const wxSize& size = wxDefaultSize,
                 long style = 0,
-                const wxString& name = wxStaticBitmapNameStr);
+                const wxString& name = wxASCII_STR(wxStaticBitmapNameStr));
 
     virtual ~wxStaticBitmap() { Free(); }
 
-    virtual void SetIcon(const wxIcon& icon) { SetImage(&icon); }
-    virtual void SetBitmap(const wxBitmap& bitmap) { SetImage(&bitmap); }
-    virtual wxBitmap GetBitmap() const;
-    virtual wxIcon GetIcon() const;
+    virtual void SetIcon(const wxIcon& icon) wxOVERRIDE { SetImage(&icon); }
+    virtual void SetBitmap(const wxBitmap& bitmap) wxOVERRIDE { SetImage(&bitmap); }
+    virtual wxBitmap GetBitmap() const wxOVERRIDE;
+    virtual wxIcon GetIcon() const wxOVERRIDE;
 
-    virtual WXDWORD MSWGetStyle(long style, WXDWORD *exstyle) const;
+    virtual WXDWORD MSWGetStyle(long style, WXDWORD *exstyle) const wxOVERRIDE;
 
     // returns true if the platform should explicitly apply a theme border
-    virtual bool CanApplyThemeBorder() const { return false; }
+    virtual bool CanApplyThemeBorder() const wxOVERRIDE { return false; }
 
 protected:
-    virtual wxSize DoGetBestClientSize() const;
+    virtual wxSize DoGetBestClientSize() const wxOVERRIDE;
 
     // ctor/dtor helpers
     void Init();
