@@ -27,6 +27,7 @@
 #include "wx/intl.h"
 
 #include "wx/validate.h"        // for wxDefaultValidator (always include it)
+#include "wx/windowid.h"
 
 #if wxUSE_PALETTE
     #include "wx/palette.h"
@@ -58,6 +59,7 @@ class WXDLLIMPEXP_FWD_CORE wxDC;
 class WXDLLIMPEXP_FWD_CORE wxDropTarget;
 class WXDLLIMPEXP_FWD_CORE wxLayoutConstraints;
 class WXDLLIMPEXP_FWD_CORE wxSizer;
+class WXDLLIMPEXP_FWD_CORE wxTextEntry;
 class WXDLLIMPEXP_FWD_CORE wxToolTip;
 class WXDLLIMPEXP_FWD_CORE wxWindowBase;
 class WXDLLIMPEXP_FWD_CORE wxWindow;
@@ -137,6 +139,13 @@ enum
     wxSEND_EVENT_POST = 1
 };
 
+// Flags for WXSetInitialFittingClientSize().
+enum
+{
+    wxSIZE_SET_CURRENT = 0x0001, // Set the current size.
+    wxSIZE_SET_MIN     = 0x0002  // Set the size as the minimum allowed size.
+};
+
 // ----------------------------------------------------------------------------
 // (pseudo)template list classes
 // ----------------------------------------------------------------------------
@@ -527,10 +536,16 @@ public:
         return wxSize( wxMax( client.x, best.x ), wxMax( client.y, best.y ) );
     }
 
-    // returns the magnification of the content of this window
-    // e.g. 2.0 for a window on a retina screen
+    // Return the magnification of the content of this window for the platforms
+    // using logical pixels different from physical ones, i.e. those for which
+    // wxHAVE_DPI_INDEPENDENT_PIXELS is defined. For the other ones, always
+    // returns 1, regardless of DPI scale factor returned by the function below.
     virtual double GetContentScaleFactor() const;
 
+    // Return the ratio of the DPI used by this window to the standard DPI,
+    // e.g. 1 for standard DPI screens and 2 for "200% scaling".
+    virtual double GetDPIScaleFactor() const;
+
     // return the size of the left/right and top/bottom borders in x and y
     // components of the result respectively
     virtual wxSize GetWindowBorderSize() const;
@@ -647,7 +662,7 @@ public:
         // state, i.e. the state in which the window would be if all its
         // parents were enabled (use IsEnabled() above to get the effective
         // window state)
-    bool IsThisEnabled() const { return m_isEnabled; }
+    virtual bool IsThisEnabled() const { return m_isEnabled; }
 
     // returns true if the window is visible, i.e. IsShown() returns true
     // if called on it and all its parents up to the first TLW
@@ -726,7 +741,11 @@ public:
         // can this window be given focus by keyboard navigation? if not, the
         // only way to give it focus (provided it accepts it at all) is to
         // click it
-    virtual bool AcceptsFocusFromKeyboard() const { return AcceptsFocus(); }
+    virtual bool AcceptsFocusFromKeyboard() const
+        { return !m_disableFocusFromKbd && AcceptsFocus(); }
+
+        // Disable any input focus from the keyboard
+    void DisableFocusFromKeyboard() { m_disableFocusFromKbd = true; }
 
 
         // Can this window be focused right now, in its current state? This
@@ -755,6 +774,9 @@ public:
         // call this when the return value of AcceptsFocus() changes
     virtual void SetCanFocus(bool WXUNUSED(canFocus)) { }
 
+        // call to customize visible focus indicator if possible in the port
+    virtual void EnableVisibleFocus(bool WXUNUSED(enabled)) { }
+
         // navigates inside this window
     bool NavigateIn(int flags = wxNavigationKeyEvent::IsForward)
         { return DoNavigateIn(flags); }
@@ -949,14 +971,23 @@ public:
     // translation between different units
     // -----------------------------------
 
+        // Get the DPI used by the given window or wxSize(0, 0) if unknown.
+    virtual wxSize GetDPI() const;
+
+    // Some ports need to modify the font object when the DPI of the window it
+    // is used with changes, this function can be used to do it.
+    //
+    // Currently it is only used in wxMSW and is not considered to be part of
+    // wxWidgets public API.
+    virtual void WXAdjustFontToOwnPPI(wxFont& WXUNUSED(font)) const { }
+
         // DPI-independent pixels, or DIPs, are pixel values for the standard
         // 96 DPI display, they are scaled to take the current resolution into
         // account (i.e. multiplied by the same factor as returned by
-        // GetContentScaleFactor()) if necessary for the current platform.
+        // GetDPIScaleFactor()) if necessary for the current platform.
         //
-        // Currently the conversion factor is the same for all windows but this
-        // will change with the monitor-specific resolution support in the
-        // future, so prefer using the non-static member functions.
+        // To support monitor-specific resolutions, prefer using the non-static
+        // member functions or use a valid (non-null) window pointer.
         //
         // Similarly, currently in practice the factor is the same in both
         // horizontal and vertical directions, but this could, in principle,
@@ -1019,7 +1050,7 @@ public:
 
         // start or end mouse capture, these functions maintain the stack of
         // windows having captured the mouse and after calling ReleaseMouse()
-        // the mouse is not released but returns to the window which had had
+        // the mouse is not released but returns to the window which had
         // captured it previously (if any)
     void CaptureMouse();
     void ReleaseMouse();
@@ -1029,7 +1060,7 @@ public:
 
         // does this window have the capture?
     virtual bool HasCapture() const
-        { return (wxWindow *)this == GetCapture(); }
+        { return reinterpret_cast<const wxWindow*>(this) == GetCapture(); }
 
         // enable the specified touch events for this window, return false if
         // the requested events are not supported
@@ -1511,6 +1542,19 @@ public:
     // Returns true if more idle time is requested.
     virtual bool SendIdleEvents(wxIdleEvent& event);
 
+    // Send wxContextMenuEvent and return true if it was processed.
+    //
+    // Note that the event may end up being sent to a different window, if this
+    // window is part of a composite control.
+    bool WXSendContextMenuEvent(const wxPoint& posInScreenCoords);
+
+    // This internal function needs to be called to set the fitting client size
+    // (i.e. the minimum size determined by the window sizer) when the size
+    // that we really need to use is not known until the window is actually
+    // shown, as is the case for TLWs with recent GTK versions, as it will
+    // update the size again when it does become known, if necessary.
+    virtual void WXSetInitialFittingClientSize(int flags);
+
         // get the handle of the window for the underlying window system: this
         // is only used for wxWin itself or for user code which wants to call
         // platform-specific APIs
@@ -1580,6 +1624,8 @@ public:
         return false;
     }
 
+    // This is an internal helper function implemented by text-like controls.
+    virtual const wxTextEntry* WXGetTextEntry() const { return NULL; }
 
 protected:
     // helper for the derived class Create() methods: the first overload, with
@@ -1591,7 +1637,7 @@ protected:
                     const wxSize& size = wxDefaultSize,
                     long style = 0,
                     const wxValidator& validator = wxDefaultValidator,
-                    const wxString& name = wxPanelNameStr);
+                    const wxString& name = wxASCII_STR(wxPanelNameStr));
 
     bool CreateBase(wxWindowBase *parent,
                     wxWindowID winid,
@@ -1721,6 +1767,9 @@ protected:
     bool                 m_inheritFgCol:1;
     bool                 m_inheritFont:1;
 
+    // flag disabling accepting focus from keyboard
+    bool                 m_disableFocusFromKbd:1;
+
     // window attributes
     long                 m_windowStyle,
                          m_exStyle;
@@ -1917,7 +1966,6 @@ private:
     // (i.e. not being updated) if it is positive
     unsigned int m_freezeCount;
 
-
     wxDECLARE_ABSTRACT_CLASS(wxWindowBase);
     wxDECLARE_NO_COPY_CLASS(wxWindowBase);
     wxDECLARE_EVENT_TABLE();
@@ -2062,7 +2110,7 @@ extern WXDLLIMPEXP_CORE wxPoint wxGetMousePosition();
 extern WXDLLIMPEXP_CORE wxWindow *wxGetActiveWindow();
 
 // get the (first) top level parent window
-WXDLLIMPEXP_CORE wxWindow* wxGetTopLevelParent(wxWindow *win);
+WXDLLIMPEXP_CORE wxWindow* wxGetTopLevelParent(wxWindowBase *win);
 
 #if wxUSE_ACCESSIBILITY
 // ----------------------------------------------------------------------------
