@@ -4,7 +4,6 @@
 // Author:      Robert Roebling
 // Modified by: Francesco Montorsi
 // Created:     1998
-// RCS-ID:      $Id$
 // Copyright:   (c) 1998-2005 Robert Roebling
 //              (c) 2005-2009 Vadim Zeitlin
 // Licence:     wxWindows licence
@@ -13,9 +12,6 @@
 // For compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-#pragma hdrstop
-#endif
 
 #ifndef WX_PRECOMP
     #include "wx/wx.h"
@@ -24,10 +20,13 @@
 #include "wx/image.h"
 #include "wx/file.h"
 #include "wx/filename.h"
+#include "wx/graphics.h"
 #include "wx/mstream.h"
 #include "wx/wfstream.h"
 #include "wx/quantize.h"
+#include "wx/scopedptr.h"
 #include "wx/stopwatch.h"
+#include "wx/versioninfo.h"
 
 #if wxUSE_CLIPBOARD
     #include "wx/dataobj.h"
@@ -47,7 +46,7 @@
 
 #include "canvas.h"
 
-#ifndef __WXMSW__
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -62,7 +61,7 @@
 class MyApp: public wxApp
 {
 public:
-    virtual bool OnInit();
+    virtual bool OnInit() wxOVERRIDE;
 };
 
 // ----------------------------------------------------------------------------
@@ -76,17 +75,24 @@ public:
 
     void OnAbout( wxCommandEvent &event );
     void OnNewFrame( wxCommandEvent &event );
+    void OnNewFrameHiDPI(wxCommandEvent&);
     void OnImageInfo( wxCommandEvent &event );
     void OnThumbnail( wxCommandEvent &event );
+    void OnUpdateNewFrameHiDPI(wxUpdateUIEvent&);
 
 #ifdef wxHAVE_RAW_BITMAP
     void OnTestRawBitmap( wxCommandEvent &event );
 #endif // wxHAVE_RAW_BITMAP
+#if wxUSE_GRAPHICS_CONTEXT
+    void OnTestGraphics(wxCommandEvent& event);
+#endif // wxUSE_GRAPHICS_CONTEXT
     void OnQuit( wxCommandEvent &event );
 
 #if wxUSE_CLIPBOARD
     void OnCopy(wxCommandEvent& event);
     void OnPaste(wxCommandEvent& event);
+    void OnCopyImage(wxCommandEvent& evt);
+    void OnPasteImage(wxCommandEvent& evt);
 #endif // wxUSE_CLIPBOARD
 
     MyCanvas         *m_canvas;
@@ -99,8 +105,8 @@ private:
     static wxString LoadUserImage(wxImage& image);
 
 
-    DECLARE_DYNAMIC_CLASS(MyFrame)
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_DYNAMIC_CLASS(MyFrame);
+    wxDECLARE_EVENT_TABLE();
 };
 
 // ----------------------------------------------------------------------------
@@ -118,9 +124,41 @@ enum
 class MyImageFrame : public wxFrame
 {
 public:
-    MyImageFrame(wxFrame *parent, const wxString& desc, const wxImage& image)
+    MyImageFrame(wxFrame *parent, const wxString& desc, const wxImage& image, double scale = 1.0)
     {
-        Create(parent, desc, wxBitmap(image), image.GetImageCount(desc));
+        // Retrieve image info
+        wxString info;
+        int xres, yres;
+        switch ( GetResolutionFromOptions(image, &xres, &yres) )
+        {
+            case wxIMAGE_RESOLUTION_NONE:
+                break;
+
+            case wxIMAGE_RESOLUTION_CM:
+                // convert to DPI
+                xres = wxRound(xres / 10.0 * inches2mm);
+                yres = wxRound(yres / 10.0 * inches2mm);
+                wxFALLTHROUGH;
+
+            case wxIMAGE_RESOLUTION_INCHES:
+                info = wxString::Format("DPI %i x %i", xres, yres);
+                break;
+
+            default:
+                wxFAIL_MSG("unexpected image resolution units");
+                break;
+        }
+
+        int numImages = desc.StartsWith("Clipboard") ? 1 : image.GetImageCount(desc);
+        if ( numImages > 1 )
+        {
+            if ( !info.empty() )
+                info += ", ";
+
+            info += wxString::Format("%d images", numImages);
+        }
+
+        Create(parent, desc, wxBitmap(image, wxBITMAP_SCREEN_DEPTH, scale), info);
     }
 
     MyImageFrame(wxFrame *parent, const wxString& desc, const wxBitmap& bitmap)
@@ -132,10 +170,10 @@ private:
     bool Create(wxFrame *parent,
                 const wxString& desc,
                 const wxBitmap& bitmap,
-                int numImages = 1)
+                wxString info = wxString())
     {
         if ( !wxFrame::Create(parent, wxID_ANY,
-                              wxString::Format(wxT("Image from %s"), desc),
+                              wxString::Format("Image from %s", desc),
                               wxDefaultPosition, wxDefaultSize,
                               wxDEFAULT_FRAME_STYLE | wxFULL_REPAINT_ON_RESIZE) )
             return false;
@@ -144,28 +182,27 @@ private:
         m_zoom = 1.;
 
         wxMenu *menu = new wxMenu;
-        menu->Append(wxID_SAVE);
+        menu->Append(wxID_SAVEAS);
         menu->AppendSeparator();
-        menu->AppendCheckItem(ID_PAINT_BG, wxT("&Paint background"),
+        menu->AppendCheckItem(ID_PAINT_BG, "&Paint background",
                               "Uncheck this for transparent images");
         menu->AppendSeparator();
-        menu->Append(ID_RESIZE, wxT("&Fit to window\tCtrl-F"));
+        menu->Append(ID_RESIZE, "&Fit to window\tCtrl-F");
         menu->Append(wxID_ZOOM_IN, "Zoom &in\tCtrl-+");
         menu->Append(wxID_ZOOM_OUT, "Zoom &out\tCtrl--");
         menu->Append(wxID_ZOOM_100, "Reset zoom to &100%\tCtrl-1");
         menu->AppendSeparator();
-        menu->Append(ID_ROTATE_LEFT, wxT("Rotate &left\tCtrl-L"));
-        menu->Append(ID_ROTATE_RIGHT, wxT("Rotate &right\tCtrl-R"));
+        menu->Append(ID_ROTATE_LEFT, "Rotate &left\tCtrl-L");
+        menu->Append(ID_ROTATE_RIGHT, "Rotate &right\tCtrl-R");
 
         wxMenuBar *mbar = new wxMenuBar;
-        mbar->Append(menu, wxT("&Image"));
+        mbar->Append(menu, "&Image");
         SetMenuBar(mbar);
 
         mbar->Check(ID_PAINT_BG, true);
 
         CreateStatusBar(2);
-        if ( numImages != 1 )
-            SetStatusText(wxString::Format("%d images", numImages), 1);
+        SetStatusText(info, 1);
 
         SetClientSize(bitmap.GetWidth(), bitmap.GetHeight());
 
@@ -205,29 +242,32 @@ private:
 #if wxUSE_FILEDLG
         wxImage image = m_bitmap.ConvertToImage();
 
-        wxString savefilename = wxFileSelector( wxT("Save Image"),
+        wxString savefilename = wxFileSelector( "Save Image",
                                                 wxEmptyString,
                                                 wxEmptyString,
-                                                (const wxChar *)NULL,
-                                                wxT("BMP files (*.bmp)|*.bmp|")
+                                                wxEmptyString,
+                                                "BMP files (*.bmp)|*.bmp|"
 #if wxUSE_LIBPNG
-                                                wxT("PNG files (*.png)|*.png|")
+                                                "PNG files (*.png)|*.png|"
 #endif
 #if wxUSE_LIBJPEG
-                                                wxT("JPEG files (*.jpg)|*.jpg|")
+                                                "JPEG files (*.jpg)|*.jpg|"
 #endif
 #if wxUSE_GIF
-                                                wxT("GIF files (*.gif)|*.gif|")
+                                                "GIF files (*.gif)|*.gif|"
 #endif
 #if wxUSE_LIBTIFF
-                                                wxT("TIFF files (*.tif)|*.tif|")
+                                                "TIFF files (*.tif)|*.tif|"
 #endif
 #if wxUSE_PCX
-                                                wxT("PCX files (*.pcx)|*.pcx|")
+                                                "PCX files (*.pcx)|*.pcx|"
 #endif
-                                                wxT("ICO files (*.ico)|*.ico|")
-                                                wxT("CUR files (*.cur)|*.cur"),
-                                                wxFD_SAVE,
+#if wxUSE_XPM
+                                                "X PixMap files (*.xpm)|*.xpm|"
+#endif
+                                                "ICO files (*.ico)|*.ico|"
+                                                "CUR files (*.cur)|*.cur",
+                                                wxFD_SAVE | wxFD_OVERWRITE_PROMPT,
                                                 this);
 
         if ( savefilename.empty() )
@@ -237,7 +277,7 @@ private:
         wxFileName::SplitPath(savefilename, NULL, NULL, &extension);
 
         bool saved = false;
-        if ( extension == wxT("bmp") )
+        if ( extension == "bmp" )
         {
             static const int bppvalues[] =
             {
@@ -247,24 +287,28 @@ private:
                 wxBMP_8BPP,
                 wxBMP_8BPP_GREY,
                 wxBMP_8BPP_RED,
+#if wxUSE_PALETTE
                 wxBMP_8BPP_PALETTE,
+#endif // wxUSE_PALETTE
                 wxBMP_24BPP
             };
 
             const wxString bppchoices[] =
             {
-                wxT("1 bpp color"),
-                wxT("1 bpp B&W"),
-                wxT("4 bpp color"),
-                wxT("8 bpp color"),
-                wxT("8 bpp greyscale"),
-                wxT("8 bpp red"),
-                wxT("8 bpp own palette"),
-                wxT("24 bpp")
+                "1 bpp color",
+                "1 bpp B&W",
+                "4 bpp color",
+                "8 bpp color",
+                "8 bpp greyscale",
+                "8 bpp red",
+#if wxUSE_PALETTE
+                "8 bpp own palette",
+#endif // wxUSE_PALETTE
+                "24 bpp"
             };
 
-            int bppselection = wxGetSingleChoiceIndex(wxT("Set BMP BPP"),
-                                                    wxT("Image sample: save file"),
+            int bppselection = wxGetSingleChoiceIndex("Set BMP BPP",
+                                                    "Image sample: save file",
                                                     WXSIZEOF(bppchoices),
                                                     bppchoices,
                                                     this);
@@ -272,7 +316,7 @@ private:
             {
                 int format = bppvalues[bppselection];
                 image.SetOption(wxIMAGE_OPTION_BMP_FORMAT, format);
-
+#if wxUSE_PALETTE
                 if ( format == wxBMP_8BPP_PALETTE )
                 {
                     unsigned char *cmap = new unsigned char [256];
@@ -282,10 +326,11 @@ private:
 
                     delete[] cmap;
                 }
+#endif // wxUSE_PALETTE
             }
         }
 #if wxUSE_LIBPNG
-        else if ( extension == wxT("png") )
+        else if ( extension == "png" )
         {
             static const int pngvalues[] =
             {
@@ -299,16 +344,16 @@ private:
 
             const wxString pngchoices[] =
             {
-                wxT("Colour 8bpp"),
-                wxT("Colour 16bpp"),
-                wxT("Grey 8bpp"),
-                wxT("Grey 16bpp"),
-                wxT("Grey red 8bpp"),
-                wxT("Grey red 16bpp"),
+                "Colour 8bpp",
+                "Colour 16bpp",
+                "Grey 8bpp",
+                "Grey 16bpp",
+                "Grey red 8bpp",
+                "Grey red 16bpp",
             };
 
-            int sel = wxGetSingleChoiceIndex(wxT("Set PNG format"),
-                                            wxT("Image sample: save file"),
+            int sel = wxGetSingleChoiceIndex("Set PNG format",
+                                            "Image sample: save file",
                                             WXSIZEOF(pngchoices),
                                             pngchoices,
                                             this);
@@ -320,26 +365,26 @@ private:
                 // these values are taken from OptiPNG with -o3 switch
                 const wxString compressionChoices[] =
                 {
-                    wxT("compression = 9, memory = 8, strategy = 0, filter = 0"),
-                    wxT("compression = 9, memory = 9, strategy = 0, filter = 0"),
-                    wxT("compression = 9, memory = 8, strategy = 1, filter = 0"),
-                    wxT("compression = 9, memory = 9, strategy = 1, filter = 0"),
-                    wxT("compression = 1, memory = 8, strategy = 2, filter = 0"),
-                    wxT("compression = 1, memory = 9, strategy = 2, filter = 0"),
-                    wxT("compression = 9, memory = 8, strategy = 0, filter = 5"),
-                    wxT("compression = 9, memory = 9, strategy = 0, filter = 5"),
-                    wxT("compression = 9, memory = 8, strategy = 1, filter = 5"),
-                    wxT("compression = 9, memory = 9, strategy = 1, filter = 5"),
-                    wxT("compression = 1, memory = 8, strategy = 2, filter = 5"),
-                    wxT("compression = 1, memory = 9, strategy = 2, filter = 5"),
+                    "compression = 9, memory = 8, strategy = 0, filter = 0",
+                    "compression = 9, memory = 9, strategy = 0, filter = 0",
+                    "compression = 9, memory = 8, strategy = 1, filter = 0",
+                    "compression = 9, memory = 9, strategy = 1, filter = 0",
+                    "compression = 1, memory = 8, strategy = 2, filter = 0",
+                    "compression = 1, memory = 9, strategy = 2, filter = 0",
+                    "compression = 9, memory = 8, strategy = 0, filter = 5",
+                    "compression = 9, memory = 9, strategy = 0, filter = 5",
+                    "compression = 9, memory = 8, strategy = 1, filter = 5",
+                    "compression = 9, memory = 9, strategy = 1, filter = 5",
+                    "compression = 1, memory = 8, strategy = 2, filter = 5",
+                    "compression = 1, memory = 9, strategy = 2, filter = 5",
                 };
 
-                int sel = wxGetSingleChoiceIndex(wxT("Select compression option (Cancel to use default)\n"),
-                                                 wxT("PNG Compression Options"),
-                                                 WXSIZEOF(compressionChoices),
-                                                 compressionChoices,
-                                                 this);
-                if (sel != -1)
+                sel = wxGetSingleChoiceIndex("Select compression option (Cancel to use default)\n",
+                                             "PNG Compression Options",
+                                             WXSIZEOF(compressionChoices),
+                                             compressionChoices,
+                                             this);
+                if ( sel != -1 )
                 {
                     const int zc[] = {9, 9, 9, 9, 1, 1, 9, 9, 9, 9, 1, 1};
                     const int zm[] = {8, 9, 8, 9, 8, 9, 8, 9, 8, 9, 8, 9};
@@ -356,7 +401,7 @@ private:
             }
         }
 #endif // wxUSE_LIBPNG
-        else if ( extension == wxT("cur") )
+        else if ( extension == "cur" )
         {
             image.Rescale(32,32);
             image.SetOption(wxIMAGE_OPTION_CUR_HOTSPOT_X, 0);
@@ -406,9 +451,9 @@ private:
 
         wxImage img(m_bitmap.ConvertToImage());
         img = img.Rotate(angle, wxPoint(img.GetWidth() / 2, img.GetHeight() / 2));
-        if ( !img.Ok() )
+        if ( !img.IsOk() )
         {
-            wxLogWarning(wxT("Rotation failed"));
+            wxLogWarning("Rotation failed");
             return;
         }
 
@@ -419,17 +464,52 @@ private:
 
     void UpdateStatusBar()
     {
-        wxLogStatus(this, wxT("Image size: (%d, %d), zoom %.2f"),
+        wxLogStatus(this, "Image size: (%d, %d), zoom %.2f",
                     m_bitmap.GetWidth(),
                     m_bitmap.GetHeight(),
                     m_zoom);
         Refresh();
     }
 
+    // This is a copy of protected wxImageHandler::GetResolutionFromOptions()
+    static wxImageResolution GetResolutionFromOptions(const wxImage& image, int* x, int* y)
+    {
+        wxCHECK_MSG(x && y, wxIMAGE_RESOLUTION_NONE, wxT("NULL pointer"));
+
+        if ( image.HasOption(wxIMAGE_OPTION_RESOLUTIONX) &&
+            image.HasOption(wxIMAGE_OPTION_RESOLUTIONY) )
+        {
+            *x = image.GetOptionInt(wxIMAGE_OPTION_RESOLUTIONX);
+            *y = image.GetOptionInt(wxIMAGE_OPTION_RESOLUTIONY);
+        }
+        else if ( image.HasOption(wxIMAGE_OPTION_RESOLUTION) )
+        {
+            *x =
+            *y = image.GetOptionInt(wxIMAGE_OPTION_RESOLUTION);
+        }
+        else // no resolution options specified
+        {
+            *x =
+            *y = 0;
+
+            return wxIMAGE_RESOLUTION_NONE;
+        }
+
+        // get the resolution unit too
+        int resUnit = image.GetOptionInt(wxIMAGE_OPTION_RESOLUTIONUNIT);
+        if ( !resUnit )
+        {
+            // this is the default
+            resUnit = wxIMAGE_RESOLUTION_INCHES;
+        }
+
+        return (wxImageResolution)resUnit;
+    }
+
     wxBitmap m_bitmap;
     double m_zoom;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 #ifdef wxHAVE_RAW_BITMAP
@@ -447,7 +527,7 @@ public:
     };
 
     MyRawBitmapFrame(wxFrame *parent)
-        : wxFrame(parent, wxID_ANY, wxT("Raw bitmaps (how exciting)")),
+        : wxFrame(parent, wxID_ANY, "Raw bitmaps (how exciting)"),
         m_bitmap(SIZE, SIZE, 24),
         m_alphaBitmap(SIZE, SIZE, 32)
     {
@@ -465,7 +545,7 @@ public:
             wxAlphaPixelData data( m_alphaBitmap, wxPoint(0,0), wxSize(SIZE, SIZE) );
             if ( !data )
             {
-                wxLogError(wxT("Failed to gain raw access to bitmap data"));
+                wxLogError("Failed to gain raw access to bitmap data");
                 return;
             }
             wxAlphaPixelData::Iterator p(data);
@@ -487,7 +567,7 @@ public:
                             wxSize(REAL_SIZE, REAL_SIZE));
         if ( !data )
         {
-            wxLogError(wxT("Failed to gain raw access to bitmap data"));
+            wxLogError("Failed to gain raw access to bitmap data");
             return;
         }
 
@@ -524,7 +604,7 @@ public:
         wxNativePixelData data(m_bitmap);
         if ( !data )
         {
-            wxLogError(wxT("Failed to gain raw access to bitmap data"));
+            wxLogError("Failed to gain raw access to bitmap data");
             return;
         }
 
@@ -553,12 +633,12 @@ public:
     void OnPaint(wxPaintEvent& WXUNUSED(event))
     {
         wxPaintDC dc( this );
-        dc.DrawText(wxT("This is alpha and raw bitmap test"), 0, BORDER);
-        dc.DrawText(wxT("This is alpha and raw bitmap test"), 0, SIZE/2 - BORDER);
-        dc.DrawText(wxT("This is alpha and raw bitmap test"), 0, SIZE - 2*BORDER);
+        dc.DrawText("This is alpha and raw bitmap test", 0, BORDER);
+        dc.DrawText("This is alpha and raw bitmap test", 0, SIZE/2 - BORDER);
+        dc.DrawText("This is alpha and raw bitmap test", 0, SIZE - 2*BORDER);
         dc.DrawBitmap( m_alphaBitmap, 0, 0, true /* use mask */ );
 
-        dc.DrawText(wxT("Raw bitmap access without alpha"), 0, SIZE+5);
+        dc.DrawText("Raw bitmap access without alpha", 0, SIZE+5);
         dc.DrawBitmap( m_bitmap, 0, SIZE+5+dc.GetCharHeight());
     }
 
@@ -566,7 +646,7 @@ private:
     wxBitmap m_bitmap;
     wxBitmap m_alphaBitmap;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 #endif // wxHAVE_RAW_BITMAP
@@ -580,18 +660,18 @@ private:
 // MyImageFrame
 //-----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(MyImageFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(MyImageFrame, wxFrame)
     EVT_ERASE_BACKGROUND(MyImageFrame::OnEraseBackground)
     EVT_PAINT(MyImageFrame::OnPaint)
 
-    EVT_MENU(wxID_SAVE, MyImageFrame::OnSave)
+    EVT_MENU(wxID_SAVEAS, MyImageFrame::OnSave)
     EVT_MENU_RANGE(ID_ROTATE_LEFT, ID_ROTATE_RIGHT, MyImageFrame::OnRotate)
     EVT_MENU(ID_RESIZE, MyImageFrame::OnResize)
 
     EVT_MENU(wxID_ZOOM_IN, MyImageFrame::OnZoom)
     EVT_MENU(wxID_ZOOM_OUT, MyImageFrame::OnZoom)
     EVT_MENU(wxID_ZOOM_100, MyImageFrame::OnZoom)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 //-----------------------------------------------------------------------------
 // MyRawBitmapFrame
@@ -599,9 +679,9 @@ END_EVENT_TABLE()
 
 #ifdef wxHAVE_RAW_BITMAP
 
-BEGIN_EVENT_TABLE(MyRawBitmapFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(MyRawBitmapFrame, wxFrame)
     EVT_PAINT(MyRawBitmapFrame::OnPaint)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 #endif // wxHAVE_RAW_BITMAP
 
@@ -614,29 +694,40 @@ enum
     ID_QUIT  = wxID_EXIT,
     ID_ABOUT = wxID_ABOUT,
     ID_NEW = 100,
+    ID_NEW_HIDPI,
     ID_INFO,
     ID_SHOWRAW,
-    ID_SHOWTHUMBNAIL
+    ID_GRAPHICS,
+    ID_SHOWTHUMBNAIL,
+    ID_COPY_IMAGE,
+    ID_PASTE_IMAGE
 };
 
-IMPLEMENT_DYNAMIC_CLASS( MyFrame, wxFrame )
-BEGIN_EVENT_TABLE(MyFrame, wxFrame)
+wxIMPLEMENT_DYNAMIC_CLASS( MyFrame, wxFrame );
+wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU    (ID_ABOUT, MyFrame::OnAbout)
     EVT_MENU    (ID_QUIT,  MyFrame::OnQuit)
     EVT_MENU    (ID_NEW,   MyFrame::OnNewFrame)
+    EVT_MENU    (ID_NEW_HIDPI,   MyFrame::OnNewFrameHiDPI)
     EVT_MENU    (ID_INFO,  MyFrame::OnImageInfo)
     EVT_MENU    (ID_SHOWTHUMBNAIL, MyFrame::OnThumbnail)
 #ifdef wxHAVE_RAW_BITMAP
     EVT_MENU    (ID_SHOWRAW, MyFrame::OnTestRawBitmap)
 #endif
+#if wxUSE_GRAPHICS_CONTEXT
+    EVT_MENU    (ID_GRAPHICS, MyFrame::OnTestGraphics)
+#endif // wxUSE_GRAPHICS_CONTEXT
 #if wxUSE_CLIPBOARD
     EVT_MENU(wxID_COPY, MyFrame::OnCopy)
     EVT_MENU(wxID_PASTE, MyFrame::OnPaste)
+    EVT_MENU(ID_COPY_IMAGE, MyFrame::OnCopyImage)
+    EVT_MENU(ID_PASTE_IMAGE, MyFrame::OnPasteImage)
 #endif // wxUSE_CLIPBOARD
-END_EVENT_TABLE()
+    EVT_UPDATE_UI(ID_NEW_HIDPI, MyFrame::OnUpdateNewFrameHiDPI)
+wxEND_EVENT_TABLE()
 
 MyFrame::MyFrame()
-    : wxFrame( (wxFrame *)NULL, wxID_ANY, wxT("wxImage sample"),
+    : wxFrame( (wxFrame *)NULL, wxID_ANY, "wxImage sample",
                 wxPoint(20, 20), wxSize(950, 700) )
 {
     SetIcon(wxICON(sample));
@@ -644,26 +735,34 @@ MyFrame::MyFrame()
     wxMenuBar *menu_bar = new wxMenuBar();
 
     wxMenu *menuImage = new wxMenu;
-    menuImage->Append( ID_NEW, wxT("&Show any image...\tCtrl-O"));
-    menuImage->Append( ID_INFO, wxT("Show image &information...\tCtrl-I"));
+    menuImage->Append( ID_NEW, "&Show any image...\tCtrl-O");
+    menuImage->Append(ID_NEW_HIDPI, "Show any image as &HiDPI...\tCtrl-H");
+    menuImage->Append( ID_INFO, "Show image &information...\tCtrl-I");
 #ifdef wxHAVE_RAW_BITMAP
     menuImage->AppendSeparator();
-    menuImage->Append( ID_SHOWRAW, wxT("Test &raw bitmap...\tCtrl-R"));
+    menuImage->Append( ID_SHOWRAW, "Test &raw bitmap...\tCtrl-R");
 #endif
+#if wxUSE_GRAPHICS_CONTEXT
     menuImage->AppendSeparator();
-    menuImage->Append( ID_SHOWTHUMBNAIL, wxT("Test &thumbnail...\tCtrl-T"),
+    menuImage->Append(ID_GRAPHICS, "Test &graphics context...\tCtrl-G");
+#endif // wxUSE_GRAPHICS_CONTEXT
+    menuImage->AppendSeparator();
+    menuImage->Append( ID_SHOWTHUMBNAIL, "Test &thumbnail...\tCtrl-T",
                         "Test scaling the image during load (try with JPEG)");
     menuImage->AppendSeparator();
-    menuImage->Append( ID_ABOUT, wxT("&About..."));
+    menuImage->Append( ID_ABOUT, "&About\tF1");
     menuImage->AppendSeparator();
-    menuImage->Append( ID_QUIT, wxT("E&xit\tCtrl-Q"));
-    menu_bar->Append(menuImage, wxT("&Image"));
+    menuImage->Append( ID_QUIT, "E&xit\tCtrl-Q");
+    menu_bar->Append(menuImage, "&Image");
 
 #if wxUSE_CLIPBOARD
     wxMenu *menuClipboard = new wxMenu;
-    menuClipboard->Append(wxID_COPY, wxT("&Copy test image\tCtrl-C"));
-    menuClipboard->Append(wxID_PASTE, wxT("&Paste image\tCtrl-V"));
-    menu_bar->Append(menuClipboard, wxT("&Clipboard"));
+    menuClipboard->Append(wxID_COPY, "&Copy test image as wxBitmap\tCtrl-C");
+    menuClipboard->Append(wxID_PASTE, "&Paste image as wxBitmap\tCtrl-V");
+    menuClipboard->AppendSeparator();
+    menuClipboard->Append(ID_COPY_IMAGE, "Copy image as wxImage");
+    menuClipboard->Append(ID_PASTE_IMAGE, "Paste image as wxImage");
+    menu_bar->Append(menuClipboard, "&Clipboard");
 #endif // wxUSE_CLIPBOARD
 
     SetMenuBar( menu_bar );
@@ -678,6 +777,7 @@ MyFrame::MyFrame()
 
     // 500 width * 2750 height
     m_canvas->SetScrollbars( 10, 10, 50, 275 );
+    m_canvas->SetCursor(wxImage("cursor.png"));
 }
 
 void MyFrame::OnQuit( wxCommandEvent &WXUNUSED(event) )
@@ -685,11 +785,35 @@ void MyFrame::OnQuit( wxCommandEvent &WXUNUSED(event) )
     Close( true );
 }
 
+#if wxUSE_ZLIB && wxUSE_STREAMS
+#include "wx/zstream.h"
+#endif
+
 void MyFrame::OnAbout( wxCommandEvent &WXUNUSED(event) )
 {
-    (void)wxMessageBox( "wxImage demo\n"
-                        "(c) Robert Roebling 1998-2005"
-                        "(c) Vadim Zeitlin 2005-2009",
+    wxArrayString array;
+
+    array.Add("wxImage demo");
+    array.Add("(c) Robert Roebling 1998-2005");
+    array.Add("(c) Vadim Zeitlin 2005-2009");
+
+    array.Add(wxEmptyString);
+    array.Add("Version of the libraries used:");
+
+#if wxUSE_LIBPNG
+    array.Add(wxPNGHandler::GetLibraryVersionInfo().ToString());
+#endif
+#if wxUSE_LIBJPEG
+    array.Add(wxJPEGHandler::GetLibraryVersionInfo().ToString());
+#endif
+#if wxUSE_LIBTIFF
+    array.Add(wxTIFFHandler::GetLibraryVersionInfo().ToString());
+#endif
+#if wxUSE_ZLIB && wxUSE_STREAMS
+    // zlib is used by libpng
+    array.Add(wxGetZlibVersionInfo().ToString());
+#endif
+    (void)wxMessageBox( wxJoin(array, '\n'),
                         "About wxImage Demo",
                         wxICON_INFORMATION | wxOK );
 }
@@ -699,12 +823,12 @@ wxString MyFrame::LoadUserImage(wxImage& image)
     wxString filename;
 
 #if wxUSE_FILEDLG
-    filename = wxLoadFileSelector(wxT("image"), wxEmptyString);
+    filename = wxLoadFileSelector("image", wxEmptyString);
     if ( !filename.empty() )
     {
         if ( !image.LoadFile(filename) )
         {
-            wxLogError(wxT("Couldn't load image from '%s'."), filename.c_str());
+            wxLogError("Couldn't load image from '%s'.", filename);
 
             return wxEmptyString;
         }
@@ -722,6 +846,19 @@ void MyFrame::OnNewFrame( wxCommandEvent &WXUNUSED(event) )
         new MyImageFrame(this, filename, image);
 }
 
+void MyFrame::OnNewFrameHiDPI(wxCommandEvent&)
+{
+    wxImage image;
+    wxString filename = LoadUserImage(image);
+    if (!filename.empty())
+        new MyImageFrame(this, filename, image, GetContentScaleFactor());
+}
+
+void MyFrame::OnUpdateNewFrameHiDPI(wxUpdateUIEvent& event)
+{
+    event.Enable(GetContentScaleFactor() > 1);
+}
+
 void MyFrame::OnImageInfo( wxCommandEvent &WXUNUSED(event) )
 {
     wxImage image;
@@ -741,7 +878,7 @@ void MyFrame::OnImageInfo( wxCommandEvent &WXUNUSED(event) )
             {
                 default:
                     wxFAIL_MSG( "unknown image resolution units" );
-                    // fall through
+                    wxFALLTHROUGH;
 
                 case wxIMAGE_RESOLUTION_NONE:
                     info += " in default units";
@@ -770,6 +907,85 @@ void MyFrame::OnTestRawBitmap( wxCommandEvent &WXUNUSED(event) )
 
 #endif // wxHAVE_RAW_BITMAP
 
+#if wxUSE_GRAPHICS_CONTEXT
+
+class MyGraphicsFrame : public wxFrame
+{
+public:
+    enum
+    {
+        WIDTH = 256,
+        HEIGHT = 90
+    };
+
+    MyGraphicsFrame(wxWindow* parent) :
+        wxFrame(parent, wxID_ANY, "Graphics context test"),
+        m_image(WIDTH, HEIGHT, false)
+    {
+        // Create a test image: it has 3 horizontal primary colour bands with
+        // alpha increasing from left to right.
+        m_image.SetAlpha();
+        unsigned char* alpha = m_image.GetAlpha();
+        unsigned char* data = m_image.GetData();
+
+        for ( int y = 0; y < HEIGHT; y++ )
+        {
+            unsigned char r = 0,
+                          g = 0,
+                          b = 0;
+            if ( y < HEIGHT/3 )
+                r = 0xff;
+            else if ( y < (2*HEIGHT)/3 )
+                g = 0xff;
+            else
+                b = 0xff;
+
+            for ( int x = 0; x < WIDTH; x++ )
+            {
+                *alpha++ = x;
+                *data++ = r;
+                *data++ = g;
+                *data++ = b;
+            }
+        }
+
+        m_bitmap = wxBitmap(m_image);
+
+        Bind(wxEVT_PAINT, &MyGraphicsFrame::OnPaint, this);
+
+        Show();
+    }
+
+private:
+    void OnPaint(wxPaintEvent& WXUNUSED(event))
+    {
+        wxPaintDC dc(this);
+        wxScopedPtr<wxGraphicsContext> gc(wxGraphicsContext::Create(dc));
+        wxGraphicsBitmap gb(gc->CreateBitmapFromImage(m_image));
+
+        gc->SetFont(*wxNORMAL_FONT, *wxBLACK);
+        gc->DrawText("Bitmap", 0, HEIGHT/2);
+        gc->DrawBitmap(m_bitmap, 0, 0, WIDTH, HEIGHT);
+
+        wxGraphicsFont gf = gc->CreateFont(wxNORMAL_FONT->GetPixelSize().y, "");
+        gc->SetFont(gf);
+        gc->DrawText("Graphics bitmap", 0, (3*HEIGHT)/2);
+        gc->DrawBitmap(gb, 0, HEIGHT, WIDTH, HEIGHT);
+    }
+
+    wxImage m_image;
+    wxBitmap m_bitmap;
+
+    wxDECLARE_NO_COPY_CLASS(MyGraphicsFrame);
+};
+
+void MyFrame::OnTestGraphics(wxCommandEvent& WXUNUSED(event))
+{
+    new MyGraphicsFrame(this);
+}
+
+#endif // wxUSE_GRAPHICS_CONTEXT
+
 #if wxUSE_CLIPBOARD
 
 void MyFrame::OnCopy(wxCommandEvent& WXUNUSED(event))
@@ -781,7 +997,7 @@ void MyFrame::OnCopy(wxCommandEvent& WXUNUSED(event))
 
     if ( !wxTheClipboard->SetData(dobjBmp) )
     {
-        wxLogError(wxT("Failed to copy bitmap to clipboard"));
+        wxLogError("Failed to copy bitmap to clipboard");
     }
 
     wxTheClipboard->Close();
@@ -794,21 +1010,53 @@ void MyFrame::OnPaste(wxCommandEvent& WXUNUSED(event))
     wxTheClipboard->Open();
     if ( !wxTheClipboard->GetData(dobjBmp) )
     {
-        wxLogMessage(wxT("No bitmap data in the clipboard"));
+        wxLogMessage("No bitmap data in the clipboard");
     }
     else
     {
-        new MyImageFrame(this, wxT("Clipboard"), dobjBmp.GetBitmap());
+        new MyImageFrame(this, "Clipboard", dobjBmp.GetBitmap());
     }
     wxTheClipboard->Close();
 }
 
+void MyFrame::OnCopyImage(wxCommandEvent& WXUNUSED(evt))
+{
+    wxImage img;
+    wxString filename = LoadUserImage(img);
+    if ( filename.empty() )
+        return;
+
+    wxImageDataObject* dobjImage = new wxImageDataObject;
+    dobjImage->SetImage(img);
+
+    wxClipboardLocker clipOpener;
+    if ( !wxTheClipboard->SetData(dobjImage) )
+    {
+        wxLogError("Failed to copy wxImage to clipboard");
+    }
+}
+
+void MyFrame::OnPasteImage(wxCommandEvent& WXUNUSED(evt))
+{
+    wxImageDataObject dobjImage;
+
+    wxClipboardLocker clipOpener;
+    if ( !wxTheClipboard->GetData(dobjImage) )
+    {
+        wxLogMessage("No wxImage data in the clipboard");
+    }
+    else
+    {
+        new MyImageFrame(this, "Clipboard (wxImage)", dobjImage.GetImage());
+    }
+}
+
 #endif // wxUSE_CLIPBOARD
 
 void MyFrame::OnThumbnail( wxCommandEvent &WXUNUSED(event) )
 {
 #if wxUSE_FILEDLG
-    wxString filename = wxLoadFileSelector(wxT("image"), wxEmptyString, wxEmptyString, this);
+    wxString filename = wxLoadFileSelector("image", wxEmptyString, wxEmptyString, this);
     if ( filename.empty() )
         return;
 
@@ -822,16 +1070,20 @@ void MyFrame::OnThumbnail( wxCommandEvent &WXUNUSED(event) )
     wxStopWatch sw;
     if ( !image.LoadFile(filename) )
     {
-        wxLogError(wxT("Couldn't load image from '%s'."), filename.c_str());
+        wxLogError("Couldn't load image from '%s'.", filename);
         return;
     }
 
+    int origWidth = image.GetOptionInt( wxIMAGE_OPTION_ORIGINAL_WIDTH );
+    int origHeight = image.GetOptionInt( wxIMAGE_OPTION_ORIGINAL_HEIGHT );
+
     const long loadTime = sw.Time();
 
     MyImageFrame * const frame = new MyImageFrame(this, filename, image);
-    wxLogStatus(frame, "Loaded \"%s\" in %ldms", filename, loadTime);
+    wxLogStatus(frame, "Loaded \"%s\" in %ldms; original size was (%d, %d)",
+                filename, loadTime, origWidth, origHeight);
 #else
-    wxLogError( wxT("Couldn't create file selector dialog") );
+    wxLogError( "Couldn't create file selector dialog" );
     return;
 #endif // wxUSE_FILEDLG
 }
@@ -840,7 +1092,7 @@ void MyFrame::OnThumbnail( wxCommandEvent &WXUNUSED(event) )
 // MyApp
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_APP(MyApp)
+wxIMPLEMENT_APP(MyApp);
 
 bool MyApp::OnInit()
 {
