@@ -19,9 +19,6 @@
 // for compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 // for all others, include the necessary headers
 #ifndef WX_PRECOMP
@@ -40,6 +37,7 @@
 #include "wx/sizer.h"
 #include "wx/dcmemory.h"
 #include "wx/commandlinkbutton.h"
+#include "wx/valnum.h"
 
 #include "widgets.h"
 
@@ -55,6 +53,7 @@ enum
     ButtonPage_Reset = wxID_HIGHEST,
     ButtonPage_ChangeLabel,
     ButtonPage_ChangeNote,
+    ButtonPage_ChangeImageMargins,
     ButtonPage_Button
 };
 
@@ -89,13 +88,12 @@ class ButtonWidgetsPage : public WidgetsPage
 {
 public:
     ButtonWidgetsPage(WidgetsBookCtrl *book, wxImageList *imaglist);
-    virtual ~ButtonWidgetsPage(){};
 
-    virtual wxControl *GetWidget() const { return m_button; }
-    virtual void RecreateWidget() { CreateButton(); }
+    virtual wxWindow *GetWidget() const wxOVERRIDE { return m_button; }
+    virtual void RecreateWidget() wxOVERRIDE { CreateButton(); }
 
     // lazy creation of the content
-    virtual void CreateContent();
+    virtual void CreateContent() wxOVERRIDE;
 
 protected:
     // event handlers
@@ -105,6 +103,7 @@ protected:
     void OnButtonReset(wxCommandEvent& event);
     void OnButtonChangeLabel(wxCommandEvent& event);
     void OnButtonChangeNote(wxCommandEvent& event);
+    void OnButtonChangeImageMargins(wxCommandEvent& event);
 
     // reset the wxButton parameters
     void Reset();
@@ -112,11 +111,8 @@ protected:
     // (re)create the wxButton
     void CreateButton();
 
-    // add m_button to m_sizerButton using current value of m_chkFit
-    void AddButtonToSizer();
-
     // helper function: create a bitmap for wxBitmapButton
-    wxBitmap CreateBitmap(const wxString& label);
+    wxBitmap CreateBitmap(const wxString& label, const wxArtID& type);
 
 
     // the controls
@@ -134,7 +130,8 @@ protected:
                *m_chkUseMarkup,
 #endif // wxUSE_MARKUP
                *m_chkDefault,
-               *m_chkUseBitmapClass;
+               *m_chkUseBitmapClass,
+               *m_chkDisable;
 
     // more checkboxes for wxBitmapButton only
     wxCheckBox *m_chkUsePressed,
@@ -168,6 +165,13 @@ protected:
     wxSizer *m_sizerNote;
 #endif // wxUSE_COMMANDLINKBUTTON
 
+    // the text entries for image margins
+    wxTextCtrl* m_textImageMarginH;
+    wxTextCtrl* m_textImageMarginV;
+
+    int m_imageMarginH;
+    int m_imageMarginV;
+
 private:
     wxDECLARE_EVENT_TABLE();
     DECLARE_WIDGETS_PAGE(ButtonWidgetsPage)
@@ -183,6 +187,7 @@ wxBEGIN_EVENT_TABLE(ButtonWidgetsPage, WidgetsPage)
     EVT_BUTTON(ButtonPage_Reset, ButtonWidgetsPage::OnButtonReset)
     EVT_BUTTON(ButtonPage_ChangeLabel, ButtonWidgetsPage::OnButtonChangeLabel)
     EVT_BUTTON(ButtonPage_ChangeNote, ButtonWidgetsPage::OnButtonChangeNote)
+    EVT_BUTTON(ButtonPage_ChangeImageMargins, ButtonWidgetsPage::OnButtonChangeImageMargins)
 
     EVT_CHECKBOX(wxID_ANY, ButtonWidgetsPage::OnCheckOrRadioBox)
     EVT_RADIOBOX(wxID_ANY, ButtonWidgetsPage::OnCheckOrRadioBox)
@@ -198,7 +203,7 @@ wxEND_EVENT_TABLE()
     #define FAMILY_CTRLS NATIVE_CTRLS
 #endif
 
-IMPLEMENT_WIDGETS_PAGE(ButtonWidgetsPage, wxT("Button"), FAMILY_CTRLS );
+IMPLEMENT_WIDGETS_PAGE(ButtonWidgetsPage, "Button", FAMILY_CTRLS );
 
 ButtonWidgetsPage::ButtonWidgetsPage(WidgetsBookCtrl *book,
                                      wxImageList *imaglist)
@@ -217,6 +222,7 @@ ButtonWidgetsPage::ButtonWidgetsPage(WidgetsBookCtrl *book,
 #endif // wxUSE_MARKUP
     m_chkDefault =
     m_chkUseBitmapClass =
+    m_chkDisable =
     m_chkUsePressed =
     m_chkUseFocused =
     m_chkUseCurrent =
@@ -228,8 +234,14 @@ ButtonWidgetsPage::ButtonWidgetsPage(WidgetsBookCtrl *book,
 
     m_textLabel = (wxTextCtrl *)NULL;
 
+    m_textImageMarginH = NULL;
+    m_textImageMarginV = NULL;
+
     m_button = (wxButton *)NULL;
     m_sizerButton = (wxSizer *)NULL;
+
+    m_imageMarginH = 0;
+    m_imageMarginV = 0;
 }
 
 void ButtonWidgetsPage::CreateContent()
@@ -237,26 +249,28 @@ void ButtonWidgetsPage::CreateContent()
     wxSizer *sizerTop = new wxBoxSizer(wxHORIZONTAL);
 
     // left pane
-    wxStaticBox *box = new wxStaticBox(this, wxID_ANY, wxT("&Set style"));
+    wxStaticBox *box = new wxStaticBox(this, wxID_ANY, "&Set style");
 
     wxSizer *sizerLeft = new wxStaticBoxSizer(box, wxVERTICAL);
 
     m_chkBitmapOnly = CreateCheckBoxAndAddToSizer(sizerLeft, "&Bitmap only");
     m_chkTextAndBitmap = CreateCheckBoxAndAddToSizer(sizerLeft, "Text &and bitmap");
-    m_chkFit = CreateCheckBoxAndAddToSizer(sizerLeft, wxT("&Fit exactly"));
-    m_chkAuthNeeded = CreateCheckBoxAndAddToSizer(sizerLeft, wxT("Require a&uth"));
+    m_chkFit = CreateCheckBoxAndAddToSizer(sizerLeft, "&Fit exactly");
+    m_chkAuthNeeded = CreateCheckBoxAndAddToSizer(sizerLeft, "Require a&uth");
 #if wxUSE_COMMANDLINKBUTTON
-    m_chkCommandLink = CreateCheckBoxAndAddToSizer(sizerLeft, wxT("Use command &link button"));
+    m_chkCommandLink = CreateCheckBoxAndAddToSizer(sizerLeft, "Use command &link button");
 #endif
 #if wxUSE_MARKUP
     m_chkUseMarkup = CreateCheckBoxAndAddToSizer(sizerLeft, "Interpret &markup");
 #endif // wxUSE_MARKUP
-    m_chkDefault = CreateCheckBoxAndAddToSizer(sizerLeft, wxT("&Default"));
+    m_chkDefault = CreateCheckBoxAndAddToSizer(sizerLeft, "&Default");
 
     m_chkUseBitmapClass = CreateCheckBoxAndAddToSizer(sizerLeft,
         "Use wxBitmapButton");
     m_chkUseBitmapClass->SetValue(true);
 
+    m_chkDisable = CreateCheckBoxAndAddToSizer(sizerLeft, "Disable");
+
     sizerLeft->AddSpacer(5);
 
     wxSizer *sizerUseLabels =
@@ -281,68 +295,90 @@ void ButtonWidgetsPage::CreateContent()
     m_radioImagePos = new wxRadioBox(this, wxID_ANY, "Image &position",
                                      wxDefaultPosition, wxDefaultSize,
                                      WXSIZEOF(dirs), dirs);
-    sizerLeft->Add(m_radioImagePos, 0, wxGROW | wxALL, 5);
+    sizerLeft->Add(m_radioImagePos, wxSizerFlags().Expand().Border());
+
+    wxSizer* sizerImageMarginsRow = CreateSizerWithTextAndButton(ButtonPage_ChangeImageMargins,
+                                           "Horizontal and vertical", wxID_ANY, &m_textImageMarginH);
+    wxIntegerValidator<int> validatorMargH;
+    validatorMargH.SetRange(0, 100);
+    m_textImageMarginH->SetValidator(validatorMargH);
+
+    wxIntegerValidator<int> validatorMargV;
+    validatorMargV.SetRange(0, 100);
+    m_textImageMarginV = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, validatorMargV);
+    sizerImageMarginsRow->Add(m_textImageMarginV, wxSizerFlags(1).CentreVertical().Border(wxLEFT));
+
+    m_textImageMarginH->SetValue(wxString::Format("%d", m_imageMarginH));
+    m_textImageMarginV->SetValue(wxString::Format("%d", m_imageMarginV));
+
+    wxSizer* sizerImageMargins = new wxStaticBoxSizer(wxVERTICAL, this, "Image margins");
+    sizerImageMargins->Add(sizerImageMarginsRow, wxSizerFlags().Border().Centre());
+    sizerLeft->Add(sizerImageMargins, wxSizerFlags().Expand().Border());
+
     sizerLeft->AddSpacer(15);
 
     // should be in sync with enums Button[HV]Align!
     static const wxString halign[] =
     {
-        wxT("left"),
-        wxT("centre"),
-        wxT("right"),
+        "left",
+        "centre",
+        "right",
     };
 
     static const wxString valign[] =
     {
-        wxT("top"),
-        wxT("centre"),
-        wxT("bottom"),
+        "top",
+        "centre",
+        "bottom",
     };
 
-    m_radioHAlign = new wxRadioBox(this, wxID_ANY, wxT("&Horz alignment"),
+    m_radioHAlign = new wxRadioBox(this, wxID_ANY, "&Horz alignment",
                                    wxDefaultPosition, wxDefaultSize,
                                    WXSIZEOF(halign), halign);
-    m_radioVAlign = new wxRadioBox(this, wxID_ANY, wxT("&Vert alignment"),
+    m_radioVAlign = new wxRadioBox(this, wxID_ANY, "&Vert alignment",
                                    wxDefaultPosition, wxDefaultSize,
                                    WXSIZEOF(valign), valign);
 
-    sizerLeft->Add(m_radioHAlign, 0, wxGROW | wxALL, 5);
-    sizerLeft->Add(m_radioVAlign, 0, wxGROW | wxALL, 5);
+    sizerLeft->Add(m_radioHAlign, wxSizerFlags().Expand().Border());
+    sizerLeft->Add(m_radioVAlign, wxSizerFlags().Expand().Border());
 
-    sizerLeft->Add(5, 5, 0, wxGROW | wxALL, 5); // spacer
+    sizerLeft->AddSpacer(5);
 
-    wxButton *btn = new wxButton(this, ButtonPage_Reset, wxT("&Reset"));
-    sizerLeft->Add(btn, 0, wxALIGN_CENTRE_HORIZONTAL | wxALL, 15);
+    wxButton *btn = new wxButton(this, ButtonPage_Reset, "&Reset");
+    sizerLeft->Add(btn, wxSizerFlags().CentreHorizontal().TripleBorder(wxALL));
 
     // middle pane
-    wxStaticBox *box2 = new wxStaticBox(this, wxID_ANY, wxT("&Operations"));
+    wxStaticBox *box2 = new wxStaticBox(this, wxID_ANY, "&Operations");
     wxSizer *sizerMiddle = new wxStaticBoxSizer(box2, wxVERTICAL);
 
     wxSizer *sizerRow = CreateSizerWithTextAndButton(ButtonPage_ChangeLabel,
-                                                     wxT("Change label"),
+                                                     "Change label",
                                                      wxID_ANY,
                                                      &m_textLabel);
-    m_textLabel->SetValue(wxT("&Press me!"));
-    sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
+    m_textLabel->SetValue("&Press me!");
+    sizerMiddle->Add(sizerRow, wxSizerFlags().Expand().Border());
 
 #if wxUSE_COMMANDLINKBUTTON
     m_sizerNote = CreateSizerWithTextAndButton(ButtonPage_ChangeNote,
-                                               wxT("Change note"),
+                                               "Change note",
                                                wxID_ANY,
                                                &m_textNote);
-    m_textNote->SetValue(wxT("Writes down button clicks in the log."));
+    m_textNote->SetValue("Writes down button clicks in the log.");
 
-    sizerMiddle->Add(m_sizerNote, 0, wxALL | wxGROW, 5);
+    sizerMiddle->Add(m_sizerNote, wxSizerFlags().Expand().Border());
 #endif
 
     // right pane
     m_sizerButton = new wxBoxSizer(wxHORIZONTAL);
-    m_sizerButton->SetMinSize(150, 0);
+    m_sizerButton->SetMinSize(FromDIP(150), 0);
 
     // the 3 panes panes compose the window
-    sizerTop->Add(sizerLeft, 0, wxGROW | (wxALL & ~wxLEFT), 10);
-    sizerTop->Add(sizerMiddle, 1, wxGROW | wxALL, 10);
-    sizerTop->Add(m_sizerButton, 1, wxGROW | (wxALL & ~wxRIGHT), 10);
+    sizerTop->Add(sizerLeft,
+                  wxSizerFlags(0).Expand().DoubleBorder(wxALL & ~wxLEFT));
+    sizerTop->Add(sizerMiddle,
+                  wxSizerFlags(1).Expand().DoubleBorder(wxALL));
+    sizerTop->Add(m_sizerButton,
+                  wxSizerFlags(1).Expand().DoubleBorder((wxALL & ~wxRIGHT)));
 
     // do create the main control
     Reset();
@@ -358,7 +394,7 @@ void ButtonWidgetsPage::CreateContent()
 void ButtonWidgetsPage::Reset()
 {
     m_chkBitmapOnly->SetValue(false);
-    m_chkFit->SetValue(true);
+    m_chkFit->SetValue(false);
     m_chkAuthNeeded->SetValue(false);
     m_chkTextAndBitmap->SetValue(false);
     m_chkDefault->SetValue(false);
@@ -369,6 +405,7 @@ void ButtonWidgetsPage::Reset()
     m_chkUseMarkup->SetValue(false);
 #endif // wxUSE_MARKUP
     m_chkUseBitmapClass->SetValue(true);
+    m_chkDisable->SetValue(false);
 
     m_chkUsePressed->SetValue(true);
     m_chkUseFocused->SetValue(true);
@@ -378,6 +415,11 @@ void ButtonWidgetsPage::Reset()
     m_radioImagePos->SetSelection(ButtonImagePos_Left);
     m_radioHAlign->SetSelection(ButtonHAlign_Centre);
     m_radioVAlign->SetSelection(ButtonVAlign_Centre);
+
+    m_imageMarginH = 0;
+    m_imageMarginV = 0;
+    m_textImageMarginH->SetValue(wxString::Format("%d", m_imageMarginH));
+    m_textImageMarginV->SetValue(wxString::Format("%d", m_imageMarginV));
 }
 
 void ButtonWidgetsPage::CreateButton()
@@ -408,7 +450,7 @@ void ButtonWidgetsPage::CreateButton()
         label = m_textLabel->GetValue();
     }
 
-    int flags = ms_defaultFlags;
+    int flags = GetAttrs().m_defaultFlags;
     switch ( m_radioHAlign->GetSelection() )
     {
         case ButtonHAlign_Left:
@@ -416,8 +458,8 @@ void ButtonWidgetsPage::CreateButton()
             break;
 
         default:
-            wxFAIL_MSG(wxT("unexpected radiobox selection"));
-            // fall through
+            wxFAIL_MSG("unexpected radiobox selection");
+            wxFALLTHROUGH;
 
         case ButtonHAlign_Centre:
             break;
@@ -434,8 +476,8 @@ void ButtonWidgetsPage::CreateButton()
             break;
 
         default:
-            wxFAIL_MSG(wxT("unexpected radiobox selection"));
-            // fall through
+            wxFAIL_MSG("unexpected radiobox selection");
+            wxFALLTHROUGH;
 
         case ButtonVAlign_Centre:
             // centre vertical alignment is the default (no style)
@@ -446,34 +488,42 @@ void ButtonWidgetsPage::CreateButton()
             break;
     }
 
-#if wxUSE_COMMANDLINKBUTTON
-    m_sizerNote->Show(m_chkCommandLink->GetValue());
-#endif
+    if ( m_chkFit->GetValue() )
+    {
+        flags |= wxBU_EXACTFIT;
+    }
 
     bool showsBitmap = false;
     if ( m_chkBitmapOnly->GetValue() )
     {
+#if wxUSE_COMMANDLINKBUTTON
+        m_chkCommandLink->SetValue(false); // wxCommandLinkButton cannot be "Bitmap only"
+#endif
+
         showsBitmap = true;
 
         wxButton *bbtn;
         if ( m_chkUseBitmapClass->GetValue() )
         {
           bbtn = new wxBitmapButton(this, ButtonPage_Button,
-                                    CreateBitmap(wxT("normal")));
+                                    CreateBitmap("normal", wxART_INFORMATION),
+                                    wxDefaultPosition, wxDefaultSize, flags);
         }
         else
         {
           bbtn = new wxButton(this, ButtonPage_Button);
-          bbtn->SetBitmapLabel(CreateBitmap(wxT("normal")));
+          bbtn->SetBitmapLabel(CreateBitmap("normal", wxART_INFORMATION));
         }
+        bbtn->SetBitmapMargins((wxCoord)m_imageMarginH, (wxCoord)m_imageMarginV);
+
         if ( m_chkUsePressed->GetValue() )
-            bbtn->SetBitmapPressed(CreateBitmap(wxT("pushed")));
+            bbtn->SetBitmapPressed(CreateBitmap("pushed", wxART_HELP));
         if ( m_chkUseFocused->GetValue() )
-            bbtn->SetBitmapFocus(CreateBitmap(wxT("focused")));
+            bbtn->SetBitmapFocus(CreateBitmap("focused", wxART_ERROR));
         if ( m_chkUseCurrent->GetValue() )
-            bbtn->SetBitmapCurrent(CreateBitmap(wxT("hover")));
+            bbtn->SetBitmapCurrent(CreateBitmap("hover", wxART_WARNING));
         if ( m_chkUseDisabled->GetValue() )
-            bbtn->SetBitmapDisabled(CreateBitmap(wxT("disabled")));
+            bbtn->SetBitmapDisabled(CreateBitmap("disabled", wxART_MISSING_IMAGE));
         m_button = bbtn;
 #if wxUSE_COMMANDLINKBUTTON
         m_cmdLnkButton = NULL;
@@ -503,6 +553,10 @@ void ButtonWidgetsPage::CreateButton()
         }
     }
 
+#if wxUSE_COMMANDLINKBUTTON
+    m_sizerNote->Show(m_chkCommandLink->GetValue());
+#endif
+
     if ( !showsBitmap && m_chkTextAndBitmap->GetValue() )
     {
         showsBitmap = true;
@@ -515,6 +569,8 @@ void ButtonWidgetsPage::CreateButton()
         m_button->SetBitmap(wxArtProvider::GetIcon(wxART_INFORMATION, wxART_BUTTON),
                             positions[m_radioImagePos->GetSelection()]);
 
+        m_button->SetBitmapMargins((wxCoord)m_imageMarginH, (wxCoord)m_imageMarginV);
+
         if ( m_chkUsePressed->GetValue() )
             m_button->SetBitmapPressed(wxArtProvider::GetIcon(wxART_HELP, wxART_BUTTON));
         if ( m_chkUseFocused->GetValue() )
@@ -525,12 +581,21 @@ void ButtonWidgetsPage::CreateButton()
             m_button->SetBitmapDisabled(wxArtProvider::GetIcon(wxART_MISSING_IMAGE, wxART_BUTTON));
     }
 
+    m_chkTextAndBitmap->Enable(!m_chkBitmapOnly->IsChecked());
+    m_chkBitmapOnly->Enable(!m_chkTextAndBitmap->IsChecked());
+#if wxUSE_COMMANDLINKBUTTON
+    m_chkCommandLink->Enable(!m_chkBitmapOnly->IsChecked());
+#endif
     m_chkUseBitmapClass->Enable(showsBitmap);
 
     m_chkUsePressed->Enable(showsBitmap);
     m_chkUseFocused->Enable(showsBitmap);
     m_chkUseCurrent->Enable(showsBitmap);
     m_chkUseDisabled->Enable(showsBitmap);
+    m_radioImagePos->Enable(m_chkTextAndBitmap->IsChecked());
+    m_textImageMarginH->Enable(showsBitmap);
+    m_textImageMarginV->Enable(showsBitmap);
+    wxWindow::FindWindowById(ButtonPage_ChangeImageMargins)->Enable(showsBitmap);
 
     if ( m_chkAuthNeeded->GetValue() )
         m_button->SetAuthNeeded();
@@ -538,25 +603,15 @@ void ButtonWidgetsPage::CreateButton()
     if ( m_chkDefault->GetValue() )
         m_button->SetDefault();
 
-    AddButtonToSizer();
+    m_button->Enable(!m_chkDisable->IsChecked());
+
+    m_sizerButton->AddStretchSpacer();
+    m_sizerButton->Add(m_button, wxSizerFlags().Centre().Border());
+    m_sizerButton->AddStretchSpacer();
 
     m_sizerButton->Layout();
 }
 
-void ButtonWidgetsPage::AddButtonToSizer()
-{
-    if ( m_chkFit->GetValue() )
-    {
-        m_sizerButton->AddStretchSpacer(1);
-        m_sizerButton->Add(m_button, wxSizerFlags(0).Centre().Border());
-        m_sizerButton->AddStretchSpacer(1);
-    }
-    else // take up the entire space
-    {
-        m_sizerButton->Add(m_button, wxSizerFlags(1).Expand().Border());
-    }
-}
-
 // ----------------------------------------------------------------------------
 // event handlers
 // ----------------------------------------------------------------------------
@@ -592,6 +647,9 @@ void ButtonWidgetsPage::OnButtonChangeLabel(wxCommandEvent& WXUNUSED(event))
             m_button->SetLabel(labelText);
     }
 
+    if ( m_chkBitmapOnly->IsChecked() )
+        CreateButton();
+
     m_sizerButton->Layout();
 }
 
@@ -604,26 +662,47 @@ void ButtonWidgetsPage::OnButtonChangeNote(wxCommandEvent& WXUNUSED(event))
 #endif // wxUSE_COMMANDLINKBUTTON
 }
 
+void ButtonWidgetsPage::OnButtonChangeImageMargins(wxCommandEvent& WXUNUSED(event))
+{
+    long margH = 0;
+    long margV = 0;
+    if ( !m_textImageMarginH->GetValue().ToLong(&margH) ||
+         !m_textImageMarginV->GetValue().ToLong(&margV) ||
+         margH < 0 || margV < 0 )
+    {
+        wxLogWarning("Invalid margin values for bitmap.");
+        return;
+    }
+
+    m_imageMarginH = (int)margH;
+    m_imageMarginV = (int)margV;
+
+    m_button->SetBitmapMargins((wxCoord)m_imageMarginH, (wxCoord)m_imageMarginV);
+    m_button->Refresh();
+    m_sizerButton->Layout();
+}
+
 void ButtonWidgetsPage::OnButton(wxCommandEvent& WXUNUSED(event))
 {
-    wxLogMessage(wxT("Test button clicked."));
+    wxLogMessage("Test button clicked.");
 }
 
 // ----------------------------------------------------------------------------
 // bitmap button stuff
 // ----------------------------------------------------------------------------
 
-wxBitmap ButtonWidgetsPage::CreateBitmap(const wxString& label)
+wxBitmap ButtonWidgetsPage::CreateBitmap(const wxString& label, const wxArtID& type)
 {
-    wxBitmap bmp(180, 70); // shouldn't hardcode but it's simpler like this
+    wxBitmap bmp(FromDIP(wxSize(180, 70))); // shouldn't hardcode but it's simpler like this
     wxMemoryDC dc;
     dc.SelectObject(bmp);
+    dc.SetFont(GetFont());
     dc.SetBackground(*wxCYAN_BRUSH);
     dc.Clear();
     dc.SetTextForeground(*wxBLACK);
-    dc.DrawLabel(wxStripMenuCodes(m_textLabel->GetValue()) + wxT("\n")
-                    wxT("(") + label + wxT(" state)"),
-                 wxArtProvider::GetBitmap(wxART_INFORMATION),
+    dc.DrawLabel(wxStripMenuCodes(m_textLabel->GetValue()) + "\n"
+                    "(" + label + " state)",
+                 wxArtProvider::GetBitmap(type),
                  wxRect(10, 10, bmp.GetWidth() - 20, bmp.GetHeight() - 20),
                  wxALIGN_CENTRE);
 
