@@ -18,13 +18,11 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_LISTCTRL
 
 #include "wx/listctrl.h"
+#include "wx/imaglist.h"
 
 #ifndef WX_PRECOMP
     #include "wx/dcclient.h"
@@ -52,6 +50,8 @@ wxDEFINE_EVENT( wxEVT_LIST_ITEM_RIGHT_CLICK, wxListEvent );
 wxDEFINE_EVENT( wxEVT_LIST_ITEM_MIDDLE_CLICK, wxListEvent );
 wxDEFINE_EVENT( wxEVT_LIST_ITEM_ACTIVATED, wxListEvent );
 wxDEFINE_EVENT( wxEVT_LIST_ITEM_FOCUSED, wxListEvent );
+wxDEFINE_EVENT( wxEVT_LIST_ITEM_CHECKED, wxListEvent );
+wxDEFINE_EVENT( wxEVT_LIST_ITEM_UNCHECKED, wxListEvent );
 wxDEFINE_EVENT( wxEVT_LIST_CACHE_HINT, wxListEvent );
 
 // -----------------------------------------------------------------------------
@@ -103,10 +103,10 @@ wxFLAGS_MEMBER(wxLC_SORT_DESCENDING)
 wxFLAGS_MEMBER(wxLC_VIRTUAL)
 wxEND_FLAGS( wxListCtrlStyle )
 
-#if ((!defined(__WXMSW__) && !(defined(__WXMAC__) && wxOSX_USE_CARBON)) || defined(__WXUNIVERSAL__))
-wxIMPLEMENT_DYNAMIC_CLASS_XTI(wxListCtrl, wxGenericListCtrl, "wx/listctrl.h")
+#if ((!defined(__WXMSW__) && !defined(__WXQT__) && !(defined(__WXMAC__) && wxOSX_USE_CARBON)) || defined(__WXUNIVERSAL__))
+wxIMPLEMENT_DYNAMIC_CLASS_XTI(wxListCtrl, wxGenericListCtrl, "wx/listctrl.h");
 #else
-wxIMPLEMENT_DYNAMIC_CLASS_XTI(wxListCtrl, wxControl, "wx/listctrl.h")
+wxIMPLEMENT_DYNAMIC_CLASS_XTI(wxListCtrl, wxControl, "wx/listctrl.h");
 #endif
 
 wxBEGIN_PROPERTIES_TABLE(wxListCtrl)
@@ -123,13 +123,13 @@ wxCONSTRUCTOR_5( wxListCtrl, wxWindow*, Parent, wxWindowID, Id, \
                 wxPoint, Position, wxSize, Size, long, WindowStyle )
 
 /*
- TODO : Expose more information of a list's layout etc. via appropriate objects 
+ TODO : Expose more information of a list's layout etc. via appropriate objects
  (see NotebookPageInfo)
  */
 
-IMPLEMENT_DYNAMIC_CLASS(wxListView, wxListCtrl)
-IMPLEMENT_DYNAMIC_CLASS(wxListItem, wxObject)
-IMPLEMENT_DYNAMIC_CLASS(wxListEvent, wxNotifyEvent)
+wxIMPLEMENT_DYNAMIC_CLASS(wxListView, wxListCtrl);
+wxIMPLEMENT_DYNAMIC_CLASS(wxListItem, wxObject);
+wxIMPLEMENT_DYNAMIC_CLASS(wxListEvent, wxNotifyEvent);
 
 // ----------------------------------------------------------------------------
 // wxListCtrlBase implementation
@@ -242,11 +242,50 @@ void wxListCtrlBase::EnableAlternateRowColours(bool enable)
     }
 }
 
-wxListItemAttr *wxListCtrlBase::OnGetItemAttr(long item) const
+wxItemAttr *wxListCtrlBase::OnGetItemAttr(long item) const
 {
     return (m_alternateRowColour.GetBackgroundColour().IsOk() && (item % 2))
-        ? wxConstCast(&m_alternateRowColour, wxListItemAttr)
+        ? wxConstCast(&m_alternateRowColour, wxItemAttr)
         : NULL; // no attributes by default
 }
 
+wxString wxListCtrlBase::OnGetItemText(long WXUNUSED(item), long WXUNUSED(col)) const
+{
+    // this is a pure virtual function, in fact - which is not really pure
+    // because the controls which are not virtual don't need to implement it
+    wxFAIL_MSG("wxListCtrl::OnGetItemText not supposed to be called");
+
+    return wxEmptyString;
+}
+
+bool wxListCtrlBase::OnGetItemIsChecked(long WXUNUSED(item)) const
+{
+    // this is a pure virtual function, in fact - which is not really pure
+    // because the controls which are not virtual don't need to implement it
+    wxFAIL_MSG("wxListCtrl::OnGetItemIsChecked not supposed to be called");
+
+    return false;
+}
+
+int wxListCtrlBase::OnGetItemImage(long WXUNUSED(item)) const
+{
+    wxCHECK_MSG(!GetImageList(wxIMAGE_LIST_SMALL),
+                -1,
+                "List control has an image list, OnGetItemImage or OnGetItemColumnImage should be overridden.");
+    return -1;
+}
+
+int wxListCtrlBase::OnGetItemColumnImage(long item, long column) const
+{
+    if ( !column )
+        return OnGetItemImage(item);
+
+    return -1;
+}
+
+wxItemAttr* wxListCtrlBase::OnGetItemColumnAttr(long item, long WXUNUSED(column)) const
+{
+    return OnGetItemAttr(item);
+}
+
 #endif // wxUSE_LISTCTRL
