@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////////
 // Name:        src/generic/animateg.cpp
-// Purpose:     wxAnimation and wxAnimationCtrl
+// Purpose:     wxAnimationGenericImpl and wxGenericAnimationCtrl
 // Author:      Julian Smart and Guillermo Rodriguez Garcia
 // Modified by: Francesco Montorsi
 // Created:     13/8/99
@@ -10,106 +10,89 @@
 
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-  #pragma hdrstop
-#endif  //__BORLANDC__
-
 #if wxUSE_ANIMATIONCTRL
 
 #include "wx/animate.h"
+#include "wx/generic/animate.h"
+#include "wx/generic/private/animate.h"
 
 #ifndef WX_PRECOMP
     #include "wx/log.h"
     #include "wx/image.h"
     #include "wx/dcmemory.h"
     #include "wx/dcclient.h"
-    #include "wx/module.h"
 #endif
 
 #include "wx/wfstream.h"
-#include "wx/gifdecod.h"
-#include "wx/anidecod.h"
-
-#include "wx/listimpl.cpp"
-WX_DEFINE_LIST(wxAnimationDecoderList)
-
-wxAnimationDecoderList wxAnimation::sm_handlers;
-
 
 // ----------------------------------------------------------------------------
 // wxAnimation
 // ----------------------------------------------------------------------------
 
-wxIMPLEMENT_DYNAMIC_CLASS(wxAnimation, wxAnimationBase);
-#define M_ANIMDATA      static_cast<wxAnimationDecoder*>(m_refData)
+#ifndef wxHAS_NATIVE_ANIMATIONCTRL
 
-wxSize wxAnimation::GetSize() const
+/* static */
+wxAnimationImpl *wxAnimationImpl::CreateDefault()
 {
-    wxCHECK_MSG( IsOk(), wxDefaultSize, wxT("invalid animation") );
-
-    return M_ANIMDATA->GetAnimationSize();
+    return new wxAnimationGenericImpl();
 }
 
-unsigned int wxAnimation::GetFrameCount() const
-{
-    wxCHECK_MSG( IsOk(), 0, wxT("invalid animation") );
+#endif // !wxHAS_NATIVE_ANIMATIONCTRL
 
-    return M_ANIMDATA->GetFrameCount();
+bool wxAnimationGenericImpl::IsCompatibleWith(wxClassInfo* ci) const
+{
+    return ci->IsKindOf(&wxGenericAnimationCtrl::ms_classInfo);
 }
 
-wxImage wxAnimation::GetFrame(unsigned int i) const
+wxSize wxAnimationGenericImpl::GetSize() const
 {
-    wxCHECK_MSG( IsOk(), wxNullImage, wxT("invalid animation") );
+    return m_decoder->GetAnimationSize();
+}
 
+unsigned int wxAnimationGenericImpl::GetFrameCount() const
+{
+    return m_decoder->GetFrameCount();
+}
+
+wxImage wxAnimationGenericImpl::GetFrame(unsigned int i) const
+{
     wxImage ret;
-    if (!M_ANIMDATA->ConvertToImage(i, &ret))
+    if (!m_decoder->ConvertToImage(i, &ret))
         return wxNullImage;
     return ret;
 }
 
-int wxAnimation::GetDelay(unsigned int i) const
+int wxAnimationGenericImpl::GetDelay(unsigned int i) const
 {
-    wxCHECK_MSG( IsOk(), 0, wxT("invalid animation") );
-
-    return M_ANIMDATA->GetDelay(i);
+    return m_decoder->GetDelay(i);
 }
 
-wxPoint wxAnimation::GetFramePosition(unsigned int frame) const
+wxPoint wxAnimationGenericImpl::GetFramePosition(unsigned int frame) const
 {
-    wxCHECK_MSG( IsOk(), wxDefaultPosition, wxT("invalid animation") );
-
-    return M_ANIMDATA->GetFramePosition(frame);
+    return m_decoder->GetFramePosition(frame);
 }
 
-wxSize wxAnimation::GetFrameSize(unsigned int frame) const
+wxSize wxAnimationGenericImpl::GetFrameSize(unsigned int frame) const
 {
-    wxCHECK_MSG( IsOk(), wxDefaultSize, wxT("invalid animation") );
-
-    return M_ANIMDATA->GetFrameSize(frame);
+    return m_decoder->GetFrameSize(frame);
 }
 
-wxAnimationDisposal wxAnimation::GetDisposalMethod(unsigned int frame) const
+wxAnimationDisposal wxAnimationGenericImpl::GetDisposalMethod(unsigned int frame) const
 {
-    wxCHECK_MSG( IsOk(), wxANIM_UNSPECIFIED, wxT("invalid animation") );
-
-    return M_ANIMDATA->GetDisposalMethod(frame);
+    return m_decoder->GetDisposalMethod(frame);
 }
 
-wxColour wxAnimation::GetTransparentColour(unsigned int frame) const
+wxColour wxAnimationGenericImpl::GetTransparentColour(unsigned int frame) const
 {
-    wxCHECK_MSG( IsOk(), wxNullColour, wxT("invalid animation") );
-
-    return M_ANIMDATA->GetTransparentColour(frame);
+    return m_decoder->GetTransparentColour(frame);
 }
 
-wxColour wxAnimation::GetBackgroundColour() const
+wxColour wxAnimationGenericImpl::GetBackgroundColour() const
 {
-    wxCHECK_MSG( IsOk(), wxNullColour, wxT("invalid animation") );
-
-    return M_ANIMDATA->GetBackgroundColour();
+    return m_decoder->GetBackgroundColour();
 }
 
-bool wxAnimation::LoadFile(const wxString& filename, wxAnimationType type)
+bool wxAnimationGenericImpl::LoadFile(const wxString& filename, wxAnimationType type)
 {
     wxFileInputStream stream(filename);
     if ( !stream.IsOk() )
@@ -118,14 +101,14 @@ bool wxAnimation::LoadFile(const wxString& filename, wxAnimationType type)
     return Load(stream, type);
 }
 
-bool wxAnimation::Load(wxInputStream &stream, wxAnimationType type)
+bool wxAnimationGenericImpl::Load(wxInputStream &stream, wxAnimationType type)
 {
     UnRef();
 
     const wxAnimationDecoder *handler;
     if ( type == wxANIMATION_TYPE_ANY )
     {
-        for ( wxAnimationDecoderList::compatibility_iterator node = sm_handlers.GetFirst();
+        for ( wxAnimationDecoderList::compatibility_iterator node = wxAnimation::GetHandlers().GetFirst();
               node; node = node->GetNext() )
         {
             handler=(const wxAnimationDecoder*)node->GetData();
@@ -134,8 +117,8 @@ bool wxAnimation::Load(wxInputStream &stream, wxAnimationType type)
             {
                 // do a copy of the handler from the static list which we will own
                 // as our reference data
-                m_refData = handler->Clone();
-                return M_ANIMDATA->Load(stream);
+                m_decoder = handler->Clone();
+                return m_decoder->Load(stream);
             }
         }
 
@@ -143,7 +126,7 @@ bool wxAnimation::Load(wxInputStream &stream, wxAnimationType type)
         return false;
     }
 
-    handler = FindHandler(type);
+    handler = wxAnimation::FindHandler(type);
 
     if (handler == NULL)
     {
@@ -155,123 +138,38 @@ bool wxAnimation::Load(wxInputStream &stream, wxAnimationType type)
 
     // do a copy of the handler from the static list which we will own
     // as our reference data
-    m_refData = handler->Clone();
+    m_decoder = handler->Clone();
 
-    if (stream.IsSeekable() && !M_ANIMDATA->CanRead(stream))
+    if (stream.IsSeekable() && !m_decoder->CanRead(stream))
     {
         wxLogError(_("Animation file is not of type %ld."), type);
         return false;
     }
     else
-        return M_ANIMDATA->Load(stream);
+        return m_decoder->Load(stream);
 }
 
-
-// ----------------------------------------------------------------------------
-// animation decoders
-// ----------------------------------------------------------------------------
-
-void wxAnimation::AddHandler( wxAnimationDecoder *handler )
+void wxAnimationGenericImpl::UnRef()
 {
-    // Check for an existing handler of the type being added.
-    if (FindHandler( handler->GetType() ) == 0)
+    if ( m_decoder )
     {
-        sm_handlers.Append( handler );
-    }
-    else
-    {
-        // This is not documented behaviour, merely the simplest 'fix'
-        // for preventing duplicate additions.  If someone ever has
-        // a good reason to add and remove duplicate handlers (and they
-        // may) we should probably refcount the duplicates.
-
-        wxLogDebug( wxT("Adding duplicate animation handler for '%d' type"),
-                    handler->GetType() );
-        delete handler;
+        m_decoder->DecRef();
+        m_decoder = NULL;
     }
 }
 
-void wxAnimation::InsertHandler( wxAnimationDecoder *handler )
-{
-    // Check for an existing handler of the type being added.
-    if (FindHandler( handler->GetType() ) == 0)
-    {
-        sm_handlers.Insert( handler );
-    }
-    else
-    {
-        // see AddHandler for additional comments.
-        wxLogDebug( wxT("Inserting duplicate animation handler for '%d' type"),
-                    handler->GetType() );
-        delete handler;
-    }
-}
-
-const wxAnimationDecoder *wxAnimation::FindHandler( wxAnimationType animType )
-{
-    wxAnimationDecoderList::compatibility_iterator node = sm_handlers.GetFirst();
-    while (node)
-    {
-        const wxAnimationDecoder *handler = (const wxAnimationDecoder *)node->GetData();
-        if (handler->GetType() == animType) return handler;
-        node = node->GetNext();
-    }
-    return 0;
-}
-
-void wxAnimation::InitStandardHandlers()
-{
-#if wxUSE_GIF
-    AddHandler(new wxGIFDecoder);
-#endif // wxUSE_GIF
-#if wxUSE_ICO_CUR
-    AddHandler(new wxANIDecoder);
-#endif // wxUSE_ICO_CUR
-}
-
-void wxAnimation::CleanUpHandlers()
-{
-    wxAnimationDecoderList::compatibility_iterator node = sm_handlers.GetFirst();
-    while (node)
-    {
-        wxAnimationDecoder *handler = (wxAnimationDecoder *)node->GetData();
-        wxAnimationDecoderList::compatibility_iterator next = node->GetNext();
-        delete handler;
-        node = next;
-    }
-
-    sm_handlers.Clear();
-}
-
-
-// A module to allow wxAnimation initialization/cleanup
-// without calling these functions from app.cpp or from
-// the user's application.
-
-class wxAnimationModule: public wxModule
-{
-    wxDECLARE_DYNAMIC_CLASS(wxAnimationModule);
-public:
-    wxAnimationModule() {}
-    bool OnInit() wxOVERRIDE { wxAnimation::InitStandardHandlers(); return true; }
-    void OnExit() wxOVERRIDE { wxAnimation::CleanUpHandlers(); }
-};
-
-wxIMPLEMENT_DYNAMIC_CLASS(wxAnimationModule, wxModule);
-
-
 // ----------------------------------------------------------------------------
 // wxAnimationCtrl
 // ----------------------------------------------------------------------------
 
-wxIMPLEMENT_CLASS(wxAnimationCtrl, wxAnimationCtrlBase);
-wxBEGIN_EVENT_TABLE(wxAnimationCtrl, wxAnimationCtrlBase)
-    EVT_PAINT(wxAnimationCtrl::OnPaint)
-    EVT_SIZE(wxAnimationCtrl::OnSize)
-    EVT_TIMER(wxID_ANY, wxAnimationCtrl::OnTimer)
+wxIMPLEMENT_CLASS(wxGenericAnimationCtrl, wxAnimationCtrlBase);
+wxBEGIN_EVENT_TABLE(wxGenericAnimationCtrl, wxAnimationCtrlBase)
+    EVT_PAINT(wxGenericAnimationCtrl::OnPaint)
+    EVT_SIZE(wxGenericAnimationCtrl::OnSize)
+    EVT_TIMER(wxID_ANY, wxGenericAnimationCtrl::OnTimer)
 wxEND_EVENT_TABLE()
 
-void wxAnimationCtrl::Init()
+void wxGenericAnimationCtrl::Init()
 {
     m_currentFrame = 0;
     m_looped = false;
@@ -282,7 +180,7 @@ void wxAnimationCtrl::Init()
     m_useWinBackgroundColour = true;
 }
 
-bool wxAnimationCtrl::Create(wxWindow *parent, wxWindowID id,
+bool wxGenericAnimationCtrl::Create(wxWindow *parent, wxWindowID id,
             const wxAnimation& animation, const wxPoint& pos,
             const wxSize& size, long style, const wxString& name)
 {
@@ -299,12 +197,13 @@ bool wxAnimationCtrl::Create(wxWindow *parent, wxWindowID id,
     return true;
 }
 
-wxAnimationCtrl::~wxAnimationCtrl()
+wxGenericAnimationCtrl::~wxGenericAnimationCtrl()
 {
-    Stop();
+    if (IsPlaying())
+        Stop();
 }
 
-bool wxAnimationCtrl::LoadFile(const wxString& filename, wxAnimationType type)
+bool wxGenericAnimationCtrl::LoadFile(const wxString& filename, wxAnimationType type)
 {
     wxFileInputStream fis(filename);
     if (!fis.IsOk())
@@ -312,9 +211,9 @@ bool wxAnimationCtrl::LoadFile(const wxString& filename, wxAnimationType type)
     return Load(fis, type);
 }
 
-bool wxAnimationCtrl::Load(wxInputStream& stream, wxAnimationType type)
+bool wxGenericAnimationCtrl::Load(wxInputStream& stream, wxAnimationType type)
 {
-    wxAnimation anim;
+    wxAnimation anim(CreateAnimation());
     if ( !anim.Load(stream, type) || !anim.IsOk() )
         return false;
 
@@ -322,15 +221,25 @@ bool wxAnimationCtrl::Load(wxInputStream& stream, wxAnimationType type)
     return true;
 }
 
-wxSize wxAnimationCtrl::DoGetBestSize() const
+wxAnimation wxGenericAnimationCtrl::CreateCompatibleAnimation()
+{
+    return MakeAnimFromImpl(new wxAnimationGenericImpl());
+}
+
+wxAnimationImpl* wxGenericAnimationCtrl::DoCreateAnimationImpl() const
+{
+    return new wxAnimationGenericImpl();
+}
+
+wxSize wxGenericAnimationCtrl::DoGetBestSize() const
 {
     if (m_animation.IsOk() && !this->HasFlag(wxAC_NO_AUTORESIZE))
         return m_animation.GetSize();
 
-    return wxSize(100, 100);
+    return FromDIP(wxSize(100, 100));
 }
 
-void wxAnimationCtrl::SetAnimation(const wxAnimation& animation)
+void wxGenericAnimationCtrl::SetAnimation(const wxAnimation& animation)
 {
     if (IsPlaying())
         Stop();
@@ -343,7 +252,10 @@ void wxAnimationCtrl::SetAnimation(const wxAnimation& animation)
         return;
     }
 
-    if (m_animation.GetBackgroundColour() == wxNullColour)
+    wxCHECK_RET(animation.IsCompatibleWith(GetClassInfo()),
+                wxT("incompatible animation") );
+
+    if (AnimationImplGetBackgroundColour() == wxNullColour)
         SetUseWindowBackgroundColour();
     if (!this->HasFlag(wxAC_NO_AUTORESIZE))
         FitToAnimation();
@@ -351,7 +263,7 @@ void wxAnimationCtrl::SetAnimation(const wxAnimation& animation)
     DisplayStaticImage();
 }
 
-void wxAnimationCtrl::SetInactiveBitmap(const wxBitmap &bmp)
+void wxGenericAnimationCtrl::SetInactiveBitmap(const wxBitmap &bmp)
 {
     // if the bitmap has an associated mask, we need to set our background to
     // the colour of our parent otherwise when calling DrawCurrentFrame()
@@ -364,12 +276,12 @@ void wxAnimationCtrl::SetInactiveBitmap(const wxBitmap &bmp)
     wxAnimationCtrlBase::SetInactiveBitmap(bmp);
 }
 
-void wxAnimationCtrl::FitToAnimation()
+void wxGenericAnimationCtrl::FitToAnimation()
 {
     SetSize(m_animation.GetSize());
 }
 
-bool wxAnimationCtrl::SetBackgroundColour(const wxColour& colour)
+bool wxGenericAnimationCtrl::SetBackgroundColour(const wxColour& colour)
 {
     if ( !wxWindow::SetBackgroundColour(colour) )
         return false;
@@ -387,7 +299,7 @@ bool wxAnimationCtrl::SetBackgroundColour(const wxColour& colour)
 // wxAnimationCtrl - stop/play methods
 // ----------------------------------------------------------------------------
 
-void wxAnimationCtrl::Stop()
+void wxGenericAnimationCtrl::Stop()
 {
     m_timer.Stop();
     m_isPlaying = false;
@@ -398,7 +310,7 @@ void wxAnimationCtrl::Stop()
     DisplayStaticImage();
 }
 
-bool wxAnimationCtrl::Play(bool looped)
+bool wxGenericAnimationCtrl::Play(bool looped)
 {
     if (!m_animation.IsOk())
         return false;
@@ -434,7 +346,7 @@ bool wxAnimationCtrl::Play(bool looped)
 // wxAnimationCtrl - rendering methods
 // ----------------------------------------------------------------------------
 
-bool wxAnimationCtrl::RebuildBackingStoreUpToFrame(unsigned int frame)
+bool wxGenericAnimationCtrl::RebuildBackingStoreUpToFrame(unsigned int frame)
 {
     // if we've not created the backing store yet or it's too
     // small, then recreate it
@@ -459,14 +371,14 @@ bool wxAnimationCtrl::RebuildBackingStoreUpToFrame(unsigned int frame)
     // Draw all intermediate frames that haven't been removed from the animation
     for (unsigned int i = 0; i < frame; i++)
     {
-        if (m_animation.GetDisposalMethod(i) == wxANIM_DONOTREMOVE ||
-            m_animation.GetDisposalMethod(i) == wxANIM_UNSPECIFIED)
+        if (AnimationImplGetDisposalMethod(i) == wxANIM_DONOTREMOVE ||
+            AnimationImplGetDisposalMethod(i) == wxANIM_UNSPECIFIED)
         {
             DrawFrame(dc, i);
         }
-        else if (m_animation.GetDisposalMethod(i) == wxANIM_TOBACKGROUND)
-            DisposeToBackground(dc, m_animation.GetFramePosition(i),
-                                    m_animation.GetFrameSize(i));
+        else if (AnimationImplGetDisposalMethod(i) == wxANIM_TOBACKGROUND)
+            DisposeToBackground(dc, AnimationImplGetFramePosition(i),
+                                    AnimationImplGetFrameSize(i));
     }
 
     // finally draw this frame
@@ -476,7 +388,7 @@ bool wxAnimationCtrl::RebuildBackingStoreUpToFrame(unsigned int frame)
     return true;
 }
 
-void wxAnimationCtrl::IncrementalUpdateBackingStore()
+void wxGenericAnimationCtrl::IncrementalUpdateBackingStore()
 {
     wxMemoryDC dc;
     dc.SelectObject(m_backingStore);
@@ -494,11 +406,11 @@ void wxAnimationCtrl::IncrementalUpdateBackingStore()
     }
     else
     {
-        switch (m_animation.GetDisposalMethod(m_currentFrame-1))
+        switch (AnimationImplGetDisposalMethod(m_currentFrame-1))
         {
         case wxANIM_TOBACKGROUND:
-            DisposeToBackground(dc, m_animation.GetFramePosition(m_currentFrame-1),
-                                    m_animation.GetFrameSize(m_currentFrame-1));
+            DisposeToBackground(dc, AnimationImplGetFramePosition(m_currentFrame-1),
+                                    AnimationImplGetFrameSize(m_currentFrame-1));
             break;
 
         case wxANIM_TOPREVIOUS:
@@ -528,7 +440,7 @@ void wxAnimationCtrl::IncrementalUpdateBackingStore()
     dc.SelectObject(wxNullBitmap);
 }
 
-void wxAnimationCtrl::DisplayStaticImage()
+void wxGenericAnimationCtrl::DisplayStaticImage()
 {
     wxASSERT(!IsPlaying());
 
@@ -538,8 +450,9 @@ void wxAnimationCtrl::DisplayStaticImage()
     if (m_bmpStaticReal.IsOk())
     {
         // copy the inactive bitmap in the backing store
-        // eventually using the mask if the static bitmap has one
-        if ( m_bmpStaticReal.GetMask() )
+        // eventually using the mask or the alpha if the static
+        // bitmap has one
+        if ( m_bmpStaticReal.GetMask() || m_bmpStaticReal.HasAlpha() )
         {
             wxMemoryDC temp;
             temp.SelectObject(m_backingStore);
@@ -563,7 +476,7 @@ void wxAnimationCtrl::DisplayStaticImage()
     Refresh();
 }
 
-void wxAnimationCtrl::DrawFrame(wxDC &dc, unsigned int frame)
+void wxGenericAnimationCtrl::DrawFrame(wxDC &dc, unsigned int frame)
 {
     // PERFORMANCE NOTE:
     // this draw stuff is not as fast as possible: the wxAnimationDecoder
@@ -572,11 +485,11 @@ void wxAnimationCtrl::DrawFrame(wxDC &dc, unsigned int frame)
     // If wxAnimationDecoder had a function to convert directly from its
     // internal format to a port-specific wxBitmap, it would be somewhat faster.
     wxBitmap bmp(m_animation.GetFrame(frame));
-    dc.DrawBitmap(bmp, m_animation.GetFramePosition(frame),
+    dc.DrawBitmap(bmp, AnimationImplGetFramePosition(frame),
                   true /* use mask */);
 }
 
-void wxAnimationCtrl::DrawCurrentFrame(wxDC& dc)
+void wxGenericAnimationCtrl::DrawCurrentFrame(wxDC& dc)
 {
     wxASSERT( m_backingStore.IsOk() );
 
@@ -584,7 +497,7 @@ void wxAnimationCtrl::DrawCurrentFrame(wxDC& dc)
     dc.DrawBitmap(m_backingStore, 0, 0, true /* use mask in case it's present */);
 }
 
-void wxAnimationCtrl::DisposeToBackground()
+void wxGenericAnimationCtrl::DisposeToBackground()
 {
     // clear the backing store
     wxMemoryDC dc;
@@ -593,22 +506,22 @@ void wxAnimationCtrl::DisposeToBackground()
         DisposeToBackground(dc);
 }
 
-void wxAnimationCtrl::DisposeToBackground(wxDC& dc)
+void wxGenericAnimationCtrl::DisposeToBackground(wxDC& dc)
 {
     wxColour col = IsUsingWindowBackgroundColour()
                     ? GetBackgroundColour()
-                    : m_animation.GetBackgroundColour();
+                    : AnimationImplGetBackgroundColour();
 
     wxBrush brush(col);
     dc.SetBackground(brush);
     dc.Clear();
 }
 
-void wxAnimationCtrl::DisposeToBackground(wxDC& dc, const wxPoint &pos, const wxSize &sz)
+void wxGenericAnimationCtrl::DisposeToBackground(wxDC& dc, const wxPoint &pos, const wxSize &sz)
 {
     wxColour col = IsUsingWindowBackgroundColour()
                     ? GetBackgroundColour()
-                    : m_animation.GetBackgroundColour();
+                    : AnimationImplGetBackgroundColour();
     wxBrush brush(col);
     dc.SetBrush(brush);         // SetBrush and not SetBackground !!
     dc.SetPen(*wxTRANSPARENT_PEN);
@@ -619,7 +532,7 @@ void wxAnimationCtrl::DisposeToBackground(wxDC& dc, const wxPoint &pos, const wx
 // wxAnimationCtrl - event handlers
 // ----------------------------------------------------------------------------
 
-void wxAnimationCtrl::OnPaint(wxPaintEvent& WXUNUSED(event))
+void wxGenericAnimationCtrl::OnPaint(wxPaintEvent& WXUNUSED(event))
 {
     // VERY IMPORTANT: the wxPaintDC *must* be created in any case
     wxPaintDC dc(this);
@@ -639,7 +552,7 @@ void wxAnimationCtrl::OnPaint(wxPaintEvent& WXUNUSED(event))
     }
 }
 
-void wxAnimationCtrl::OnTimer(wxTimerEvent &WXUNUSED(event))
+void wxGenericAnimationCtrl::OnTimer(wxTimerEvent &WXUNUSED(event))
 {
     m_currentFrame++;
     if (m_currentFrame == m_animation.GetFrameCount())
@@ -671,7 +584,7 @@ void wxAnimationCtrl::OnTimer(wxTimerEvent &WXUNUSED(event))
     m_timer.Start(delay, true);
 }
 
-void wxAnimationCtrl::OnSize(wxSizeEvent &WXUNUSED(event))
+void wxGenericAnimationCtrl::OnSize(wxSizeEvent &WXUNUSED(event))
 {
     // NB: resizing an animation control may take a lot of time
     //     for big animations as the backing store must be
@@ -692,5 +605,39 @@ void wxAnimationCtrl::OnSize(wxSizeEvent &WXUNUSED(event))
     }
 }
 
-#endif // wxUSE_ANIMATIONCTRL
+// ----------------------------------------------------------------------------
+// helpers to safely access wxAnimationGenericImpl methods
+// ----------------------------------------------------------------------------
+#define ANIMATION (static_cast<wxAnimationGenericImpl*>(GetAnimImpl()))
 
+wxPoint wxGenericAnimationCtrl::AnimationImplGetFramePosition(unsigned int frame) const
+{
+    wxCHECK_MSG( m_animation.IsOk(), wxDefaultPosition, wxT("invalid animation") );
+    return ANIMATION->GetFramePosition(frame);
+}
+
+wxSize wxGenericAnimationCtrl::AnimationImplGetFrameSize(unsigned int frame) const
+{
+    wxCHECK_MSG( m_animation.IsOk(), wxDefaultSize, wxT("invalid animation") );
+    return ANIMATION->GetFrameSize(frame);
+}
+
+wxAnimationDisposal wxGenericAnimationCtrl::AnimationImplGetDisposalMethod(unsigned int frame) const
+{
+    wxCHECK_MSG( m_animation.IsOk(), wxANIM_UNSPECIFIED, wxT("invalid animation") );
+    return ANIMATION->GetDisposalMethod(frame);
+}
+
+wxColour wxGenericAnimationCtrl::AnimationImplGetTransparentColour(unsigned int frame) const
+{
+    wxCHECK_MSG( m_animation.IsOk(), wxNullColour, wxT("invalid animation") );
+    return ANIMATION->GetTransparentColour(frame);
+}
+
+wxColour wxGenericAnimationCtrl::AnimationImplGetBackgroundColour() const
+{
+    wxCHECK_MSG( m_animation.IsOk(), wxNullColour, wxT("invalid animation") );
+    return ANIMATION->GetBackgroundColour();
+}
+
+#endif // wxUSE_ANIMATIONCTRL
