@@ -16,97 +16,39 @@
 #include "wx/glcanvas.h"
 
 #include "wx/gtk/private/wrapgtk.h"
+#ifdef GDK_WINDOWING_WAYLAND
+#include <gdk/gdkwayland.h>
+#endif
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
-
-#if WXWIN_COMPATIBILITY_2_8
-
-//-----------------------------------------------------------------------------
-// "realize" from m_wxwindow: used to create m_glContext implicitly
-//-----------------------------------------------------------------------------
-
-extern "C" {
-static void
-gtk_glwindow_realized_callback( GtkWidget *WXUNUSED(widget), wxGLCanvas *win )
-{
-    win->GTKInitImplicitContext();
-}
-}
-
-#endif // WXWIN_COMPATIBILITY_2_8
-
-//-----------------------------------------------------------------------------
-// "map" from m_wxwindow
-//-----------------------------------------------------------------------------
-
-#ifndef __WXGTK3__
-extern "C" {
-static void
-gtk_glwindow_map_callback( GtkWidget * WXUNUSED(widget), wxGLCanvas *win )
-{
-    wxPaintEvent event( win->GetId() );
-    event.SetEventObject( win );
-    win->HandleWindowEvent( event );
-
-    win->m_exposed = false;
-    win->GetUpdateRegion().Clear();
-}
-}
 #endif
 
-//-----------------------------------------------------------------------------
-// "expose_event" of m_wxwindow
-//-----------------------------------------------------------------------------
-
-extern "C" {
 #ifdef __WXGTK3__
-static gboolean draw(GtkWidget*, cairo_t* cr, wxGLCanvas* win)
-{
-    win->m_exposed = true;
-    if (win->m_cairoPaintContext == NULL)
-    {
-        win->m_cairoPaintContext = cr;
-        cairo_reference(cr);
-    }
-    double x1, y1, x2, y2;
-    cairo_clip_extents(cr, &x1, &y1, &x2, &y2);
-    win->GetUpdateRegion().Union(int(x1), int(y1), int(x2 - x1), int(y2 - y1));
-    return false;
-}
-#else
-static gboolean
-gtk_glwindow_expose_callback( GtkWidget *WXUNUSED(widget), GdkEventExpose *gdk_event, wxGLCanvas *win )
-{
-    win->m_exposed = true;
-
-    win->GetUpdateRegion().Union( gdk_event->area.x,
-                                  gdk_event->area.y,
-                                  gdk_event->area.width,
-                                  gdk_event->area.height );
-    return false;
-}
-#endif
-}
-
-//-----------------------------------------------------------------------------
-// "size_allocate" of m_wxwindow
-//-----------------------------------------------------------------------------
-
 extern "C" {
-static void
-gtk_glcanvas_size_callback(GtkWidget *WXUNUSED(widget),
-                           GtkAllocation * WXUNUSED(alloc),
-                           wxGLCanvas *win)
+static gboolean draw(GtkWidget* widget, cairo_t* cr, wxGLCanvas* win)
 {
-    wxSizeEvent event( wxSize(win->m_width,win->m_height), win->GetId() );
-    event.SetEventObject( win );
-    win->HandleWindowEvent( event );
+    GtkAllocation a;
+    gtk_widget_get_allocation(widget, &a);
+    if (a.width > win->m_size.x || a.height > win->m_size.y)
+    {
+        // GLX buffers are apparently not reliably updated to the new size
+        // before the paint event occurs, resulting in newly exposed window
+        // areas sometimes not being painted at the end of a drag resize.
+        gdk_display_sync(gtk_widget_get_display(widget));
+    }
+    win->m_size.Set(a.width, a.height);
+
+    win->GTKSendPaintEvents(cr);
+    return false;
 }
 }
+#endif // __WXGTK3__
 
 //-----------------------------------------------------------------------------
 // emission hook for "parent-set"
 //-----------------------------------------------------------------------------
 
+#if !wxUSE_GLCANVAS_EGL
 extern "C" {
 static gboolean
 parent_set_hook(GSignalInvocationHint*, guint, const GValue* param_values, void* data)
@@ -114,7 +56,7 @@ parent_set_hook(GSignalInvocationHint*, guint, const GValue* param_values, void*
     wxGLCanvas* win = (wxGLCanvas*)data;
     if (g_value_peek_pointer(&param_values[0]) == win->m_wxwindow)
     {
-        const XVisualInfo* xvi = win->GetXVisualInfo();
+        const XVisualInfo* xvi = static_cast<XVisualInfo*>(win->GetXVisualInfo());
         GdkVisual* visual = gtk_widget_get_visual(win->m_wxwindow);
         if (GDK_VISUAL_XVISUAL(visual)->visualid != xvi->visualid)
         {
@@ -134,6 +76,7 @@ parent_set_hook(GSignalInvocationHint*, guint, const GValue* param_values, void*
     return true;
 }
 }
+#endif
 
 //---------------------------------------------------------------------------
 // wxGlCanvas
@@ -224,15 +167,34 @@ wxGLCanvas::wxGLCanvas(wxWindow *parent,
 
 static bool IsAvailable()
 {
+#if defined(__WXGTK3__) && (defined(GDK_WINDOWING_WAYLAND) || defined(GDK_WINDOWING_X11))
+    GdkDisplay* display = gdk_display_get_default();
+    const char* displayTypeName = g_type_name(G_TYPE_FROM_INSTANCE(display));
+#endif
+
+#ifdef GDK_WINDOWING_WAYLAND
+    if (strcmp("GdkWaylandDisplay", displayTypeName) == 0)
+    {
+#if wxUSE_GLCANVAS_EGL
+        return true;
+#else
+        wxSafeShowMessage(_("Fatal Error"), _("This program wasn't compiled with EGL support required under Wayland, either\ninstall EGL libraries and rebuild or run it under X11 backend by setting\nenvironment variable GDK_BACKEND=x11 before starting your program."));
+        return false;
+#endif // wxUSE_GLCANVAS_EGL
+    }
+#endif // GDK_WINDOWING_WAYLAND
+
 #ifdef GDK_WINDOWING_X11
-    if ( !GDK_IS_X11_DISPLAY(gdk_display_get_default()) )
+#ifdef __WXGTK3__
+    if (strcmp("GdkX11Display", displayTypeName) == 0)
 #endif
     {
-        wxSafeShowMessage(_("Fatal Error"), _("wxGLCanvas is only supported on X11 currently.  You may be able to\nwork around this by setting environment variable GDK_BACKEND=x11 before starting\nyour program."));
-        return false;
+        return true;
     }
+#endif
 
-    return true;
+    wxSafeShowMessage(_("Fatal Error"), _("wxGLCanvas is only supported on Wayland and X11 currently.  You may be able to\nwork around this by setting environment variable GDK_BACKEND=x11 before\nstarting your program."));
+    return false;
 }
 
 bool wxGLCanvas::Create(wxWindow *parent,
@@ -273,11 +235,9 @@ bool wxGLCanvas::Create(wxWindow *parent,
 #endif // wxUSE_PALETTE
     wxUnusedVar(palette); // Unused when wxDEBUG_LEVEL==0
 
-    m_exposed = false;
-    m_noExpose = true;
     m_nativeSizeEvent = true;
 #ifdef __WXGTK3__
-    m_cairoPaintContext = NULL;
+    m_noExpose = true;
     m_backgroundStyle = wxBG_STYLE_PAINT;
 #endif
 
@@ -287,36 +247,17 @@ bool wxGLCanvas::Create(wxWindow *parent,
     // watch for the "parent-set" signal on m_wxwindow so we can set colormap
     // before m_wxwindow is realized (which will occur before
     // wxWindow::Create() returns if parent is already visible)
+#if !wxUSE_GLCANVAS_EGL
     unsigned sig_id = g_signal_lookup("parent-set", GTK_TYPE_WIDGET);
     g_signal_add_emission_hook(sig_id, 0, parent_set_hook, this, NULL);
+#endif
 
     wxWindow::Create( parent, id, pos, size, style, name );
-
-    gtk_widget_set_double_buffered(m_wxwindow, false);
-
-#if WXWIN_COMPATIBILITY_2_8
-    g_signal_connect(m_wxwindow, "realize",       G_CALLBACK(gtk_glwindow_realized_callback), this);
-#endif // WXWIN_COMPATIBILITY_2_8
 #ifdef __WXGTK3__
     g_signal_connect(m_wxwindow, "draw", G_CALLBACK(draw), this);
-#else
-    g_signal_connect(m_wxwindow, "map",           G_CALLBACK(gtk_glwindow_map_callback),      this);
-    g_signal_connect(m_wxwindow, "expose_event",  G_CALLBACK(gtk_glwindow_expose_callback),   this);
 #endif
-    g_signal_connect(m_widget,   "size_allocate", G_CALLBACK(gtk_glcanvas_size_callback),     this);
 
-#if WXWIN_COMPATIBILITY_2_8
-    // if our parent window is already visible, we had been realized before we
-    // connected to the "realize" signal and hence our m_glContext hasn't been
-    // initialized yet and we have to do it now
-    if (gtk_widget_get_realized(m_wxwindow))
-        gtk_glwindow_realized_callback( m_wxwindow, this );
-#endif // WXWIN_COMPATIBILITY_2_8
-
-#ifndef __WXGTK3__
-    if (gtk_widget_get_mapped(m_wxwindow))
-        gtk_glwindow_map_callback( m_wxwindow, this );
-#endif
+    gtk_widget_set_double_buffered(m_wxwindow, false);
 
     return true;
 }
@@ -326,31 +267,27 @@ bool wxGLCanvas::SetBackgroundStyle(wxBackgroundStyle /* style */)
     return false;
 }
 
-Window wxGLCanvas::GetXWindow() const
+unsigned long wxGLCanvas::GetXWindow() const
 {
+#if defined(GDK_WINDOWING_X11)
     GdkWindow* window = GTKGetDrawingWindow();
     return window ? GDK_WINDOW_XID(window) : 0;
+#else
+    return 0;
+#endif
 }
 
-void wxGLCanvas::OnInternalIdle()
+void wxGLCanvas::GTKHandleRealized()
 {
-    if (m_exposed)
-    {
-#ifdef __WXGTK3__
-        GTKSendPaintEvents(m_cairoPaintContext);
-        cairo_destroy(m_cairoPaintContext);
-        m_cairoPaintContext = NULL;
-#else
-        wxPaintEvent event( GetId() );
-        event.SetEventObject( this );
-        HandleWindowEvent( event );
+    BaseType::GTKHandleRealized();
+
+#if WXWIN_COMPATIBILITY_2_8
+    GTKInitImplicitContext();
 #endif
-
-        m_exposed = false;
-        GetUpdateRegion().Clear();
-    }
-
-    wxWindow::OnInternalIdle();
+#if wxUSE_GLCANVAS_EGL
+    CreateSurface();
+#endif
+    SendSizeEvent();
 }
 
 #if WXWIN_COMPATIBILITY_2_8
