@@ -19,48 +19,23 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#if defined(__BORLANDC__)
-    #pragma hdrstop
-#endif
 
 #if wxUSE_OLE
 
-#ifndef WX_PRECOMP
-    #include "wx/log.h"
-#endif
-
 #ifndef __CYGWIN10__
 
 #include "wx/msw/private.h"
 
-// OLE
-#include  "wx/msw/ole/uuid.h"
-
 #include  "wx/msw/ole/oleutils.h"
 #include "wx/msw/ole/safearray.h"
 
-#if defined(__VISUALC__)
-    #include  <docobj.h>
-#endif
-
 // ============================================================================
 // Implementation
 // ============================================================================
 
-// return true if the iid is in the array
-WXDLLEXPORT bool IsIidFromList(REFIID riid, const IID *aIids[], size_t nCount)
-{
-  for ( size_t i = 0; i < nCount; i++ ) {
-    if ( riid == *aIids[i] )
-      return true;
-  }
-
-  return false;
-}
-
 WXDLLEXPORT BSTR wxConvertStringToOle(const wxString& str)
 {
-    return wxBasicString(str).Get();
+    return wxBasicString(str).Detach();
 }
 
 WXDLLEXPORT wxString wxConvertStringFromOle(BSTR bStr)
@@ -92,29 +67,41 @@ WXDLLEXPORT wxString wxConvertStringFromOle(BSTR bStr)
 // ----------------------------------------------------------------------------
 // wxBasicString
 // ----------------------------------------------------------------------------
-
-wxBasicString::wxBasicString(const wxString& str)
+void wxBasicString::AssignFromString(const wxString& str)
 {
+    SysFreeString(m_bstrBuf);
     m_bstrBuf = SysAllocString(str.wc_str(*wxConvCurrent));
 }
 
-wxBasicString::wxBasicString(const wxBasicString& src)
+BSTR wxBasicString::Detach()
 {
-    m_bstrBuf = src.Get();
+    BSTR bstr = m_bstrBuf;
+
+    m_bstrBuf = NULL;
+
+    return bstr;
+}
+
+BSTR* wxBasicString::ByRef()
+{
+    wxASSERT_MSG(!m_bstrBuf,
+        wxS("Can't get direct access to initialized BSTR"));
+    return &m_bstrBuf;
 }
 
 wxBasicString& wxBasicString::operator=(const wxBasicString& src)
 {
-    SysReAllocString(&m_bstrBuf, src);
+    if ( this != &src )
+    {
+        wxCHECK_MSG(m_bstrBuf == NULL || m_bstrBuf != src.m_bstrBuf,
+            *this, wxS("Attempting to assign already owned BSTR"));
+        SysFreeString(m_bstrBuf);
+        m_bstrBuf = src.Copy();
+    }
+
     return *this;
 }
 
-wxBasicString::~wxBasicString()
-{
-    SysFreeString(m_bstrBuf);
-}
-
-
 // ----------------------------------------------------------------------------
 // Convert variants
 // ----------------------------------------------------------------------------
@@ -393,10 +380,6 @@ WXDLLEXPORT bool wxConvertVariantToOle(const wxVariant& variant, VARIANTARG& ole
     return true;
 }
 
-#ifndef VT_TYPEMASK
-#define VT_TYPEMASK 0xfff
-#endif
-
 WXDLLEXPORT bool
 wxConvertOleToVariant(const VARIANTARG& oleVariant, wxVariant& variant, long flags)
 {
@@ -504,10 +487,12 @@ wxConvertOleToVariant(const VARIANTARG& oleVariant, wxVariant& variant, long fla
 #endif // wxUSE_LONGLONG
 
             case VT_I4:
+            case VT_UI4:
                 variant = (long) oleVariant.lVal;
                 break;
 
             case VT_I2:
+            case VT_UI2:
                 variant = (long) oleVariant.iVal;
                 break;
 
@@ -544,141 +529,6 @@ wxConvertOleToVariant(const VARIANTARG& oleVariant, wxVariant& variant, long fla
 
 #endif // wxUSE_VARIANT
 
-
-// ----------------------------------------------------------------------------
-// Debug support
-// ----------------------------------------------------------------------------
-
-#if wxUSE_DATAOBJ
-
-#if wxDEBUG_LEVEL && defined(__VISUALC__)
-static wxString GetIidName(REFIID riid)
-{
-  // an association between symbolic name and numeric value of an IID
-  struct KNOWN_IID {
-    const IID  *pIid;
-    const wxChar *szName;
-  };
-
-  // construct the table containing all known interfaces
-  #define ADD_KNOWN_IID(name) { &IID_I##name, wxT(#name) }
-
-  static const KNOWN_IID aKnownIids[] = {
-    ADD_KNOWN_IID(AdviseSink),
-    ADD_KNOWN_IID(AdviseSink2),
-    ADD_KNOWN_IID(BindCtx),
-    ADD_KNOWN_IID(ClassFactory),
-#if ( !defined( __VISUALC__) || (__VISUALC__!=1010) )
-    ADD_KNOWN_IID(ContinueCallback),
-    ADD_KNOWN_IID(EnumOleDocumentViews),
-    ADD_KNOWN_IID(OleCommandTarget),
-    ADD_KNOWN_IID(OleDocument),
-    ADD_KNOWN_IID(OleDocumentSite),
-    ADD_KNOWN_IID(OleDocumentView),
-    ADD_KNOWN_IID(Print),
-#endif
-    ADD_KNOWN_IID(DataAdviseHolder),
-    ADD_KNOWN_IID(DataObject),
-    ADD_KNOWN_IID(Debug),
-    ADD_KNOWN_IID(DebugStream),
-    ADD_KNOWN_IID(DfReserved1),
-    ADD_KNOWN_IID(DfReserved2),
-    ADD_KNOWN_IID(DfReserved3),
-    ADD_KNOWN_IID(Dispatch),
-    ADD_KNOWN_IID(DropSource),
-    ADD_KNOWN_IID(DropTarget),
-    ADD_KNOWN_IID(EnumCallback),
-    ADD_KNOWN_IID(EnumFORMATETC),
-    ADD_KNOWN_IID(EnumGeneric),
-    ADD_KNOWN_IID(EnumHolder),
-    ADD_KNOWN_IID(EnumMoniker),
-    ADD_KNOWN_IID(EnumOLEVERB),
-    ADD_KNOWN_IID(EnumSTATDATA),
-    ADD_KNOWN_IID(EnumSTATSTG),
-    ADD_KNOWN_IID(EnumString),
-    ADD_KNOWN_IID(EnumUnknown),
-    ADD_KNOWN_IID(EnumVARIANT),
-    ADD_KNOWN_IID(ExternalConnection),
-    ADD_KNOWN_IID(InternalMoniker),
-    ADD_KNOWN_IID(LockBytes),
-    ADD_KNOWN_IID(Malloc),
-    ADD_KNOWN_IID(Marshal),
-    ADD_KNOWN_IID(MessageFilter),
-    ADD_KNOWN_IID(Moniker),
-    ADD_KNOWN_IID(OleAdviseHolder),
-    ADD_KNOWN_IID(OleCache),
-    ADD_KNOWN_IID(OleCache2),
-    ADD_KNOWN_IID(OleCacheControl),
-    ADD_KNOWN_IID(OleClientSite),
-    ADD_KNOWN_IID(OleContainer),
-    ADD_KNOWN_IID(OleInPlaceActiveObject),
-    ADD_KNOWN_IID(OleInPlaceFrame),
-    ADD_KNOWN_IID(OleInPlaceObject),
-    ADD_KNOWN_IID(OleInPlaceSite),
-    ADD_KNOWN_IID(OleInPlaceUIWindow),
-    ADD_KNOWN_IID(OleItemContainer),
-    ADD_KNOWN_IID(OleLink),
-    ADD_KNOWN_IID(OleManager),
-    ADD_KNOWN_IID(OleObject),
-    ADD_KNOWN_IID(OlePresObj),
-    ADD_KNOWN_IID(OleWindow),
-    ADD_KNOWN_IID(PSFactory),
-    ADD_KNOWN_IID(ParseDisplayName),
-    ADD_KNOWN_IID(Persist),
-    ADD_KNOWN_IID(PersistFile),
-    ADD_KNOWN_IID(PersistStorage),
-    ADD_KNOWN_IID(PersistStream),
-    ADD_KNOWN_IID(ProxyManager),
-    ADD_KNOWN_IID(RootStorage),
-    ADD_KNOWN_IID(RpcChannel),
-    ADD_KNOWN_IID(RpcProxy),
-    ADD_KNOWN_IID(RpcStub),
-    ADD_KNOWN_IID(RunnableObject),
-    ADD_KNOWN_IID(RunningObjectTable),
-    ADD_KNOWN_IID(StdMarshalInfo),
-    ADD_KNOWN_IID(Storage),
-    ADD_KNOWN_IID(Stream),
-    ADD_KNOWN_IID(StubManager),
-    ADD_KNOWN_IID(Unknown),
-    ADD_KNOWN_IID(ViewObject),
-    ADD_KNOWN_IID(ViewObject2),
-  };
-
-  // don't clobber preprocessor name space
-  #undef ADD_KNOWN_IID
-
-  // try to find the interface in the table
-  for ( size_t ui = 0; ui < WXSIZEOF(aKnownIids); ui++ ) {
-    if ( riid == *aKnownIids[ui].pIid ) {
-      return aKnownIids[ui].szName;
-    }
-  }
-
-  // unknown IID, just transform to string
-  Uuid uuid(riid);
-  return wxString((const wxChar *)uuid);
-}
-
-WXDLLEXPORT void wxLogQueryInterface(const wxChar *szInterface, REFIID riid)
-{
-  wxLogTrace(wxTRACE_OleCalls, wxT("%s::QueryInterface (iid = %s)"),
-             szInterface, GetIidName(riid).c_str());
-}
-
-WXDLLEXPORT void wxLogAddRef(const wxChar *szInterface, ULONG cRef)
-{
-  wxLogTrace(wxTRACE_OleCalls, wxT("After %s::AddRef: m_cRef = %d"), szInterface, cRef + 1);
-}
-
-WXDLLEXPORT void wxLogRelease(const wxChar *szInterface, ULONG cRef)
-{
-  wxLogTrace(wxTRACE_OleCalls, wxT("After %s::Release: m_cRef = %d"), szInterface, cRef - 1);
-}
-
-#endif  // wxDEBUG_LEVEL
-
-#endif // wxUSE_DATAOBJ
-
 #endif // __CYGWIN10__
 
 #endif // wxUSE_OLE
