@@ -19,9 +19,6 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_STATBOX
 
@@ -39,16 +36,32 @@
 #include "wx/sysopt.h"
 
 #include "wx/msw/uxtheme.h"
+
+#include <windowsx.h> // needed by GET_X_LPARAM and GET_Y_LPARAM macros
+
 #include "wx/msw/private.h"
 #include "wx/msw/missing.h"
 #include "wx/msw/dc.h"
+#include "wx/msw/private/winstyle.h"
 
-// the values coincide with those in tmschema.h
-#define BP_GROUPBOX 4
+namespace
+{
 
-#define GBS_NORMAL 1
+// Offset of the first pixel of the label from the box left border.
+//
+// FIXME: value is hardcoded as this is what it is on my system, no idea if
+//        it's true everywhere
+const int LABEL_HORZ_OFFSET = 9;
 
-#define TMT_FONT 210
+// Extra borders around the label on left/right and bottom sides.
+const int LABEL_HORZ_BORDER = 2;
+const int LABEL_VERT_BORDER = 2;
+
+// Offset of the box contents from left/right/bottom edge (top one is
+// different, see GetBordersForSizer()). This one is completely arbitrary.
+const int CHILDREN_OFFSET = 5;
+
+} // anonymous namespace
 
 // ----------------------------------------------------------------------------
 // wxWin macros
@@ -78,7 +91,7 @@ bool wxStaticBox::Create(wxWindow *parent,
 
     if (!wxSystemOptions::IsFalse(wxT("msw.staticbox.optimized-paint")))
     {
-        Connect(wxEVT_PAINT, wxPaintEventHandler(wxStaticBox::OnPaint));
+        Bind(wxEVT_PAINT, &wxStaticBox::OnPaint, this);
 
         // Our OnPaint() completely erases our background, so don't do it in
         // WM_ERASEBKGND too to avoid flicker.
@@ -88,6 +101,41 @@ bool wxStaticBox::Create(wxWindow *parent,
     return true;
 }
 
+bool wxStaticBox::Create(wxWindow* parent,
+                         wxWindowID id,
+                         wxWindow* labelWin,
+                         const wxPoint& pos,
+                         const wxSize& size,
+                         long style,
+                         const wxString& name)
+{
+    wxCHECK_MSG( labelWin, false, wxS("Label window can't be null") );
+
+    if ( !Create(parent, id, wxString(), pos, size, style, name) )
+        return false;
+
+    m_labelWin = labelWin;
+    m_labelWin->Reparent(this);
+
+    PositionLabelWindow();
+
+    return true;
+}
+
+void wxStaticBox::PositionLabelWindow()
+{
+    m_labelWin->SetSize(m_labelWin->GetBestSize());
+    m_labelWin->Move(FromDIP(LABEL_HORZ_OFFSET), 0);
+}
+
+wxWindowList wxStaticBox::GetCompositeWindowParts() const
+{
+    wxWindowList parts;
+    if ( m_labelWin )
+        parts.push_back(m_labelWin);
+    return parts;
+}
+
 WXDWORD wxStaticBox::MSWGetStyle(long style, WXDWORD *exstyle) const
 {
     long styleWin = wxStaticBoxBase::MSWGetStyle(style, exstyle);
@@ -145,10 +193,53 @@ wxSize wxStaticBox::DoGetBestSize() const
 
 void wxStaticBox::GetBordersForSizer(int *borderTop, int *borderOther) const
 {
-    wxStaticBoxBase::GetBordersForSizer(borderTop, borderOther);
+    // Base class version doesn't leave enough space at the top when the label
+    // is empty, so we can't use it here, even though the code is pretty
+    // similar.
+    if ( m_labelWin )
+    {
+        *borderTop = m_labelWin->GetSize().y;
+    }
+    else if ( !GetLabel().empty() )
+    {
+        *borderTop = GetCharHeight();
+    }
+    else // No label window nor text.
+    {
+        // This is completely arbitrary, but using the full char height in
+        // this case too seems bad as it leaves too much space at the top
+        // (although it does have the advantage of aligning the controls
+        // inside static boxes with and without labels vertically).
+        *borderTop = 2*FromDIP(CHILDREN_OFFSET);
+    }
 
-    // need extra space, don't know how much but this seems to be enough
-    *borderTop += GetCharHeight()/3;
+    *borderTop += FromDIP(LABEL_VERT_BORDER);
+
+    *borderOther = FromDIP(CHILDREN_OFFSET);
+}
+
+bool wxStaticBox::SetBackgroundColour(const wxColour& colour)
+{
+    // Do _not_ call the immediate base class method, we don't need to set the
+    // label window (which is the only sub-window of this composite window)
+    // background explicitly because it will almost always be a wxCheckBox or
+    // wxRadioButton which inherits its background from the box anyhow, so
+    // setting it would be at best useless.
+    return wxStaticBoxBase::SetBackgroundColour(colour);
+}
+
+bool wxStaticBox::SetFont(const wxFont& font)
+{
+    if ( !wxCompositeWindowSettersOnly<wxStaticBoxBase>::SetFont(font) )
+        return false;
+
+    // We need to reposition the label as its size may depend on the font.
+    if ( m_labelWin )
+    {
+        PositionLabelWindow();
+    }
+
+    return true;
 }
 
 WXLRESULT wxStaticBox::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam)
@@ -185,9 +276,10 @@ WXLRESULT wxStaticBox::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lPar
         if ( !HandlePrintClient((WXHDC)wParam) )
         {
             // no, we don't, erase the background ourselves
-            // (don't use our own) - see PaintBackground for explanation
-            wxBrush brush(GetParent()->GetBackgroundColour());
-            wxFillRect(GetHwnd(), (HDC)wParam, GetHbrushOf(brush));
+            RECT rc;
+            ::GetClientRect(GetHwnd(), &rc);
+            wxDCTemp dc((WXHDC)wParam);
+            PaintBackground(dc, rc);
         }
 
         return 0;
@@ -249,7 +341,21 @@ void wxStaticBox::MSWGetRegionWithoutSelf(WXHRGN hRgn, int w, int h)
     GetBordersForSizer(&borderTop, &border);
 
     // top
-    SubtractRectFromRgn(hrgn, 0, 0, w, borderTop);
+    if ( m_labelWin )
+    {
+        // Don't exclude the entire rectangle at the top, we do need to paint
+        // the background of the gap between the label window and the box
+        // frame.
+        const wxRect labelRect = m_labelWin->GetRect();
+        const int gap = FromDIP(LABEL_HORZ_BORDER);
+
+        SubtractRectFromRgn(hrgn, 0, 0, labelRect.GetLeft() - gap, borderTop);
+        SubtractRectFromRgn(hrgn, labelRect.GetRight() + gap, 0, w, borderTop);
+    }
+    else
+    {
+        SubtractRectFromRgn(hrgn, 0, 0, w, borderTop);
+    }
 
     // bottom
     SubtractRectFromRgn(hrgn, 0, h - border, w, h);
@@ -300,10 +406,10 @@ WXHRGN wxStaticBox::MSWGetRegionWithoutChildren()
             continue;
         }
 
-        LONG style = ::GetWindowLong(child, GWL_STYLE);
+        wxMSWWinStyleUpdater updateStyle(child);
         wxString str(wxGetWindowClass(child));
         str.UpperCase();
-        if ( str == wxT("BUTTON") && (style & BS_GROUPBOX) == BS_GROUPBOX )
+        if ( str == wxT("BUTTON") && updateStyle.IsOn(BS_GROUPBOX) )
         {
             if ( child == GetHwnd() )
                 foundThis = true;
@@ -326,10 +432,9 @@ WXHRGN wxStaticBox::MSWGetRegionWithoutChildren()
         {
             // need to remove WS_CLIPSIBLINGS from all sibling windows
             // that are within this staticbox if set
-            if ( style & WS_CLIPSIBLINGS )
+            if ( updateStyle.IsOn(WS_CLIPSIBLINGS) )
             {
-                style &= ~WS_CLIPSIBLINGS;
-                ::SetWindowLong(child, GWL_STYLE, style);
+                updateStyle.TurnOff(WS_CLIPSIBLINGS).Apply();
 
                 // MSDN: "If you have changed certain window data using
                 // SetWindowLong, you must call SetWindowPos to have the
@@ -371,14 +476,6 @@ WXHRGN wxStaticBox::MSWGetRegionWithoutChildren()
 // do anything in such case)
 void wxStaticBox::PaintBackground(wxDC& dc, const RECT& rc)
 {
-    // note that we do not use the box background colour here, it shouldn't
-    // apply to its interior for several reasons:
-    //  1. wxGTK doesn't do it
-    //  2. controls inside the box don't get correct bg colour because they
-    //     are not our children so we'd have some really ugly colour mix if
-    //     we did it
-    //  3. this is backwards compatible behaviour and some people rely on it,
-    //     see http://groups.google.com/groups?selm=4252E932.3080801%40able.es
     wxMSWDCImpl *impl = (wxMSWDCImpl*) dc.GetImpl();
     HBRUSH hbr = MSWGetBgBrush(impl->GetHDC());
 
@@ -404,7 +501,7 @@ void wxStaticBox::PaintForeground(wxDC& dc, const RECT&)
     // background mode doesn't change anything: the static box def window proc
     // still draws the label in its own colours, so we need to redraw the text
     // ourselves if we have a non default fg colour
-    if ( m_hasFgCol && wxUxThemeEngine::GetIfActive() )
+    if ( m_hasFgCol && wxUxThemeIsActive() && !m_labelWin )
     {
         // draw over the text in default colour in our colour
         HDC hdc = GetHdcOf(*impl);
@@ -426,7 +523,7 @@ void wxStaticBox::PaintForeground(wxDC& dc, const RECT&)
             if ( hTheme )
             {
                 wxUxThemeFont themeFont;
-                if ( wxUxThemeEngine::Get()->GetThemeFont
+                if ( ::GetThemeFont
                                              (
                                                 hTheme,
                                                 hdc,
@@ -448,23 +545,18 @@ void wxStaticBox::PaintForeground(wxDC& dc, const RECT&)
         dc.GetTextExtent(wxStripMenuCodes(label, wxStrip_Mnemonics),
                          &width, &height);
 
-        int x;
-        int y = height;
-
         // first we need to correctly paint the background of the label
         // as Windows ignores the brush offset when doing it
-        //
-        // FIXME: value of x is hardcoded as this is what it is on my system,
-        //        no idea if it's true everywhere
-        RECT dimensions = {0, 0, 0, y};
-        x = 9;
+        // NOTE: Border intentionally does not use DIPs in order to match native look
+        const int x = LABEL_HORZ_OFFSET;
+        RECT dimensions = { x, 0, 0, height };
         dimensions.left = x;
         dimensions.right = x + width;
 
         // need to adjust the rectangle to cover all the label background
-        dimensions.left -= 2;
-        dimensions.right += 2;
-        dimensions.bottom += 2;
+        dimensions.left -= LABEL_HORZ_BORDER;
+        dimensions.right += LABEL_HORZ_BORDER;
+        dimensions.bottom += LABEL_VERT_BORDER;
 
         if ( UseBgCol() )
         {
@@ -494,7 +586,7 @@ void wxStaticBox::PaintForeground(wxDC& dc, const RECT&)
         }
 
         // now draw the text
-        RECT rc2 = { x, 0, x + width, y };
+        RECT rc2 = { x, 0, x + width, height };
         ::DrawText(hdc, label.t_str(), label.length(), &rc2,
                    drawTextFlags);
     }
@@ -507,10 +599,10 @@ void wxStaticBox::OnPaint(wxPaintEvent& WXUNUSED(event))
     ::GetClientRect(GetHwnd(), &rc);
     wxPaintDC dc(this);
 
-	// No need to do anything if the client rectangle is empty and, worse,
+    // No need to do anything if the client rectangle is empty and, worse,
     // doing it would result in an assert when creating the bitmap below.
-	if ( !rc.right || !rc.bottom )
-		return;
+    if ( !rc.right || !rc.bottom )
+        return;
 
     // draw the entire box in a memory DC
     wxMemoryDC memdc(&dc);
@@ -530,8 +622,31 @@ void wxStaticBox::OnPaint(wxPaintEvent& WXUNUSED(event))
     GetBordersForSizer(&borderTop, &border);
 
     // top
-    dc.Blit(border, 0, rc.right - border, borderTop,
-            &memdc, border, 0);
+    if ( m_labelWin )
+    {
+        // We also have to exclude the area taken by the label window,
+        // otherwise there would be flicker when it draws itself on top of it.
+        const wxRect labelRect = m_labelWin->GetRect();
+
+        // We also leave a small border around label window to make it appear
+        // more similarly to a plain text label.
+        const int gap = FromDIP(LABEL_HORZ_BORDER);
+
+        dc.Blit(border, 0,
+                labelRect.GetLeft() - gap - border,
+                borderTop,
+                &memdc, border, 0);
+        dc.Blit(labelRect.GetRight() + gap, 0,
+                rc.right - (labelRect.GetRight() + gap),
+                borderTop,
+                &memdc, border, 0);
+    }
+    else
+    {
+        dc.Blit(border, 0, rc.right - border, borderTop,
+                &memdc, border, 0);
+    }
+
     // bottom
     dc.Blit(border, rc.bottom - border, rc.right - border, border,
             &memdc, border, rc.bottom - border);
