@@ -10,9 +10,6 @@
 
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #ifndef WX_PRECOMP
     #include "wx/icon.h"
@@ -29,6 +26,7 @@
 #include "wx/msw/taskbarbutton.h"
 #include "wx/scopedptr.h"
 #include "wx/msw/private/comptr.h"
+#include "wx/msw/private/cotaskmemptr.h"
 
 #include <shlwapi.h>
 #include <initguid.h>
@@ -498,10 +496,9 @@ wxTaskBarJumpListItem* GetItemFromIShellItem(IShellItem *shellItem)
     wxTaskBarJumpListItem *item =
         new wxTaskBarJumpListItem(NULL, wxTASKBAR_JUMP_LIST_DESTINATION);
 
-    wchar_t *name;
+    wxCoTaskMemPtr<wchar_t> name;
     shellItem->GetDisplayName(SIGDN_FILESYSPATH, &name);
     item->SetFilePath(wxString(name));
-    CoTaskMemFree(name);
     return item;
 }
 
@@ -882,20 +879,7 @@ bool wxTaskBarButtonImpl::InsertThumbBarButton(size_t pos,
 wxThumbBarButton* wxTaskBarButtonImpl::RemoveThumbBarButton(
     wxThumbBarButton *button)
 {
-    for ( wxThumbBarButtons::iterator iter = m_thumbBarButtons.begin();
-          iter != m_thumbBarButtons.end();
-          ++iter )
-    {
-        if ( button == *iter )
-        {
-            m_thumbBarButtons.erase(iter);
-            button->SetParent(NULL);
-            InitOrUpdateThumbBarButtons();
-            return *iter;
-        }
-    }
-
-    return NULL;
+    return RemoveThumbBarButton(button->GetID());
 }
 
 wxThumbBarButton* wxTaskBarButtonImpl::RemoveThumbBarButton(int id)
@@ -904,12 +888,13 @@ wxThumbBarButton* wxTaskBarButtonImpl::RemoveThumbBarButton(int id)
           iter != m_thumbBarButtons.end();
           ++iter )
     {
-        if ( id == (*iter)->GetID() )
+        wxThumbBarButton* button = *iter;
+        if ( id == button->GetID() )
         {
             m_thumbBarButtons.erase(iter);
-            (*iter)->SetParent(NULL);
+            button->SetParent(NULL);
             InitOrUpdateThumbBarButtons();
-            return *iter;
+            return button;
         }
     }
 
@@ -1391,10 +1376,11 @@ wxTaskBarJumpListImpl::RemoveCustomCategory(const wxString& title)
           it != m_customCategories.end();
           ++it )
     {
-        if ( (*it)->GetTitle() == title )
+        wxTaskBarJumpListCategory* tbJlCat = *it;
+        if ( tbJlCat->GetTitle() == title )
         {
             m_customCategories.erase(it);
-            return *it;
+            return tbJlCat;
         }
     }
 
