@@ -18,26 +18,75 @@
 
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
-#if wxUSE_DISPLAY
-
-#include "wx/display.h"
+#include "wx/private/display.h"
 
 #ifndef WX_PRECOMP
     #include "wx/dynarray.h"
     #include "wx/log.h"
     #include "wx/string.h"
     #include "wx/gdicmn.h"
+    #include "wx/nonownedwnd.h"
 #endif
 
-#include "wx/display_impl.h"
-#include "wx/scopedarray.h"
 #include "wx/osx/private.h"
 
-#if wxOSX_USE_COCOA_OR_CARBON
+// ----------------------------------------------------------------------------
+// common helpers compiled even in wxUSE_DISPLAY==0 case
+// ----------------------------------------------------------------------------
+
+// This one is defined in Objective C++ code.
+extern wxRect wxOSXGetMainDisplayClientArea();
+extern wxRect wxOSXGetDisplayClientArea(CGDirectDisplayID id);
+
+namespace
+{
+
+double wxGetScaleFactor( CGDirectDisplayID ID)
+{
+    wxCFRef<CGDisplayModeRef> mode = CGDisplayCopyDisplayMode(ID);
+    size_t width = CGDisplayModeGetWidth(mode);
+    size_t pixelsw = CGDisplayModeGetPixelWidth(mode);
+    return (double)pixelsw/width;
+}
+
+wxRect wxGetDisplayGeometry(CGDirectDisplayID id)
+{
+    CGRect theRect = CGDisplayBounds(id);
+    return wxRect( (int)theRect.origin.x,
+                   (int)theRect.origin.y,
+                   (int)theRect.size.width,
+                   (int)theRect.size.height ); //floats
+}
+
+int wxGetDisplayDepth(CGDirectDisplayID id)
+{
+    CGDisplayModeRef currentMode = CGDisplayCopyDisplayMode(id);
+    CFStringRef encoding = CGDisplayModeCopyPixelEncoding(currentMode);
+
+    int theDepth = 32; // some reasonable default
+    if(encoding)
+    {
+        if(CFStringCompare(encoding, CFSTR(IO32BitDirectPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo)
+            theDepth = 32;
+        else if(CFStringCompare(encoding, CFSTR(IO16BitDirectPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo)
+            theDepth = 16;
+        else if(CFStringCompare(encoding, CFSTR(IO8BitIndexedPixels), kCFCompareCaseInsensitive) == kCFCompareEqualTo)
+            theDepth = 8;
+
+        CFRelease(encoding);
+    }
+
+    CGDisplayModeRelease(currentMode);
+
+    return theDepth;
+}
+
+} // anonymous namespace
+
+#if wxUSE_DISPLAY
+
+#include "wx/scopedarray.h"
 
 // ----------------------------------------------------------------------------
 // display classes implementation
@@ -54,7 +103,8 @@ public:
 
     virtual wxRect GetGeometry() const wxOVERRIDE;
     virtual wxRect GetClientArea() const wxOVERRIDE;
-    virtual wxString GetName() const wxOVERRIDE { return wxString(); }
+    virtual int GetDepth() const wxOVERRIDE;
+    virtual double GetScaleFactor() const wxOVERRIDE;
 
     virtual wxArrayVideoModes GetModes(const wxVideoMode& mode) const wxOVERRIDE;
     virtual wxVideoMode GetCurrentMode() const wxOVERRIDE;
@@ -76,6 +126,7 @@ public:
     virtual wxDisplayImpl *CreateDisplay(unsigned n) wxOVERRIDE;
     virtual unsigned GetCount() wxOVERRIDE;
     virtual int GetFromPoint(const wxPoint& pt) wxOVERRIDE;
+    virtual int GetFromWindow(const wxWindow *window) wxOVERRIDE;
 
 protected:
     wxDECLARE_NO_COPY_CLASS(wxDisplayFactoryMacOSX);
@@ -128,29 +179,14 @@ static CGDisplayErr wxOSXGetDisplayList(CGDisplayCount maxDisplays,
     return error;
 }
 
-unsigned wxDisplayFactoryMacOSX::GetCount()
+static int wxOSXGetDisplayFromID( CGDirectDisplayID theID )
 {
-    CGDisplayCount count;
-    CGDisplayErr err = wxOSXGetDisplayList(0, NULL, &count);
-
-    wxCHECK_MSG( err == CGDisplayNoErr, 0, "wxOSXGetDisplayList() failed" );
-
-    return count;
-}
-
-int wxDisplayFactoryMacOSX::GetFromPoint(const wxPoint& p)
-{
-    CGPoint thePoint = {(float)p.x, (float)p.y};
-    CGDirectDisplayID theID;
-    CGDisplayCount theCount;
-    CGDisplayErr err = CGGetDisplaysWithPoint(thePoint, 1, &theID, &theCount);
-    wxASSERT(err == CGDisplayNoErr);
-
     int nWhich = wxNOT_FOUND;
+    CGDisplayCount theCount;
+    CGDisplayErr err = wxOSXGetDisplayList(0, NULL, &theCount);
 
-    if (theCount)
+    if (err == CGDisplayNoErr && theCount > 0 )
     {
-        theCount = GetCount();
         CGDirectDisplayID* theIDs = new CGDirectDisplayID[theCount];
         err = wxOSXGetDisplayList(theCount, theIDs, &theCount);
         wxASSERT(err == CGDisplayNoErr);
@@ -173,6 +209,68 @@ int wxDisplayFactoryMacOSX::GetFromPoint(const wxPoint& p)
     return nWhich;
 }
 
+unsigned wxDisplayFactoryMacOSX::GetCount()
+{
+    CGDisplayCount count;
+    CGDisplayErr err = wxOSXGetDisplayList(0, NULL, &count);
+
+    wxCHECK_MSG( err == CGDisplayNoErr, 0, "wxOSXGetDisplayList() failed" );
+
+    return count;
+}
+
+int wxDisplayFactoryMacOSX::GetFromPoint(const wxPoint& p)
+{
+    CGPoint thePoint = { CGFloat(p.x), CGFloat(p.y) };
+    CGDirectDisplayID theID;
+    CGDisplayCount theCount;
+    CGDisplayErr err = CGGetDisplaysWithPoint(thePoint, 1, &theID, &theCount);
+    wxASSERT(err == CGDisplayNoErr);
+    wxUnusedVar(err); // suppress "unused" warning in non-debug builds
+
+    if (theCount)
+        return wxOSXGetDisplayFromID(theID);
+
+    return wxNOT_FOUND;
+}
+
+int wxDisplayFactoryMacOSX::GetFromWindow(const wxWindow *window)
+{
+    wxCHECK_MSG( window, wxNOT_FOUND, "window can't be NULL" );
+
+    wxNonOwnedWindow* const tlw = window->MacGetTopLevelWindow();
+    // not yet instantiated
+    if ( tlw->GetWXWindow() == NULL )
+        return wxNOT_FOUND;
+
+    int x,y,w,h;
+
+    tlw->GetPosition(&x, &y);
+    tlw->GetSize(&w, &h);
+
+    CGRect r = CGRectMake(x, y, w, h);
+    CGDisplayCount theCount;
+    CGDisplayErr err = CGGetDisplaysWithRect(r, 0, NULL, &theCount);
+    wxASSERT(err == CGDisplayNoErr);
+
+    wxScopedArray<CGDirectDisplayID> theIDs(theCount);
+    err = CGGetDisplaysWithRect(r, theCount, theIDs.get(), &theCount);
+    wxASSERT(err == CGDisplayNoErr);
+
+    const double scaleWindow = tlw->GetContentScaleFactor();
+    for ( unsigned i = 0; i < theCount; ++i )
+    {
+        // find a screen intersecting having the same contentScale as the window itself
+        double scale = wxGetScaleFactor(theIDs[i]);
+        if ( fabs(scale - scaleWindow) < 0.01 )
+        {
+            return wxOSXGetDisplayFromID(theIDs[i]);
+        }
+    }
+
+    return wxNOT_FOUND;
+}
+
 wxDisplayImpl *wxDisplayFactoryMacOSX::CreateDisplay(unsigned n)
 {
     CGDisplayCount theCount = GetCount();
@@ -197,22 +295,22 @@ bool wxDisplayImplMacOSX::IsPrimary() const
 
 wxRect wxDisplayImplMacOSX::GetGeometry() const
 {
-    CGRect theRect = CGDisplayBounds(m_id);
-    return wxRect( (int)theRect.origin.x,
-                   (int)theRect.origin.y,
-                   (int)theRect.size.width,
-                   (int)theRect.size.height ); //floats
+    return wxGetDisplayGeometry(m_id);
 }
 
 wxRect wxDisplayImplMacOSX::GetClientArea() const
 {
-    // VZ: I don't know how to get client area for arbitrary display but
-    //     wxGetClientDisplayRect() does work correctly for at least the main
-    //     one (TODO: do it correctly for the other displays too)
-    if ( IsPrimary() )
-        return wxGetClientDisplayRect();
+    return wxOSXGetDisplayClientArea(m_id);
+}
 
-    return wxDisplayImpl::GetClientArea();
+int wxDisplayImplMacOSX::GetDepth() const
+{
+    return wxGetDisplayDepth(m_id);
+}
+
+double wxDisplayImplMacOSX::GetScaleFactor() const
+{
+    return wxGetScaleFactor(m_id);
 }
 
 static int wxOSXCGDisplayModeGetBitsPerPixel( CGDisplayModeRef theValue )
@@ -237,7 +335,7 @@ wxArrayVideoModes wxDisplayImplMacOSX::GetModes(const wxVideoMode& mode) const
     
     for (CFIndex i = 0; i < CFArrayGetCount(theArray); ++i)
     {
-        CGDisplayModeRef theValue = (CGDisplayModeRef) CFArrayGetValueAtIndex( theArray, i );
+        CGDisplayModeRef theValue = static_cast<CGDisplayModeRef>(const_cast<void*>(CFArrayGetValueAtIndex(theArray, i)));
         
         wxVideoMode theMode(
                             CGDisplayModeGetWidth(theValue),
@@ -281,7 +379,7 @@ bool wxDisplayImplMacOSX::ChangeMode( const wxVideoMode& mode )
     
     for (CFIndex i = 0; i < CFArrayGetCount(theArray); ++i)
     {
-        CGDisplayModeRef theValue = (CGDisplayModeRef) CFArrayGetValueAtIndex( theArray, i );
+        CGDisplayModeRef theValue = static_cast<CGDisplayModeRef>(const_cast<void*>(CFArrayGetValueAtIndex(theArray, i)));
         
         wxVideoMode theMode(
                             CGDisplayModeGetWidth(theValue),
@@ -311,13 +409,39 @@ bool wxDisplayImplMacOSX::ChangeMode( const wxVideoMode& mode )
     return new wxDisplayFactoryMacOSX;
 }
 
-#else
+#else // !wxUSE_DISPLAY
+
+class wxDisplayImplSingleMacOSX : public wxDisplayImplSingle
+{
+public:
+    virtual wxRect GetGeometry() const wxOVERRIDE
+    {
+        return wxGetDisplayGeometry(CGMainDisplayID());
+    }
+
+    virtual wxRect GetClientArea() const wxOVERRIDE
+    {
+        return wxOSXGetMainDisplayClientArea();
+    }
+
+    virtual int GetDepth() const wxOVERRIDE
+    {
+        return wxGetDisplayDepth(CGMainDisplayID());
+    }
+};
+
+class wxDisplayFactorySingleMacOSX : public wxDisplayFactorySingle
+{
+protected:
+    virtual wxDisplayImpl *CreateSingleDisplay() wxOVERRIDE
+    {
+        return new wxDisplayImplSingleMacOSX;
+    }
+};
 
 /* static */ wxDisplayFactory *wxDisplay::CreateFactory()
 {
-    return new wxDisplayFactorySingle;
+    return new wxDisplayFactorySingleMacOSX;
 }
 
-#endif
-
 #endif // wxUSE_DISPLAY
