@@ -34,11 +34,7 @@
 #endif
 
 #if wxUSE_STD_IOSTREAM
-    #if wxUSE_IOSTREAMH
-        #include <fstream.h>
-    #else
-        #include <fstream>
-    #endif
+    #include <fstream>
 #endif
 
 #include "wx/filefn.h"
@@ -238,8 +234,7 @@ protected :
 - (BOOL)textFieldShouldReturn:(UITextField *)textField
 {
     wxUnusedVar(textField);
-    
-    
+
     return NO;
 }
 
@@ -317,7 +312,7 @@ wxUITextViewControl::wxUITextViewControl( wxTextCtrl *wxPeer, UITextView* v) :
 {
     m_textView = v;
     m_delegate= [[wxUITextViewDelegate alloc] init];
-    
+
     [m_textView setDelegate:m_delegate];
 }
 
@@ -337,23 +332,24 @@ bool wxUITextViewControl::CanFocus() const
 
 wxString wxUITextViewControl::GetStringValue() const
 {
+    wxString result;
     if (m_textView)
     {
-        wxString result = wxCFStringRef::AsString([m_textView text], m_wxPeer->GetFont().GetEncoding());
-        wxMacConvertNewlines13To10( &result ) ;
-        return result;
+        result = wxMacConvertNewlines13To10(
+            wxCFStringRef::AsString([m_textView text], m_wxPeer->GetFont().GetEncoding()));
     }
-    return wxEmptyString;
+    return result;
 }
 
 void wxUITextViewControl::SetStringValue( const wxString &str)
 {
-    wxString st = str;
-    wxMacConvertNewlines10To13( &st );
     wxMacEditHelper helper(m_textView);
 
     if (m_textView)
+    {
+        wxString st(wxMacConvertNewlines10To13(str));
         [m_textView setText: wxCFStringRef( st , m_wxPeer->GetFont().GetEncoding() ).AsNSString()];
+    }
 }
 
 void wxUITextViewControl::Copy()
@@ -420,10 +416,9 @@ void wxUITextViewControl::SetSelection( long from , long to )
 
 void wxUITextViewControl::WriteText(const wxString& str)
 {
-    wxString st = str;
-    wxMacConvertNewlines10To13( &st );
+    wxString st(wxMacConvertNewlines10To13(str));
     wxMacEditHelper helper(m_textView);
-    
+
     wxCFStringRef insert( st , m_wxPeer->GetFont().GetEncoding() );
     NSMutableString* subst = [NSMutableString stringWithString:[m_textView text]];
     [subst replaceCharactersInRange:[m_textView selectedRange] withString:insert.AsNSString()];
@@ -431,7 +426,7 @@ void wxUITextViewControl::WriteText(const wxString& str)
     [m_textView setText:subst];
 }
 
-void wxUITextViewControl::SetFont( const wxFont & font , const wxColour& WXUNUSED(foreground) , long WXUNUSED(windowStyle), bool WXUNUSED(ignoreBlack) )
+void wxUITextViewControl::SetFont(const wxFont & font)
 {
     if ([m_textView respondsToSelector:@selector(setFont:)])
         [m_textView setFont: font.OSXGetUIFont()];
@@ -466,10 +461,10 @@ bool wxUITextViewControl::GetStyle(long position, wxTextAttr& style)
         /*
         if (font)
             style.SetFont(wxFont(font));
-        
+
         if (bgcolor)
             style.SetBackgroundColour(wxColour(bgcolor));
-            
+
         if (fgcolor)
             style.SetTextColour(wxColour(fgcolor));
         */
@@ -489,15 +484,15 @@ void wxUITextViewControl::SetStyle(long start,
             range = [m_textView selectedRange];
 /*
         UITextStorage* storage = [m_textView textStorage];
-        
+
         wxFont font = style.GetFont();
         if (style.HasFont() && font.IsOk())
             [storage addAttribute:NSFontAttributeName value:font.OSXGetNSFont() range:range];
-        
+
         wxColour bgcolor = style.GetBackgroundColour();
         if (style.HasBackgroundColour() && bgcolor.IsOk())
             [storage addAttribute:NSBackgroundColorAttributeName value:bgcolor.OSXGetNSColor() range:range];
-        
+
         wxColour fgcolor = style.GetTextColour();
         if (style.HasTextColour() && fgcolor.IsOk())
             [storage addAttribute:NSForegroundColorAttributeName value:fgcolor.OSXGetNSColor() range:range];
@@ -512,9 +507,9 @@ void wxUITextViewControl::CheckSpelling(bool check)
 wxSize wxUITextViewControl::GetBestSize() const
 {
     wxRect r;
-    
+
     GetBestRect(&r);
-    
+
     /*
     if (m_textView && [m_textView layoutManager])
     {
@@ -526,7 +521,7 @@ wxSize wxUITextViewControl::GetBestSize() const
     }
     return wxSize(0,0);
     */
-    
+
     wxSize sz = r.GetSize();
     if ( sz.y < 31 )
         sz.y = 31;
@@ -571,7 +566,7 @@ void wxUITextFieldControl::SetStringValue( const wxString &str)
 wxSize wxUITextFieldControl::GetBestSize() const
 {
     wxRect r;
-    
+
     GetBestRect(&r);
     wxSize sz = r.GetSize();
     if ( sz.y < 31 )
@@ -605,7 +600,7 @@ void wxUITextFieldControl::SetEditable(bool editable)
         if ( !editable ) {
             [m_textField resignFirstResponder];
         }
-        
+
         [m_textField setEnabled: editable];
     }
 }
@@ -710,7 +705,7 @@ wxWidgetImplType* wxWidgetImpl::CreateTextControl( wxTextCtrl* wxpeer,
         UITextView * v = nil;
         v = [[UITextView alloc] initWithFrame:r];
         tv = v;
-        
+
         wxUITextViewControl* tc = new wxUITextViewControl( wxpeer, v );
         c = tc;
         t = tc;
@@ -721,39 +716,39 @@ wxWidgetImplType* wxWidgetImpl::CreateTextControl( wxTextCtrl* wxpeer,
         wxUITextField* v = [[wxUITextField alloc] initWithFrame:r];
         tv = v;
 
-		v.textColor = [UIColor blackColor];
-		v.font = [UIFont systemFontOfSize:17.0];
-		v.backgroundColor = [UIColor whiteColor];
-		
-		v.clearButtonMode = UITextFieldViewModeNever;
-		
+        v.textColor = [UIColor blackColor];
+        v.font = [UIFont systemFontOfSize:17.0];
+        v.backgroundColor = [UIColor whiteColor];
+
+        v.clearButtonMode = UITextFieldViewModeNever;
+
         [v setBorderStyle:UITextBorderStyleBezel];
         if ( style & wxNO_BORDER )
             v.borderStyle = UITextBorderStyleNone;
-         
+
         wxUITextFieldControl* tc = new wxUITextFieldControl( wxpeer, v );
         c = tc;
         t = tc;
     }
 #endif
-    
+
     if ( style & wxTE_PASSWORD )
         [tv setSecureTextEntry:YES];
-    
+
     if ( style & wxTE_CAPITALIZE )
         [tv setAutocapitalizationType:UITextAutocapitalizationTypeWords];
     else
         [tv setAutocapitalizationType:UITextAutocapitalizationTypeSentences];
-    
+
     if ( !(style & wxTE_MULTILINE) )
     {
         [tv setAutocorrectionType:UITextAutocorrectionTypeNo];
-		[tv setReturnKeyType:UIReturnKeyDone];
+        [tv setReturnKeyType:UIReturnKeyDone];
     }
     [tv setKeyboardType:UIKeyboardTypeDefault];
-    
+
     t->SetStringValue(str);
-    
+
     return c;
 }
 
