@@ -279,6 +279,14 @@ private:
 
 
 wxDialUpManagerImpl::wxDialUpManagerImpl()
+   : m_BeaconHost(WXDIALUP_MANAGER_DEFAULT_BEACONHOST)
+#ifdef __SGI__
+   , m_ConnectCommand("/usr/etc/ppp")
+#elif defined(__LINUX__)
+   // default values for Debian/GNU linux
+   , m_ConnectCommand("pon")
+   , m_HangUpCommand("poff")
+#endif
 {
    m_IsOnline =
    m_connCard = Net_Unknown;
@@ -286,17 +294,8 @@ wxDialUpManagerImpl::wxDialUpManagerImpl()
    m_timer = NULL;
    m_CanUseIfconfig = -1; // unknown
    m_CanUsePing = -1; // unknown
-   m_BeaconHost = WXDIALUP_MANAGER_DEFAULT_BEACONHOST;
    m_BeaconPort = 80;
 
-#ifdef __SGI__
-   m_ConnectCommand = wxT("/usr/etc/ppp");
-#elif defined(__LINUX__)
-   // default values for Debian/GNU linux
-   m_ConnectCommand = wxT("pon");
-   m_HangUpCommand = wxT("poff");
-#endif
-
    wxChar * dial = wxGetenv(wxT("WXDIALUP_DIALCMD"));
    wxChar * hup = wxGetenv(wxT("WXDIALUP_HUPCMD"));
    SetConnectCommand(dial ? wxString(dial) : m_ConnectCommand,
@@ -305,7 +304,7 @@ wxDialUpManagerImpl::wxDialUpManagerImpl()
 
 wxDialUpManagerImpl::~wxDialUpManagerImpl()
 {
-   if(m_timer) delete m_timer;
+   delete m_timer;
    if(m_DialProcess)
    {
       m_DialProcess->Disconnect();
@@ -421,7 +420,7 @@ void wxDialUpManagerImpl::CheckStatus(bool fromAsync) const
     // which is OS - specific and then sends the events.
 
     NetConnection oldIsOnline = m_IsOnline;
-    ( /* non-const */ (wxDialUpManagerImpl *)this)->CheckStatusInternal();
+    const_cast<wxDialUpManagerImpl*>(this)->CheckStatusInternal();
 
     // now send the events as appropriate: i.e. if the status changed and
     // if we're in defined state
