@@ -12,9 +12,6 @@
 
 #include "testprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_FILE
 
@@ -89,7 +86,9 @@ private:
                       const wxString& destFilePath);
 
     wxString m_fileNameASCII;
+#if wxUSE_UNICODE
     wxString m_fileNameNonASCII;
+#endif // wxUSE_UNICODE
     wxString m_fileNameWork;
 
     wxDECLARE_NO_COPY_CLASS(FileFunctionsTestCase);
@@ -113,10 +112,12 @@ void FileFunctionsTestCase::setUp()
     wxFileName fn1(wxFileName::GetTempDir(), wxT("wx_file_mask.txt"));
     m_fileNameASCII = fn1.GetFullPath();
 
+#if wxUSE_UNICODE
     // This file name is 'wx_file_mask.txt' in Russian.
     wxFileName fn2(wxFileName::GetTempDir(),
       wxT("wx_\u043C\u0430\u0441\u043A\u0430_\u0444\u0430\u0439\u043B\u0430.txt"));
     m_fileNameNonASCII = fn2.GetFullPath();
+#endif // wxUSE_UNICODE
 
     wxFileName fn3(wxFileName::GetTempDir(), wxT("wx_test_copy"));
     m_fileNameWork = fn3.GetFullPath();
@@ -129,10 +130,12 @@ void FileFunctionsTestCase::tearDown()
     {
         wxRemoveFile(m_fileNameASCII);
     }
+#if wxUSE_UNICODE
     if ( wxFileExists(m_fileNameNonASCII) )
     {
         wxRemoveFile(m_fileNameNonASCII);
     }
+#endif // wxUSE_UNICODE
     if ( wxFileExists(m_fileNameWork) )
     {
         wxRemoveFile(m_fileNameWork);
@@ -179,8 +182,10 @@ void FileFunctionsTestCase::CreateFile()
 {
     // Create file name containing ASCII characters only.
     DoCreateFile(m_fileNameASCII);
+#if wxUSE_UNICODE
     // Create file name containing non-ASCII characters.
     DoCreateFile(m_fileNameNonASCII);
+#endif // wxUSE_UNICODE
 }
 
 void FileFunctionsTestCase::DoCreateFile(const wxString& filePath)
@@ -202,8 +207,10 @@ void FileFunctionsTestCase::FileExists()
 
     // Check file name containing ASCII characters only.
     DoFileExists(m_fileNameASCII);
+#if wxUSE_UNICODE
     // Check file name containing non-ASCII characters.
     DoFileExists(m_fileNameNonASCII);
+#endif // wxUSE_UNICODE
 }
 
 void FileFunctionsTestCase::DoFileExists(const wxString& filePath)
@@ -226,8 +233,10 @@ void FileFunctionsTestCase::FindFile()
 {
     // Find file name containing ASCII characters only.
     DoFindFile(m_fileNameASCII);
+#if wxUSE_UNICODE
     // Find file name containing non-ASCII characters.
     DoFindFile(m_fileNameNonASCII);
+#endif // wxUSE_UNICODE
 }
 
 void FileFunctionsTestCase::DoFindFile(const wxString& filePath)
@@ -246,7 +255,7 @@ void FileFunctionsTestCase::DoFindFile(const wxString& filePath)
     // Check if file can be found (method 2).
     wxFileSystem fs;
     wxString furl = fs.FindFirst(filePath, wxFILE);
-    wxFileName fname = wxFileSystem::URLToFileName(furl);
+    wxFileName fname = wxFileName::URLToFileName(furl);
     foundFile = fname.GetFullPath();
     CPPUNIT_ASSERT_MESSAGE( msg, foundFile == filePath );
 
@@ -271,10 +280,10 @@ void FileFunctionsTestCase::FindFileNext()
     // Check using method 2.
     wxFileSystem fs;
     wxString furl = fs.FindFirst(fileMask, wxFILE);
-    fn1 = wxFileSystem::URLToFileName(furl);
+    fn1 = wxFileName::URLToFileName(furl);
     foundFile1 = fn1.GetFullPath();
     furl = fs.FindNext();
-    fn2 = wxFileSystem::URLToFileName(furl);
+    fn2 = wxFileName::URLToFileName(furl);
     foundFile2 = fn2.GetFullPath();
     // Full names must be different.
     CPPUNIT_ASSERT( fn1.GetFullPath() != fn2.GetFullPath() );
@@ -286,8 +295,10 @@ void FileFunctionsTestCase::RemoveFile()
 {
     // Create & remove file with name containing ASCII characters only.
     DoRemoveFile(m_fileNameASCII);
+#if wxUSE_UNICODE
     // Create & remove file with name containing non-ASCII characters.
     DoRemoveFile(m_fileNameNonASCII);
+#endif // wxUSE_UNICODE
 }
 
 void FileFunctionsTestCase::DoRemoveFile(const wxString& filePath)
@@ -308,6 +319,7 @@ void FileFunctionsTestCase::RenameFile()
 {
     // Verify renaming file with/without overwriting
     // when new file already exist/don't exist.
+#if wxUSE_UNICODE
     DoRenameFile(m_fileNameASCII, m_fileNameNonASCII, false, false);
     DoRenameFile(m_fileNameASCII, m_fileNameNonASCII, false, true);
     DoRenameFile(m_fileNameASCII, m_fileNameNonASCII, true, false);
@@ -316,6 +328,7 @@ void FileFunctionsTestCase::RenameFile()
     DoRenameFile(m_fileNameNonASCII, m_fileNameASCII, false, true);
     DoRenameFile(m_fileNameNonASCII, m_fileNameASCII, true, false);
     DoRenameFile(m_fileNameNonASCII, m_fileNameASCII, true, true);
+#endif // wxUSE_UNICODE
 }
 
 void
@@ -381,8 +394,10 @@ FileFunctionsTestCase::DoRenameFile(const wxString& oldFilePath,
 
 void FileFunctionsTestCase::ConcatenateFiles()
 {
+#if wxUSE_UNICODE
     DoConcatFile(m_fileNameASCII, m_fileNameNonASCII, m_fileNameWork);
     DoConcatFile(m_fileNameNonASCII, m_fileNameASCII, m_fileNameWork);
+#endif // wxUSE_UNICODE
 }
 
 void FileFunctionsTestCase::DoConcatFile(const wxString& filePath1,
@@ -510,6 +525,12 @@ void FileFunctionsTestCase::IsAbsolutePath()
     CPPUNIT_ASSERT( filename.MakeAbsolute() );
     // wxFileName::GetFullPath returns absolute path
     CPPUNIT_ASSERT_MESSAGE( msg, wxIsAbsolutePath(filename.GetFullPath()));
+
+#ifdef __WINDOWS__
+    CPPUNIT_ASSERT( wxIsAbsolutePath("\\"));
+    CPPUNIT_ASSERT( wxIsAbsolutePath("c:"));
+    CPPUNIT_ASSERT( !wxIsAbsolutePath("c"));
+#endif
 }
 
 void FileFunctionsTestCase::PathOnly()
@@ -528,21 +549,25 @@ void FileFunctionsTestCase::PathOnly()
 // Rmdir fails on them on Linux. See ticket #17644.
 void FileFunctionsTestCase::Mkdir()
 {
+#if wxUSE_UNICODE
     wxString dirname = wxString::FromUTF8("__wxMkdir_test_dir_with_\xc3\xb6");
     const std::string msg = wxString::Format("Dir: %s", dirname).ToStdString();
     CPPUNIT_ASSERT_MESSAGE( msg, wxMkdir(dirname) );
     CPPUNIT_ASSERT_MESSAGE( msg, wxDirExists(dirname) );
     CPPUNIT_ASSERT_MESSAGE( msg, wxRmdir(dirname) );
+#endif // wxUSE_UNICODE
 }
 
 void FileFunctionsTestCase::Rmdir()
 {
+#if wxUSE_UNICODE
     wxString dirname = wxString::FromUTF8("__wxRmdir_test_dir_with_\xc3\xb6");
     const std::string msg = wxString::Format("Dir: %s", dirname).ToStdString();
 
     CPPUNIT_ASSERT_MESSAGE( msg, wxMkdir(dirname) );
     CPPUNIT_ASSERT_MESSAGE( msg, wxRmdir(dirname) );
     CPPUNIT_ASSERT_MESSAGE( msg, !wxDirExists(dirname) );
+#endif // wxUSE_UNICODE
 }
 
 /*
