@@ -3,7 +3,6 @@
 // Purpose:     wxFileName unit test
 // Author:      Vadim Zeitlin
 // Created:     2004-07-25
-// RCS-ID:      $Id$
 // Copyright:   (c) 2004 Vadim Zeitlin
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -13,9 +12,6 @@
 
 #include "testprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #ifndef WX_PRECOMP
     #include "wx/utils.h"
@@ -24,12 +20,21 @@
 #include "wx/filename.h"
 #include "wx/filefn.h"
 #include "wx/stdpaths.h"
+#include "wx/scopeguard.h"
 
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
     #include "wx/msw/registry.h"
-#endif // __WXMSW__
+    #include "wx/msw/wrapshl.h"
+    #include "wx/msw/ole/oleutils.h"
+    #include "wx/msw/private/comptr.h"
+#endif // __WINDOWS__
+
+#ifdef __UNIX__
+    #include <unistd.h>
+#endif // __UNIX__
 
 #include "testfile.h"
+#include "testdate.h"
 
 // ----------------------------------------------------------------------------
 // test data
@@ -125,6 +130,7 @@ private:
         CPPUNIT_TEST( TestSetPath );
         CPPUNIT_TEST( TestStrip );
         CPPUNIT_TEST( TestNormalize );
+        CPPUNIT_TEST( TestRelative );
         CPPUNIT_TEST( TestReplace );
         CPPUNIT_TEST( TestGetHumanReadable );
 #ifdef __WINDOWS__
@@ -134,6 +140,15 @@ private:
         CPPUNIT_TEST( TestVolumeUniqueName );
         CPPUNIT_TEST( TestCreateTempFileName );
         CPPUNIT_TEST( TestGetTimes );
+        CPPUNIT_TEST( TestSetTimes );
+        CPPUNIT_TEST( TestExists );
+        CPPUNIT_TEST( TestIsSame );
+#if defined(__UNIX__)
+        CPPUNIT_TEST( TestSymlinks );
+#endif // __UNIX__
+#ifdef __WINDOWS__
+        CPPUNIT_TEST( TestShortcuts );
+#endif // __WINDOWS__
     CPPUNIT_TEST_SUITE_END();
 
     void TestConstruction();
@@ -141,6 +156,7 @@ private:
     void TestSplit();
     void TestSetPath();
     void TestStrip();
+    void TestRelative();
     void TestNormalize();
     void TestReplace();
     void TestGetHumanReadable();
@@ -151,14 +167,23 @@ private:
     void TestVolumeUniqueName();
     void TestCreateTempFileName();
     void TestGetTimes();
+    void TestSetTimes();
+    void TestExists();
+    void TestIsSame();
+#if defined(__UNIX__)
+    void TestSymlinks();
+#endif // __UNIX__
+#ifdef __WINDOWS__
+    void TestShortcuts();
+#endif // __WINDOWS__
 
-    DECLARE_NO_COPY_CLASS(FileNameTestCase)
+    wxDECLARE_NO_COPY_CLASS(FileNameTestCase);
 };
 
 // register in the unnamed registry so that these tests are run by default
 CPPUNIT_TEST_SUITE_REGISTRATION( FileNameTestCase );
 
-// also include in it's own registry so that these tests can be run alone
+// also include in its own registry so that these tests can be run alone
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( FileNameTestCase, "FileNameTestCase" );
 
 void FileNameTestCase::TestConstruction()
@@ -314,7 +339,7 @@ void FileNameTestCase::TestNormalize()
     } tests[] =
     {
         // test wxPATH_NORM_ENV_VARS
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
         { "%ABCDEF%/g/h/i", wxPATH_NORM_ENV_VARS, "abcdef/g/h/i", wxPATH_UNIX },
 #else
         { "$(ABCDEF)/g/h/i", wxPATH_NORM_ENV_VARS, "abcdef/g/h/i", wxPATH_UNIX },
@@ -322,8 +347,12 @@ void FileNameTestCase::TestNormalize()
 
         // test wxPATH_NORM_DOTS
         { "a/.././b/c/../../", wxPATH_NORM_DOTS, "", wxPATH_UNIX },
-        { "./", wxPATH_NORM_DOTS, "", wxPATH_UNIX },
-        { "b/../", wxPATH_NORM_DOTS, "", wxPATH_UNIX },
+        { "", wxPATH_NORM_DOTS, "", wxPATH_UNIX },
+        { "./foo", wxPATH_NORM_DOTS, "foo", wxPATH_UNIX },
+        { "b/../bar", wxPATH_NORM_DOTS, "bar", wxPATH_UNIX },
+        { "c/../../quux", wxPATH_NORM_DOTS, "../quux", wxPATH_UNIX },
+        { "/c/../../quux", wxPATH_NORM_DOTS, "/quux", wxPATH_UNIX },
+        { "../../quux", wxPATH_NORM_DOTS, "../../quux", wxPATH_UNIX },
 
         // test wxPATH_NORM_TILDE: notice that ~ is only interpreted specially
         // when it is the first character in the file name
@@ -395,7 +424,7 @@ void FileNameTestCase::TestNormalize()
     // and also that the registry key was changed recently and didn't take
     // effect yet but these are marginal cases which we consciously choose to
     // ignore for now)
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
     long shortNamesDisabled;
     if ( wxRegKey
          (
@@ -409,7 +438,25 @@ void FileNameTestCase::TestNormalize()
         CPPUNIT_ASSERT_EQUAL( "..\\mkinstalldirs", fn.GetFullPath() );
     }
     //else: when in doubt, don't run the test
-#endif // __WXMSW__
+#endif // __WINDOWS__
+}
+
+void FileNameTestCase::TestRelative()
+{
+    const wxString pathSep = wxFileName::GetPathSeparator();
+
+    wxFileName fn("a" + pathSep + "b.cpp");
+    fn.MakeRelativeTo("a");
+    CPPUNIT_ASSERT_EQUAL( "b.cpp", fn.GetFullPath() );
+
+    fn.AssignDir("a" + pathSep + "b");
+    fn.MakeRelativeTo("a");
+
+    CPPUNIT_ASSERT_EQUAL( "b" + pathSep, fn.GetFullPath() );
+
+    fn.AssignDir("a");
+    fn.MakeRelativeTo("a");
+    CPPUNIT_ASSERT_EQUAL( "." + pathSep, fn.GetFullPath() );
 }
 
 void FileNameTestCase::TestReplace()
@@ -593,7 +640,7 @@ void FileNameTestCase::TestCreateTempFileName()
         { "foo", "$SYSTEM_TEMP", true },
         { "..", "$SYSTEM_TEMP", true },
         { "../bar", "..", true },
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
         { "$USER_DOCS_DIR\\", "$USER_DOCS_DIR", true },
         { "c:\\a\\directory\\which\\does\\not\\exist", "", false },
 #elif defined( __UNIX__ )
@@ -616,7 +663,7 @@ void FileNameTestCase::TestCreateTempFileName()
         if (testData[n].shouldSucceed)
         {
             errDesc += "; path is " + path.ToStdString();
-        
+
             // test the place where the temp file has been created
             wxString expected = testData[n].expectedFolder;
             expected.Replace("$SYSTEM_TEMP", wxStandardPaths::Get().GetTempDir());
@@ -634,13 +681,442 @@ void FileNameTestCase::TestGetTimes()
 {
     wxFileName fn(wxFileName::CreateTempFileName("filenametest"));
     CPPUNIT_ASSERT( fn.IsOk() );
+    wxON_BLOCK_EXIT1( wxRemoveFile, fn.GetFullPath() );
 
     wxDateTime dtAccess, dtMod, dtCreate;
     CPPUNIT_ASSERT( fn.GetTimes(&dtAccess, &dtMod, &dtCreate) );
 
-    // make sure all retrieved dates are equal to the current date&time 
+    // make sure all retrieved dates are equal to the current date&time
     // with an accuracy up to 1 minute
     CPPUNIT_ASSERT(dtCreate.IsEqualUpTo(wxDateTime::Now(), wxTimeSpan(0,1)));
     CPPUNIT_ASSERT(dtMod.IsEqualUpTo(wxDateTime::Now(), wxTimeSpan(0,1)));
     CPPUNIT_ASSERT(dtAccess.IsEqualUpTo(wxDateTime::Now(), wxTimeSpan(0,1)));
 }
+
+void FileNameTestCase::TestSetTimes()
+{
+    wxFileName fn(wxFileName::CreateTempFileName("filenametest"));
+    CPPUNIT_ASSERT( fn.IsOk() );
+    wxON_BLOCK_EXIT1( wxRemoveFile, fn.GetFullPath() );
+
+    const wxDateTime dtAccess(1, wxDateTime::Jan, 2013);
+    const wxDateTime dtModify(1, wxDateTime::Feb, 2013);
+    const wxDateTime dtCreate(1, wxDateTime::Mar, 2013);
+
+    CPPUNIT_ASSERT( fn.SetTimes(&dtAccess, &dtModify, &dtCreate) );
+
+    wxDateTime dtAccess2,
+               dtModify2,
+               dtCreate2;
+    CPPUNIT_ASSERT( fn.GetTimes(&dtAccess2, &dtModify2, &dtCreate2) );
+    CPPUNIT_ASSERT_EQUAL( dtAccess, dtAccess2 );
+    CPPUNIT_ASSERT_EQUAL( dtModify, dtModify2 );
+
+    // Under Unix the creation time can't be set.
+#ifdef __WINDOWS__
+    CPPUNIT_ASSERT_EQUAL( dtCreate, dtCreate2 );
+#endif // __WINDOWS__
+}
+
+void FileNameTestCase::TestExists()
+{
+    wxFileName fn(wxFileName::CreateTempFileName("filenametest"));
+    CPPUNIT_ASSERT( fn.IsOk() );
+    wxON_BLOCK_EXIT1( wxRemoveFile, fn.GetFullPath() );
+
+    CPPUNIT_ASSERT( fn.FileExists() );
+    CPPUNIT_ASSERT( !wxFileName::DirExists(fn.GetFullPath()) );
+
+    CPPUNIT_ASSERT( fn.Exists(wxFILE_EXISTS_REGULAR) );
+    CPPUNIT_ASSERT( !fn.Exists(wxFILE_EXISTS_DIR) );
+    CPPUNIT_ASSERT( fn.Exists() );
+
+    const wxString& tempdir = wxFileName::GetTempDir();
+
+    wxFileName fileInTempDir(tempdir, "bloordyblop");
+    CPPUNIT_ASSERT( !fileInTempDir.Exists() );
+    CPPUNIT_ASSERT( fileInTempDir.DirExists() );
+
+    wxFileName dirTemp(wxFileName::DirName(tempdir));
+    CPPUNIT_ASSERT( !dirTemp.FileExists() );
+    CPPUNIT_ASSERT( dirTemp.DirExists() );
+
+    CPPUNIT_ASSERT( dirTemp.Exists(wxFILE_EXISTS_DIR) );
+    CPPUNIT_ASSERT( !dirTemp.Exists(wxFILE_EXISTS_REGULAR) );
+    CPPUNIT_ASSERT( dirTemp.Exists() );
+
+#ifdef __UNIX__
+    CPPUNIT_ASSERT( !wxFileName::FileExists("/dev/null") );
+    CPPUNIT_ASSERT( !wxFileName::DirExists("/dev/null") );
+    CPPUNIT_ASSERT( wxFileName::Exists("/dev/null") );
+    CPPUNIT_ASSERT( wxFileName::Exists("/dev/null", wxFILE_EXISTS_DEVICE) );
+#ifdef __LINUX__
+    // These files are only guaranteed to exist under Linux.
+    // No need for wxFILE_EXISTS_NO_FOLLOW here; wxFILE_EXISTS_SYMLINK implies it
+    CPPUNIT_ASSERT( wxFileName::Exists("/proc/self", wxFILE_EXISTS_SYMLINK) );
+    CPPUNIT_ASSERT( wxFileName::Exists("/dev/log", wxFILE_EXISTS_SOCKET) );
+#endif // __LINUX__
+#ifndef __VMS
+    wxString fifo = dirTemp.GetPath() + "/fifo";
+   if (mkfifo(fifo.c_str(), 0600) == 0)
+    {
+        wxON_BLOCK_EXIT1(wxRemoveFile, fifo);
+
+        CPPUNIT_ASSERT( wxFileName::Exists(fifo, wxFILE_EXISTS_FIFO) );
+    }
+#endif
+#endif // __UNIX__
+}
+
+void FileNameTestCase::TestIsSame()
+{
+    wxFileName fn1( wxFileName::CreateTempFileName( "filenametest1" ) );
+    CPPUNIT_ASSERT( fn1.IsOk() );
+    wxON_BLOCK_EXIT1( wxRemoveFile, fn1.GetFullPath() );
+
+    wxFileName fn2( wxFileName::CreateTempFileName( "filenametest2" ) );
+    CPPUNIT_ASSERT( fn2.IsOk() );
+    wxON_BLOCK_EXIT1( wxRemoveFile, fn2.GetFullPath() );
+
+    CPPUNIT_ASSERT( fn1.SameAs( fn1 ) );
+    CPPUNIT_ASSERT( !fn1.SameAs( fn2 ) );
+
+#if defined(__UNIX__)
+    // We need to create a temporary directory and a temporary link.
+    // Unfortunately we can't use wxFileName::CreateTempFileName() for neither
+    // as it creates plain files, so use tempnam() explicitly instead.
+    char* tn = tempnam(NULL, "wxfn1");
+    const wxString tempdir1 = wxString::From8BitData(tn);
+    free(tn);
+
+    CPPUNIT_ASSERT( wxFileName::Mkdir(tempdir1) );
+    // Unfortunately the casts are needed to select the overload we need here.
+    wxON_BLOCK_EXIT2( static_cast<bool (*)(const wxString&, int)>(wxFileName::Rmdir),
+                      tempdir1, static_cast<int>(wxPATH_RMDIR_RECURSIVE) );
+
+    tn = tempnam(NULL, "wxfn2");
+    const wxString tempdir2 = wxString::From8BitData(tn);
+    free(tn);
+    CPPUNIT_ASSERT_EQUAL( 0, symlink(tempdir1.c_str(), tempdir2.c_str()) );
+    wxON_BLOCK_EXIT1( wxRemoveFile, tempdir2 );
+
+
+    wxFileName fn3(tempdir1, "foo");
+    wxFileName fn4(tempdir2, "foo");
+
+    // These files have different paths, hence are different.
+    CPPUNIT_ASSERT( !fn3.SameAs(fn4) );
+
+    // Create and close a file to trigger creating it.
+    wxFile(fn3.GetFullPath(), wxFile::write);
+
+    // Now that both files do exist we should be able to detect that they are
+    // actually the same file.
+    CPPUNIT_ASSERT( fn3.SameAs(fn4) );
+#endif // __UNIX__
+}
+
+#if defined(__UNIX__)
+
+// Tests for functions that are changed by ShouldFollowLink()
+void FileNameTestCase::TestSymlinks()
+{
+    const wxString tmpdir(wxStandardPaths::Get().GetTempDir());
+
+    wxFileName tmpfn(wxFileName::DirName(tmpdir));
+
+    // Create a temporary directory
+#ifdef __VMS
+    wxString name = tmpdir + ".filenametestXXXXXX]";
+    mkdir( name.char_str() , 0222 );
+    wxString tempdir = name;
+#else
+    wxString name = tmpdir + "/filenametestXXXXXX";
+    wxString tempdir = wxString::From8BitData(mkdtemp(name.char_str()));
+    tempdir << wxFileName::GetPathSeparator();
+#endif
+    wxFileName tempdirfn(wxFileName::DirName(tempdir));
+    CPPUNIT_ASSERT(tempdirfn.DirExists());
+
+    // Create a regular file in that dir, to act as a symlink target
+    wxFileName targetfn(wxFileName::CreateTempFileName(tempdir));
+    CPPUNIT_ASSERT(targetfn.FileExists());
+
+    // Resolving a non-symlink will just return the same thing
+    CPPUNIT_ASSERT_EQUAL_MESSAGE
+    (
+        "Non-symlink didn't resolve to the same file",
+        targetfn, targetfn.ResolveLink()
+    );
+
+    // Create a symlink to that file
+    wxFileName linktofile(tempdir, "linktofile");
+    CPPUNIT_ASSERT_EQUAL(0, symlink(targetfn.GetFullPath().c_str(),
+                                    linktofile.GetFullPath().c_str()));
+
+    // Test resolving the filename to the symlink
+    CPPUNIT_ASSERT_EQUAL_MESSAGE
+    (
+        "Failed to resolve symlink to file",
+        targetfn, linktofile.ResolveLink()
+    );
+
+    // Create a relative symlink to that file
+    wxFileName relativelinktofile(tempdir, "relativelinktofile");
+    wxString relativeFileName = "./" + targetfn.GetFullName();
+    CPPUNIT_ASSERT_EQUAL(0, symlink(relativeFileName.c_str(),
+                                    relativelinktofile.GetFullPath().c_str()));
+
+    CPPUNIT_ASSERT_EQUAL_MESSAGE
+    (
+        "Failed to resolve relative symlink to absolute file path",
+        targetfn, relativelinktofile.ResolveLink()
+    );
+
+    // ... and another to the temporary directory
+    const wxString linktodirName(tempdir + "/linktodir");
+    wxFileName linktodirfn(linktodirName);
+    wxFileName linktodir(wxFileName::DirName(linktodirName));
+    CPPUNIT_ASSERT_EQUAL(0, symlink(tmpfn.GetFullPath().c_str(),
+                                    linktodirName.c_str()));
+
+    CPPUNIT_ASSERT_EQUAL_MESSAGE
+    (
+        "Failed to resolve symlink to directory",
+        tmpfn, linktodirfn.ResolveLink()
+    );
+
+    // And symlinks to both of those symlinks
+    wxFileName linktofilelnk(tempdir, "linktofilelnk");
+    CPPUNIT_ASSERT_EQUAL(0, symlink(linktofile.GetFullPath().c_str(),
+                                    linktofilelnk.GetFullPath().c_str()));
+
+    CPPUNIT_ASSERT_EQUAL_MESSAGE
+    (
+        "Failed to resolve symlink to file symlink",
+        targetfn, linktofilelnk.ResolveLink()
+    );
+
+    wxFileName linktodirlnk(tempdir, "linktodirlnk");
+    CPPUNIT_ASSERT_EQUAL(0, symlink(linktodir.GetFullPath().c_str(),
+                                    linktodirlnk.GetFullPath().c_str()));
+
+    CPPUNIT_ASSERT_EQUAL_MESSAGE
+    (
+        "Failed to resolve symlink to directory symlink",
+        tmpfn, linktodirlnk.ResolveLink()
+    );
+
+    // Run the tests twice: once in the default symlink following mode and the
+    // second time without following symlinks.
+    bool deref = true;
+    for ( int n = 0; n < 2; ++n, deref = !deref )
+    {
+        const std::string msg(deref ? " failed for the link target"
+                                    : " failed for the path itself");
+
+        if ( !deref )
+        {
+            linktofile.DontFollowLink();
+            linktodir.DontFollowLink();
+            linktofilelnk.DontFollowLink();
+            linktodirlnk.DontFollowLink();
+        }
+
+        // Test SameAs()
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Comparison with file" + msg,
+            deref, linktofile.SameAs(targetfn)
+        );
+
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Comparison with directory" + msg,
+            deref, linktodir.SameAs(tmpfn)
+        );
+
+        // A link-to-a-link should dereference through to the final target
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Comparison with link to a file" + msg,
+            deref,
+            linktofilelnk.SameAs(targetfn)
+        );
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Comparison with link to a directory" + msg,
+            deref,
+            linktodirlnk.SameAs(tmpfn)
+        );
+
+        // Test GetTimes()
+        wxDateTime dtAccess, dtMod, dtCreate;
+        CPPUNIT_ASSERT_MESSAGE
+        (
+            "Getting times of a directory" + msg,
+            linktodir.GetTimes(&dtAccess, &dtMod, &dtCreate)
+        );
+
+        // Test (File|Dir)Exists()
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Testing file existence" + msg,
+            deref,
+            linktofile.FileExists()
+        );
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Testing directory existence" + msg,
+            deref,
+            linktodir.DirExists()
+        );
+
+        // Test wxFileName::Exists
+        // The wxFILE_EXISTS_NO_FOLLOW flag should override DontFollowLink()
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Testing file existence" + msg,
+            false,
+            linktofile.Exists(wxFILE_EXISTS_REGULAR | wxFILE_EXISTS_NO_FOLLOW)
+        );
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Testing directory existence" + msg,
+            false,
+            linktodir.Exists(wxFILE_EXISTS_DIR | wxFILE_EXISTS_NO_FOLLOW)
+        );
+        // and the static versions
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Testing file existence" + msg,
+            false,
+            wxFileName::Exists(linktofile.GetFullPath(), wxFILE_EXISTS_REGULAR | wxFILE_EXISTS_NO_FOLLOW)
+        );
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Testing file existence" + msg,
+            true,
+            wxFileName::Exists(linktofile.GetFullPath(), wxFILE_EXISTS_REGULAR)
+        );
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Testing directory existence" + msg,
+            false,
+            wxFileName::Exists(linktodir.GetFullPath(), wxFILE_EXISTS_DIR | wxFILE_EXISTS_NO_FOLLOW)
+        );
+        CPPUNIT_ASSERT_EQUAL_MESSAGE
+        (
+            "Testing directory existence" + msg,
+            true,
+            wxFileName::Exists(linktodir.GetFullPath(), wxFILE_EXISTS_DIR)
+        );
+    }
+
+    // Finally test Exists() after removing the file.
+    CPPUNIT_ASSERT(wxRemoveFile(targetfn.GetFullPath()));
+
+    // Resolving a file that doesn't exist returns empty
+    CPPUNIT_ASSERT_EQUAL_MESSAGE
+    (
+        "Non-existent file didn't resolve correctly",
+        wxFileName(), targetfn.ResolveLink()
+    );
+
+    // This should succeed, as the symlink still exists and
+    // the default wxFILE_EXISTS_ANY implies wxFILE_EXISTS_NO_FOLLOW
+    CPPUNIT_ASSERT(wxFileName(tempdir, "linktofile").Exists());
+    // So should this one, as wxFILE_EXISTS_SYMLINK does too
+    CPPUNIT_ASSERT(wxFileName(tempdir, "linktofile").
+                                            Exists(wxFILE_EXISTS_SYMLINK));
+    // but not this one, as the now broken symlink is followed
+    CPPUNIT_ASSERT(!wxFileName(tempdir, "linktofile").
+                                            Exists(wxFILE_EXISTS_REGULAR));
+    CPPUNIT_ASSERT(linktofile.Exists());
+
+    // This is also a convenient place to test Rmdir() as we have things to
+    // remove.
+
+    // First, check that removing a symlink to a directory fails.
+    CPPUNIT_ASSERT( !wxFileName::Rmdir(linktodirName) );
+
+    // And recursively removing it only removes the symlink itself, not the
+    // directory.
+    CPPUNIT_ASSERT( wxFileName::Rmdir(linktodirName, wxPATH_RMDIR_RECURSIVE) );
+    CPPUNIT_ASSERT( tmpfn.Exists() );
+
+    // Finally removing the directory itself does remove everything.
+    CPPUNIT_ASSERT(tempdirfn.Rmdir(wxPATH_RMDIR_RECURSIVE));
+    CPPUNIT_ASSERT( !tempdirfn.Exists() );
+}
+
+#endif // __UNIX__
+
+#ifdef __WINDOWS__
+
+namespace
+{
+
+void CreateShortcut(const wxString& pathFile, const wxString& pathLink)
+{
+   wxOleInitializer oleInit;
+
+   HRESULT hr;
+
+   wxCOMPtr<IShellLink> sl;
+   hr = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
+                         IID_IShellLink, (void **)&sl);
+   CPPUNIT_ASSERT( SUCCEEDED(hr) );
+
+   wxCOMPtr<IPersistFile> pf;
+   hr = sl->QueryInterface(IID_IPersistFile, (void **)&pf);
+   CPPUNIT_ASSERT( SUCCEEDED(hr) );
+
+   hr = sl->SetPath(pathFile.t_str());
+   CPPUNIT_ASSERT( SUCCEEDED(hr) );
+
+   hr = pf->Save(pathLink.wc_str(), TRUE);
+   CPPUNIT_ASSERT( SUCCEEDED(hr) );
+}
+
+} // anonymous namespace
+
+void FileNameTestCase::TestShortcuts()
+{
+   wxFileName fn(wxFileName::CreateTempFileName("filenametest"));
+   CPPUNIT_ASSERT( fn.IsOk() );
+   wxON_BLOCK_EXIT1( wxRemoveFile, fn.GetFullPath() );
+
+   wxFileName fnLink(fn.GetPath(), "sc_" + fn.GetName(), "lnk");
+   CPPUNIT_ASSERT( fnLink.IsOk() );
+   wxON_BLOCK_EXIT1( wxRemoveFile, fnLink.GetFullPath() );
+
+   CreateShortcut(fn.GetFullPath(), fnLink.GetFullPath());
+
+   // MakeRelativeTo() is supposed to change only the path of the file, not its
+   // name.
+   wxFileName fnLinkRel(fnLink);
+   fnLink.MakeRelativeTo(".");
+   CPPUNIT_ASSERT_EQUAL(fnLink.GetFullName(), fnLinkRel.GetFullName());
+}
+
+#endif // __WINDOWS__
+
+#ifdef __LINUX__
+
+// Check that GetSize() works correctly for special files.
+TEST_CASE("wxFileName::GetSizeSpecial", "[filename][linux][special-file]")
+{
+    if ( IsRunningInLXC() )
+        return;
+
+    wxULongLong size = wxFileName::GetSize("/proc/kcore");
+    INFO( "size of /proc/kcore=" << size );
+    CHECK( size > 0 );
+
+    // All files in /sys are one page in size, irrespectively of the size of
+    // their actual contents.
+    CHECK( wxFileName::GetSize("/sys/power/state") == sysconf(_SC_PAGESIZE) );
+}
+
+#endif // __LINUX__
