@@ -12,16 +12,15 @@
 
 #include "testprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #include "wx/bitmap.h"
 #include "wx/dcmemory.h"
 #if wxUSE_GRAPHICS_CONTEXT
 #include "wx/dcgraph.h"
 #endif // wxUSE_GRAPHICS_CONTEXT
+#include "wx/dcsvg.h"
 
+#include "testfile.h"
 
 // ----------------------------------------------------------------------------
 // test class
@@ -125,7 +124,7 @@ public:
     {
     }
 
-    virtual void setUp() { wxASSERT( m_dc ); }
+    virtual void setUp() wxOVERRIDE { wxASSERT( m_dc ); }
     virtual wxDC* GetDC(wxMemoryDC* dc) = 0;
 
 protected:
@@ -244,11 +243,7 @@ protected:
 // For GTK+ 3 wxDC is equivalent to wxGCDC
 // and hence doesn't need to be tested individually.
 #if !defined(__WXGTK3__)
-// register in the unnamed registry so that these tests are run by default
-CPPUNIT_TEST_SUITE_REGISTRATION( ClippingBoxTestCaseDC );
-
-// also include in it's own registry so that these tests can be run alone
-CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ClippingBoxTestCaseDC, "ClippingBoxTestCaseDC" );
+wxREGISTER_UNIT_TEST_WITH_TAGS(ClippingBoxTestCaseDC, "[clip][dc]");
 #endif // !__WXGTK3__
 
 #if wxUSE_GRAPHICS_CONTEXT
@@ -342,11 +337,7 @@ protected:
 // For MSW we have individual test cases for each graphics renderer
 // so we don't need to test wxGCDC with default renderer.
 #ifndef __WXMSW__
-// register in the unnamed registry so that these tests are run by default
-CPPUNIT_TEST_SUITE_REGISTRATION( ClippingBoxTestCaseGCDC );
-
-// also include in it's own registry so that these tests can be run alone
-CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ClippingBoxTestCaseGCDC, "ClippingBoxTestCaseGCDC" );
+wxREGISTER_UNIT_TEST_WITH_TAGS(ClippingBoxTestCaseGCDC, "[clip][gcdc]");
 #endif // !__WXMSW__
 
 #ifdef __WXMSW__
@@ -403,11 +394,7 @@ protected:
     wxDECLARE_NO_COPY_CLASS(ClippingBoxTestCaseGCDCGDIPlus);
 };
 
-// register in the unnamed registry so that these tests are run by default
-CPPUNIT_TEST_SUITE_REGISTRATION( ClippingBoxTestCaseGCDCGDIPlus );
-
-// also include in it's own registry so that these tests can be run alone
-CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ClippingBoxTestCaseGCDCGDIPlus, "ClippingBoxTestCaseGCDCGDIPlus" );
+wxREGISTER_UNIT_TEST_WITH_TAGS(ClippingBoxTestCaseGCDCGDIPlus, "[clip][gcdc][gdi+]");
 
 #endif // wxUSE_GRAPHICS_GDIPLUS
 
@@ -424,7 +411,7 @@ public:
 
     virtual ~ClippingBoxTestCaseGCDCDirect2D() {}
 
-    virtual void FlushDC()
+    virtual void FlushDC() wxOVERRIDE
     {
         // Apparently, flushing native Direct2D renderer
         // is not enough to update underlying DC (bitmap)
@@ -472,11 +459,7 @@ protected:
     wxDECLARE_NO_COPY_CLASS(ClippingBoxTestCaseGCDCDirect2D);
 };
 
-// register in the unnamed registry so that these tests are run by default
-CPPUNIT_TEST_SUITE_REGISTRATION( ClippingBoxTestCaseGCDCDirect2D );
-
-// also include in it's own registry so that these tests can be run alone
-CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ClippingBoxTestCaseGCDCDirect2D, "ClippingBoxTestCaseGCDCDirect2D" );
+wxREGISTER_UNIT_TEST_WITH_TAGS(ClippingBoxTestCaseGCDCDirect2D, "[clip][gcdc][d2d]");
 
 #endif // wxUSE_GRAPHICS_DIRECT2D
 
@@ -533,11 +516,7 @@ protected:
     wxDECLARE_NO_COPY_CLASS(ClippingBoxTestCaseGCDCCairo);
 };
 
-// register in the unnamed registry so that these tests are run by default
-CPPUNIT_TEST_SUITE_REGISTRATION( ClippingBoxTestCaseGCDCCairo );
-
-// also include in it's own registry so that these tests can be run alone
-CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ClippingBoxTestCaseGCDCCairo, "ClippingBoxTestCaseGCDCCairo" );
+wxREGISTER_UNIT_TEST_WITH_TAGS(ClippingBoxTestCaseGCDCCairo, "[clip][gc][cairo]");
 
 #endif // wxUSE_CAIRO
 
@@ -1619,11 +1598,7 @@ protected:
     wxDECLARE_NO_COPY_CLASS(ClippingBoxTestCaseGCGDIPlus);
 };
 
-// register in the unnamed registry so that these tests are run by default
-CPPUNIT_TEST_SUITE_REGISTRATION( ClippingBoxTestCaseGCGDIPlus );
-
-// also include in it's own registry so that these tests can be run alone
-CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ClippingBoxTestCaseGCGDIPlus, "ClippingBoxTestCaseGCGDIPlus" );
+wxREGISTER_UNIT_TEST_WITH_TAGS(ClippingBoxTestCaseGCGDIPlus, "[clip][gc][gdi+]");
 
 #endif // wxUSE_GRAPHICS_GDIPLUS
 
@@ -1638,7 +1613,7 @@ public:
 
     virtual ~ClippingBoxTestCaseGCDirect2D() {}
 
-    virtual void FlushGC()
+    virtual void FlushGC() wxOVERRIDE
     {
         // Apparently, flushing native Direct2D renderer
         // is not enough to update underlying DC (bitmap)
@@ -1673,11 +1648,7 @@ protected:
     wxDECLARE_NO_COPY_CLASS(ClippingBoxTestCaseGCDirect2D);
 };
 
-// register in the unnamed registry so that these tests are run by default
-CPPUNIT_TEST_SUITE_REGISTRATION( ClippingBoxTestCaseGCDirect2D );
-
-// also include in it's own registry so that these tests can be run alone
-CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ClippingBoxTestCaseGCDirect2D, "ClippingBoxTestCaseGCDirect2D" );
+wxREGISTER_UNIT_TEST_WITH_TAGS(ClippingBoxTestCaseGCDirect2D, "[clip][gc][d2d]");
 
 #endif // wxUSE_GRAPHICS_DIRECT2D
 
@@ -1718,11 +1689,7 @@ protected:
     wxDECLARE_NO_COPY_CLASS(ClippingBoxTestCaseGCCairo);
 };
 
-// register in the unnamed registry so that these tests are run by default
-CPPUNIT_TEST_SUITE_REGISTRATION( ClippingBoxTestCaseGCCairo );
-
-// also include in it's own registry so that these tests can be run alone
-CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ClippingBoxTestCaseGCCairo, "ClippingBoxTestCaseGCCairo" );
+wxREGISTER_UNIT_TEST_WITH_TAGS(ClippingBoxTestCaseGCCairo, "[clip][gc][cairo]");
 
 #endif // wxUSE_CAIRO
 
@@ -1761,11 +1728,7 @@ protected:
     wxDECLARE_NO_COPY_CLASS(ClippingBoxTestCaseGCCoreGraphics);
 };
 
-// register in the unnamed registry so that these tests are run by default
-CPPUNIT_TEST_SUITE_REGISTRATION( ClippingBoxTestCaseGCCoreGraphics );
-
-// also include in it's own registry so that these tests can be run alone
-CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ClippingBoxTestCaseGCCoreGraphics, "ClippingBoxTestCaseGCCoreGraphics" );
+wxREGISTER_UNIT_TEST_WITH_TAGS(ClippingBoxTestCaseGCCoreGraphics, "[clip][gc][cg]");
 
 #endif // __WXOSX__
 
@@ -2158,3 +2121,29 @@ void ClippingBoxTestCaseGCBase::RegionsAndPushPopState()
 }
 
 #endif // wxUSE_GRAPHICS_CONTEXT
+
+#if wxUSE_SVG
+
+// We can't reuse the existing tests for wxSVGFileDC as we can't check its
+// output, but we can still at least check the behaviour of GetClippingBox().
+TEST_CASE("ClippingBoxTestCaseSVGDC", "[clip][svgdc]")
+{
+    TestFile tf;
+    wxSVGFileDC dc(tf.GetName(), s_dcSize.x, s_dcSize.y);
+
+    wxRect rect;
+    dc.GetClippingBox(rect);
+    CHECK( rect == wxRect(s_dcSize) );
+
+    const wxRect rectClip(10, 20, 80, 75);
+    dc.SetClippingRegion(rectClip);
+
+    dc.GetClippingBox(rect);
+    CHECK( rect == rectClip );
+
+    dc.DestroyClippingRegion();
+    dc.GetClippingBox(rect);
+    CHECK( rect == wxRect(s_dcSize) );
+}
+
+#endif // wxUSE_SVG
